/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: SignerFactory.java 9982 2006-11-01 13:00:31Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applications.signerclient.crypto.jce;

import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.MD2WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.MD4WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.MD5WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.SHA1WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.SHA224WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.SHA256WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.SHA384WithRSAEncryption;
import org.exoplatform.applications.signerclient.crypto.jce.RSASigner.SHA512WithRSAEncryption;

/**
 * @version $Rev: 9982 $ $Date: 2006-11-01 20:00:31 +0700 (Wed, 01 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class SignerFactory {

	private static final String SHA1_WITH_RSA = "SHA1withRSA".intern();
	private static final String SHA224_WITH_RSA = "SHA224withRSA".intern();
	private static final String SHA256_WITH_RSA = "SHA256withRSA".intern();
	private static final String SHA384_WITH_RSA = "SHA384withRSA".intern();
	private static final String SHA512_WITH_RSA = "SHA512withRSA".intern();
	private static final String MD2_WITH_RSA = "MD2withRSA".intern();
	private static final String MD4_WITH_RSA = "MD4withRSA".intern();
	private static final String MD5_WITH_RSA = "MD5withRSA".intern();

	private SignerFactory() {
	    super();
    }

    public static Signer getSigner(byte[] hash, String algorithm) {

		Signer instance = null;

		if (SHA1_WITH_RSA.equals(algorithm))
			instance = new SHA1WithRSAEncryption(hash);
		else if (SHA224_WITH_RSA.equals(algorithm))
			instance = new SHA224WithRSAEncryption(hash);
		else if (SHA256_WITH_RSA.equals(algorithm))
			instance = new SHA256WithRSAEncryption(hash);
		else if (SHA384_WITH_RSA.equals(algorithm))
			instance = new SHA384WithRSAEncryption(hash);
		else if (SHA512_WITH_RSA.equals(algorithm))
			instance = new SHA512WithRSAEncryption(hash);
		else if (MD2_WITH_RSA.equals(algorithm))
			instance = new MD2WithRSAEncryption(hash);
		else if (MD4_WITH_RSA.equals(algorithm))
			instance = new MD4WithRSAEncryption(hash);
		else if (MD5_WITH_RSA.equals(algorithm))
			instance = new MD5WithRSAEncryption(hash);

		return instance;
    }
}
