package org.exoplatform.portlets.content.contentbrowser.component;

import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;

public class UIPathConfig extends UISimpleForm implements SelectorHandler{         
  final public static String NEW_CONFIG_ACTION = "newConfig".intern() ;
  final public static String EDIT_CONFIG_ACTION = "editConfig".intern() ;  
  final public static String SELECT_PATH_ACTION = "selectPath".intern() ;

  private UIStringInput pathInput_ ;  		
  private UISelectBox pathTemplatesBox ;
  private UIStringInput nbPerPage_ ;	
  private UISelectBox detailTemplatesBox_ ;  
  private UIStringInput documentNameInput_ ;  
  private UICheckBox viewToolbarBox_ ;  
  private UICheckBox childDocumentBox_ ;
  private UICheckBox refDocumentBox_ ;
  protected UISelectBox workspaces_ ;
  private UICheckBox viewTagsCloud_ ;
  private UICheckBox allowVoting_ ;
  private UICheckBox allowComment_ ;
  private UICheckBox allowExporting_ ;
  private boolean isPathUsecase_ ;
  private boolean editable_ ;
  private boolean isChangedForm_ ;

  public UIPathConfig() throws Exception {
    super("PathConfig","post",null) ;		
    setId("PathConfig") ;
    setRendererType("SimpleFormVelocityRenderer");		

    PortletPreferences prefs = FacesUtil.getPortletPreferences();                
    String[] workspaceNames = CBUtils.getWorkspaceNames() ;    
    List<SelectItem> workspaceOptions = new ArrayList<SelectItem>() ;    
    for(String workspace:workspaceNames) {
      workspaceOptions.add(new SelectItem(workspace,workspace)) ;
    }   
    workspaces_ = new  UISelectBox(CBUtils.WORKSPACE,prefs.getValue(CBUtils.WORKSPACE,""), workspaceOptions) ;
    add(workspaces_);

    pathInput_ = new UIStringInput(CBUtils.JCR_PATH, prefs.getValue(CBUtils.JCR_PATH,"")) ;
    add(pathInput_) ;    
    documentNameInput_ = new UIStringInput(CBUtils.DOCUMENT_NAME,prefs.getValue(CBUtils.DOCUMENT_NAME,"")) ;
    add(documentNameInput_) ;

    refDocumentBox_ = new UICheckBox(CBUtils.REF_DOCUMENT, prefs.getValue(CBUtils.REF_DOCUMENT,"")) ;
    add(refDocumentBox_) ;
    childDocumentBox_ = new UICheckBox(CBUtils.CHILD_DOCUMENT, prefs.getValue(CBUtils.CHILD_DOCUMENT,"")) ; 
    add(childDocumentBox_) ;


    pathTemplatesBox = new UISelectBox(CBUtils.TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(pathTemplatesBox) ;

    nbPerPage_ = new UIStringInput(CBUtils.NB_PER_PAGE, prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;
    add(nbPerPage_) ;


    detailTemplatesBox_ = new UISelectBox(CBUtils.BOX_TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(detailTemplatesBox_) ;    

    viewToolbarBox_ = new UICheckBox(CBUtils.VIEW_TOOLBAR,prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ; 
    add(viewToolbarBox_) ;

    viewTagsCloud_ = new UICheckBox(CBUtils.VIEW_TAGS_CLOUD,prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
    add(viewTagsCloud_) ;

    allowVoting_ = new UICheckBox(CBUtils.ALLOW_VOTING,prefs.getValue(CBUtils.ALLOW_VOTING,"")) ;
    add(allowVoting_) ;

    allowComment_ = new UICheckBox(CBUtils.ALLOW_COMMENT,prefs.getValue(CBUtils.ALLOW_COMMENT,"")) ;
    add(allowComment_) ;

    allowExporting_ = new UICheckBox(CBUtils.ALLOW_EXPORTING,prefs.getValue(CBUtils.ALLOW_EXPORTING,"")) ;
    add(allowExporting_) ;

    updateSelectBoxes() ;

    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;      
    addActionListener(NewConfigActionListener.class,NEW_CONFIG_ACTION) ;
    addActionListener(EditConfigActionListener.class,EDIT_CONFIG_ACTION) ;
    addActionListener(SelectPathActionListener.class,SELECT_PATH_ACTION) ;
  }

  public boolean isEditable() { return editable_ ;}
  public boolean isPathUsecase() { return isPathUsecase_ ; }

  public String getEditConfigParams() { return EDIT_CONFIG_ACTION ;}
  public String getNewConfigParams() { return NEW_CONFIG_ACTION ; } 
  public String getSelectPathParams() { return SELECT_PATH_ACTION ; }  

  public void updateFormValues() throws Exception {
    PortletPreferences prefs = FacesUtil.getPortletPreferences();
    workspaces_.setValue(prefs.getValue(CBUtils.WORKSPACE,"")) ;
    pathInput_.setValue(prefs.getValue(CBUtils.JCR_PATH,"")) ;         
    pathTemplatesBox.setValue(prefs.getValue(CBUtils.TEMPLATE,"")) ;
    nbPerPage_.setValue(prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;  
    detailTemplatesBox_.setValue(prefs.getValue(CBUtils.BOX_TEMPLATE,"")) ;
    String showToolBar = prefs.getValue(CBUtils.VIEW_TOOLBAR,"") ;
    String showTagsCloud = prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,"") ;
    viewTagsCloud_.setSelect(Boolean.parseBoolean(showTagsCloud)) ;
    viewToolbarBox_.setSelect(Boolean.parseBoolean(showToolBar)) ;
    childDocumentBox_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.CHILD_DOCUMENT,""))) ;
    refDocumentBox_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.REF_DOCUMENT,""))) ;
    allowComment_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_COMMENT,""))) ;
    allowVoting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_VOTING,""))) ;
    allowExporting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_EXPORTING,""))) ;
  }

  public void setEditable(boolean editable) {  
    editable_ = editable ;
    workspaces_.setEditable(editable_) ;
    pathInput_.setEditable(editable_) ;      
    pathTemplatesBox.setEditable(editable_) ;
    nbPerPage_.setEditable(editable_) ;  
    detailTemplatesBox_ .setEditable(editable_);    
    documentNameInput_.setEditable(editable_) ;    
    viewToolbarBox_.setEditable(editable_) ;
    viewTagsCloud_.setEditable(editable_) ;
    childDocumentBox_.setEditable(editable_) ;
    refDocumentBox_.setEditable(editable_) ;
    allowComment_.setEditable(editable_) ;
    allowVoting_.setEditable(editable_) ;
    allowExporting_.setEditable(editable_) ;
  }    

  public void updateSelectBoxes() throws Exception{
    ManageViewService viewService = 
      (ManageViewService)PortalContainer.getComponent(ManageViewService.class) ;
    List<SelectItem> pathTempOptions = new ArrayList<SelectItem>() ;        
    List<Node> pathTemplates = viewService.getAllTemplates(BasePath.CB_PATH_TEMPLATES) ;

    for(Node template:pathTemplates) {
      pathTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }    
    pathTemplatesBox.setOptions(pathTempOptions) ;
    List<SelectItem> docTempOptions = new ArrayList<SelectItem>() ;
    List<Node> docTemplates = viewService.getAllTemplates(BasePath.CB_DETAIL_VIEW_TEMPLATES) ;
    for(Node template: docTemplates) {
      docTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }   
    detailTemplatesBox_.setOptions(docTempOptions) ;
  }
  public void setChangedForm(boolean isChangedForm) {
    isChangedForm_ = isChangedForm ;
  }

  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIPathConfig.class);
  }

  public void select(Selector selector) throws Exception {
    pathInput_.setValue(selector.getInputValue()) ;
    selector.setRenderedSibling(UIPathConfig.class);
  }   

  static public class SelectPathActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPathConfig uicomp = (UIPathConfig) event.getComponent() ;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uicomp) ;
      jcrBrowser.setParameters(new String[] {null, "/"});
      uicomp.setRenderedSibling(JcrBrowser.class);   
    }
  } 

  public static class NewConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPathConfig pathConfig = (UIPathConfig)event.getSource() ;      
      pathConfig.setRenderedSibling(UINewConfigForm.class) ;
    }    
  }      

  public static class EditConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPathConfig pathConfig = (UIPathConfig)event.getSource() ;
      pathConfig.setEditable(true) ;
      pathConfig.updateSelectBoxes() ;
      pathConfig.setChangedForm(false) ;
    }    
  }        

  public static class CancelActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UIPathConfig pathConfig = (UIPathConfig)event.getSource() ;
      if(pathConfig.isChangedForm_) {
        pathConfig.setRenderedSibling(UINewConfigForm.class) ;
      }else {
        pathConfig.setEditable(false) ;
      }
    }    
  }  

  public static class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPathConfig pathConfig = (UIPathConfig)event.getSource() ;  
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;

      String workspace = pathConfig.getUIInput(CBUtils.WORKSPACE).getValue() ;
      String path = pathConfig.pathInput_.getValue() ;
      UICheckBox referenceDocBox = pathConfig.getUICheckBox(CBUtils.REF_DOCUMENT) ;
      UICheckBox childDocBox = pathConfig.getUICheckBox(CBUtils.CHILD_DOCUMENT) ;
      String pathTemplate = pathConfig.getUISelectBox(CBUtils.TEMPLATE).getValue() ;
      String showRefDocument = Boolean.toString(referenceDocBox.getSelect()) ;
      String showChildDocument = Boolean.toString(childDocBox.getSelect()) ;
      String nbPerPage = pathConfig.getUIInput(CBUtils.NB_PER_PAGE).getValue() ;
      String boxTemplate = pathConfig.getUISelectBox(CBUtils.BOX_TEMPLATE).getValue() ;            
      UICheckBox viewToolbarBox = (UICheckBox)pathConfig.getUIInput(CBUtils.VIEW_TOOLBAR) ;           
      String viewToolbar = Boolean.toString(viewToolbarBox.getSelect()) ;      

      UICheckBox viewTagsCloudBox = pathConfig.getUICheckBox(CBUtils.VIEW_TAGS_CLOUD) ;
      String viewTagsCloud = Boolean.toString(viewTagsCloudBox.getSelect()) ;
      UICheckBox allowVotingBox = (UICheckBox)pathConfig.getUIInput(CBUtils.ALLOW_VOTING) ;
      UICheckBox allowCommentBox = (UICheckBox)pathConfig.getUIInput(CBUtils.ALLOW_COMMENT) ;
      String allowVoting = Boolean.toString(allowVotingBox.getSelect()) ;
      String allowComment = Boolean.toString(allowCommentBox.getSelect());

      UICheckBox allowExportingBox = (UICheckBox)pathConfig.getUIInput(CBUtils.ALLOW_EXPORTING) ;
      String allowExporting = Boolean.toString(allowExportingBox.getSelect()) ;

      String usecase = CBUtils.FORM_PATH ;      
      Information iprovider = findInformationProvider(pathConfig);			
      ResourceBundle res = getApplicationResourceBundle() ;
      RepositoryService repositoryService = 
        (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;
      Session session = null ;
      try {                       
        if(path.startsWith("/jcr:system")) {
          session = repositoryService.getRepository().getSystemSession(workspace) ;
          session.getItem(path) ;
        }else {
          session = repositoryService.getRepository().login(workspace) ;          
          ExtendedNode extNode = (ExtendedNode)session.getItem(path) ;
          extNode.checkPermission(PermissionType.READ) ;
        }                         
      }catch (AccessControlException ae) {
        if(session !=null) { session.logout() ;}
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            res.getString("UIContentBrowserConfig.msg.unable-access"),new Object[]{path})) ;
        return ;        
      } catch (Exception e) { 
        if(session !=null) { session.logout() ;}
        JCRExceptionManager.process(iprovider,e) ;
        return ;
      }
      session.logout() ;
      prefs.setValue(CBUtils.WORKSPACE,workspace) ;
      prefs.setValue(CBUtils.JCR_PATH,path) ;            
      prefs.setValue(CBUtils.NB_PER_PAGE,nbPerPage) ;
      prefs.setValue(CBUtils.TEMPLATE,pathTemplate) ;
      prefs.setValue(CBUtils.BOX_TEMPLATE,boxTemplate) ;    
      prefs.setValue(CBUtils.REF_DOCUMENT,showRefDocument) ;
      prefs.setValue(CBUtils.CHILD_DOCUMENT,showChildDocument) ;    
      prefs.setValue(CBUtils.VIEW_TOOLBAR,viewToolbar) ;    
      prefs.setValue(CBUtils.USECASE,usecase) ;      
      prefs.setValue(CBUtils.VIEW_TAGS_CLOUD,viewTagsCloud) ;
      prefs.setValue(CBUtils.ALLOW_VOTING,allowVoting) ;
      prefs.setValue(CBUtils.ALLOW_COMMENT,allowComment) ;
      prefs.setValue(CBUtils.ALLOW_EXPORTING,allowExporting) ;                  
      prefs.setValue(CBUtils.QUERY_LANGUAGE,getValue(prefs,CBUtils.QUERY_LANGUAGE)) ;
      prefs.setValue(CBUtils.QUERY_STATEMENT,getValue(prefs,CBUtils.QUERY_STATEMENT)) ;
      prefs.setValue(CBUtils.IS_NEW_QUERY,getValue(prefs,CBUtils.IS_NEW_QUERY)) ;
      prefs.setValue(CBUtils.IS_PERSONAL_QUERY,getValue(prefs,CBUtils.IS_PERSONAL_QUERY)) ;
      prefs.setValue(CBUtils.QUERY_STORE,getValue(prefs,CBUtils.QUERY_STORE)) ;                                                   
      prefs.setValue(CBUtils.DOCUMENT_NAME,getValue(prefs,CBUtils.DOCUMENT_NAME)) ;
      prefs.setValue(CBUtils.SCRIPT_NAME, getValue(prefs,CBUtils.SCRIPT_NAME)) ;      
      prefs.store() ;             

      iprovider.addMessage(
          new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UIContentBrowserConfig.msg.save-config-successfully"))) ;
      pathConfig.updateFormValues() ;
      pathConfig.setEditable(false) ;

    }

    private String getValue(PortletPreferences pref,String name) {
      String value = pref.getValue(name,"") ;
      if(value == null) value = "" ;
      return value ;
    }
  }
}
