package org.exoplatform.portlets.content.drives.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.Value;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIListStringInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;

public class UIDriveForm extends UISimpleForm implements SelectorHandler {

  final private static String     SELECT_VIEWS_ACTION = "selectView".intern();
  final private static String     VIEW_PREFERENCES    = "viewPreferences".intern();
  final private static String     VIEW_NON_DOCUMENT   = "viewNonDocument".intern();
  final private static String     VIEW_EXPLORER       = "viewExplorer".intern();
  final private static String     VIEW_CLIPBOARD      = "viewClipboard".intern();
  final private static String     FOLDER_TYPE         = "folderType".intern() ;
  final private static String     NAME                = "name".intern();  
  final private static String     PATH                = "path".intern();
  final private static String     VIEWS               = "views".intern();
  final private static String     WORKSPACE           = "workspace".intern();
  final private static String     ICON                = "icon".intern();
  final private static String     EMPTY               = "".intern();
  final public static String      SELECT_PATH_ACTION  = "selectPath".intern();
  final public static String      SELECT_ICON_ACTION  = "selectIcon".intern();
  final public static String      SEARCH_GROUP        = "searchGroup";  
  final public static String      BOTH_TYPES          = "BothTypes".intern() ;
  final public static String      NT_UNSTRUCTURED     = "nt:unstructured".intern() ;
  final public static String      NT_FOLDER           = "nt:folder".intern() ;

  private static String WORKSPACE_PROP = "exo:workspace".intern() ;
  private static String PERMISSIONS_PROP = "exo:permissions".intern() ;
  private static String VIEWS_PROP = "exo:views".intern() ;
  private static String ICON_PROP = "exo:icon".intern() ;
  private static String PATH_PROP = "exo:path".intern() ;
  private static String FOLDER_TYPE_PROP = "exo:folderTypes".intern() ;
  private static String VIEW_REFERENCES_PROP = "exo:viewPreferences".intern() ;
  private static String VIEW_NON_DOCUMENT_PROP = "exo:viewNonDocument".intern() ;
  private static String VIEW_EXPLORER_PROP = "exo:viewExplorer".intern() ;
  private static String VIEW_CLIPBOARD_PROP = "exo:viewClipboard".intern() ;

  private UIExoPermissionSelector userOrGroup_;
  private UIListStringInput permissionList_ ;
  private UIStringInput           path_;
  private UIStringInput           icon_;
  private UISelectBox             workspace_;
  private UISelectBox             folderType_ ;
  private ManageDriveService      dservice_;
  private ManageableRepository    repository_;
  private ManageViewService       vservice_;
  private String                  selectAction_       = SELECT_PATH_ACTION;
  private boolean                 isAddNew_           = false;  

  public UIDriveForm(ManageDriveService dservice, RepositoryService repositoryService,
      ManageViewService vservice) throws Exception {
    super("uiDriveForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    dservice_ = dservice;
    vservice_ = vservice;
    repository_ = repositoryService.getRepository();
    UIStringInput name = new UIStringInput(NAME, EMPTY);
    path_ = new UIStringInput(PATH, EMPTY);
    List<SelectItem> workspaceItems = new ArrayList<SelectItem>();
    String[] wsNames = repository_.getWorkspaceNames();
    for (int i = 0; i < wsNames.length; i++) {
      workspaceItems.add(new SelectItem(wsNames[i].toString(), wsNames[i].toString()));
    }
    workspace_ = new UISelectBox(WORKSPACE, EMPTY, workspaceItems);    
    icon_ = new UIStringInput(ICON, EMPTY);
    UIListStringInput viewList = new UIListStringInput(VIEWS,new ArrayList<String>(1)) ;
    userOrGroup_ = new UIExoPermissionSelector("userOrGroup", null);    
    permissionList_ = new UIListStringInput("permissions", new ArrayList<String>(1)) ;
    UICheckBox viewReferences = new UICheckBox(VIEW_PREFERENCES, EMPTY, false);
    UICheckBox viewNonDocument = new UICheckBox(VIEW_NON_DOCUMENT, EMPTY, false);
    UICheckBox viewExplorer = new UICheckBox(VIEW_EXPLORER, EMPTY, false);
    UICheckBox viewClipboard = new UICheckBox(VIEW_CLIPBOARD, EMPTY, false);
    List<SelectItem> folderTypes = new ArrayList<SelectItem>() ;
    folderTypes.add(new SelectItem(NT_FOLDER, NT_FOLDER)) ;
    folderTypes.add(new SelectItem(NT_UNSTRUCTURED,NT_UNSTRUCTURED)) ;
    folderTypes.add(new SelectItem(BOTH_TYPES,BOTH_TYPES)) ;
    folderType_ = new UISelectBox(FOLDER_TYPE,"",folderTypes) ;
    add(name);
    add(path_);
    add(workspace_);
    add(icon_);
    add(viewList);
    add(permissionList_) ;
    add(viewReferences);
    add(viewNonDocument);
    add(viewExplorer);
    add(viewClipboard);
    add(folderType_) ;

    addActionListener(SelectViewsActionListener.class, SELECT_VIEWS_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP);
    addActionListener(SelectPathActionListener.class, SELECT_PATH_ACTION);
    addActionListener(SelectIconActionListener.class, SELECT_ICON_ACTION);
  }

  public String getSearchGroupAction() { return SEARCH_GROUP; }
  public String getSelectViewsAction() { return SELECT_VIEWS_ACTION; }
  public String getSelectPathParams() { return SELECT_PATH_ACTION; }
  public String getSelectIconParams() { return SELECT_ICON_ACTION; }

  public void refresh(String driveName) throws Exception {
    if (driveName == null) {
      getUIStringInput(NAME).setValue(EMPTY);
      getUIStringInput(NAME).setEditable(true);
      ((UIListStringInput)getUIInput(VIEWS)).getValues().clear();
      getUIStringInput(PATH).setValue(EMPTY);
      getUIStringInput(ICON).setValue(EMPTY);
      List permissions = ((UIListStringInput)getUIInput("permissions")).getValues();
      if(permissions != null && !permissions.isEmpty()) permissions.clear() ;
      getUICheckBox(VIEW_PREFERENCES).setSelect(false);
      getUICheckBox(VIEW_NON_DOCUMENT).setSelect(false);
      getUICheckBox(VIEW_EXPLORER).setSelect(false);
      getUICheckBox(VIEW_CLIPBOARD).setSelect(false);
      isAddNew_ = true;
    } else {
      Node drive = dservice_.getDriveByName(driveName);      
      getUIStringInput(NAME).setValue(drive.getName());
      getUIStringInput(NAME).setEditable(false);       
      List<String> views = new ArrayList<String>() ;
      for(Value v:drive.getProperty(VIEWS_PROP).getValues()) 
        views.add(v.getString()) ;    
      (((UIListStringInput)getUIInput(VIEWS))).setValues(views) ;
      getUISelectBox(WORKSPACE).setValue(drive.getProperty(WORKSPACE_PROP).getString());
      Value[] folderTypes = drive.getProperty(FOLDER_TYPE_PROP).getValues() ;
      if(folderTypes.length>1) {
        getUISelectBox(FOLDER_TYPE).setValue(BOTH_TYPES) ; 
      }else {
        getUISelectBox(FOLDER_TYPE).setValue(folderTypes[0].getString()) ;
      }      
      getUIStringInput(PATH).setValue(drive.getProperty(PATH_PROP).getString());
      getUIStringInput(ICON).setValue(drive.getProperty(ICON_PROP).getString());
      getUICheckBox(VIEW_PREFERENCES).setSelect(drive.getProperty(VIEW_REFERENCES_PROP).getBoolean());
      getUICheckBox(VIEW_NON_DOCUMENT).setSelect(drive.getProperty(VIEW_NON_DOCUMENT_PROP).getBoolean());
      getUICheckBox(VIEW_EXPLORER).setSelect(drive.getProperty(VIEW_EXPLORER_PROP).getBoolean());
      getUICheckBox(VIEW_CLIPBOARD).setSelect(drive.getProperty(VIEW_CLIPBOARD_PROP).getBoolean());
      List<String> permission = new ArrayList<String>() ;      
      for(Value v:drive.getProperty(PERMISSIONS_PROP).getValues()) 
        permission.add(v.getString()) ;
      ((UIListStringInput)getUIInput("permissions")).setValues(permission) ;
      isAddNew_ = false;
    }    
  }

  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIDriveForm.class);
  }

  public void select(Selector selector) throws Exception {
    if (selectAction_.equals(SELECT_PATH_ACTION)) {
      path_.setValue(selector.getInputValue());
    } else if (selectAction_.equals(SEARCH_GROUP)) {
      UIExoPermissionBrowser browser = (UIExoPermissionBrowser)selector ;
      if(browser.isGroup()) {
        permissionList_.getValues().add(selector.getInputValue()) ;        
      }else {
        permissionList_.getValues().add(browser.getSelectedUser()) ;        
      }      
    } else {
      icon_.setValue(selector.getInputValue());
    }
    selector.setRenderedSibling(UIDriveForm.class);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uiForm = (UIDriveForm) event.getSource();
      ResourceBundle res = getApplicationResourceBundle();
      Information iprovider = findInformationProvider(uiForm);
      String name = uiForm.getUIStringInput(NAME).getValue();
      if (name == null || name.length() < 1) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIDriveForm.msg.drive-name-invalid")));
        return;
      }
      String workspace = uiForm.getUISelectBox(WORKSPACE).getValue();
      if (workspace == null || workspace.length() < 1) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIDriveForm.msg.drive-workspace-invalid")));
        return;
      }
      String path = uiForm.getUIStringInput(PATH).getValue();
      if (path == null || path.length() == 0)
        path = "/";
      Session session = uiForm.repository_.login(workspace);
      try {        
        session.getItem(path);
        session.logout() ;
      } catch (Exception e) {
        session.logout() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            res.getString("UIDriveForm.msg.workspace-path-invalid")));
        return;        
      }
      List<String> views = ((UIListStringInput)uiForm.getUIInput(VIEWS)).getValues();
      if (views.isEmpty()) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIDriveForm.msg.drive-views-invalid")));
        return;
      }            
      for (String view:views) {
        if (!uiForm.vservice_.hasView(view.trim())) {
          Object[] arg = { view.trim() };
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIDriveForm.msg.view-not-exist"), arg));
          return;
        }
      }      

      String icon = uiForm.getUIStringInput(ICON).getValue();
      List<String> permissions = uiForm.permissionList_.getValues() ;
      if(permissions.isEmpty()) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIDriveForm.msg.permissions-not-empty")));
        return;
      }        
      boolean viewReferences = uiForm.getUICheckBox(VIEW_PREFERENCES).getSelect();
      boolean viewExplorer = uiForm.getUICheckBox(VIEW_EXPLORER).getSelect();
      boolean viewNonDocument = uiForm.getUICheckBox(VIEW_NON_DOCUMENT).getSelect();
      boolean viewClipboard = uiForm.getUICheckBox(VIEW_CLIPBOARD).getSelect();
      String folderType = uiForm.getUISelectBox(FOLDER_TYPE).getValue() ;
      String folderTypes[] = null ;
      if(BOTH_TYPES.equals(folderType))
        folderTypes = new String[]{NT_FOLDER,NT_UNSTRUCTURED} ;
      else
        folderTypes = new String[]{folderType} ;
      String[] viewsArray = views.<String>toArray(new String[views.size()]) ;
      String[] permissionsArray = permissions.<String>toArray(new String[views.size()]) ;
      try {                                                                                 
        uiForm.dservice_.addDrive(name, workspace, permissionsArray, path, viewsArray, icon,
            viewReferences, viewNonDocument, viewExplorer, viewClipboard,folderTypes);                  
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider,e) ;
        return ;
      }      
      UIDriveList driveList = (UIDriveList) uiForm.getSibling(UIDriveList.class);
      driveList.refresh();
      uiForm.setRenderedSibling(UIDriveList.class);      
    }

  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm viewForm = (UIDriveForm) event.getSource();
      viewForm.setRenderedSibling(UIDriveList.class);
    }
  }

  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uiForm = (UIDriveForm) event.getSource();
      uiForm.selectAction_ = SEARCH_GROUP;
      UIExoPermissionBrowser uiGroupBrowser = (UIExoPermissionBrowser) uiForm
      .getSibling(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);
    }
  }

  static public class SelectPathActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uicomp = (UIDriveForm) event.getComponent();
      uicomp.selectAction_ = SELECT_PATH_ACTION;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setFilter(null);
      jcrBrowser.setHandler(uicomp);
      jcrBrowser.setParameters(new String[] { uicomp.workspace_.getValue() , "/" });
      uicomp.setRenderedSibling(JcrBrowser.class);
    }
  }

  static public class SelectIconActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uicomp = (UIDriveForm) event.getComponent();
      uicomp.selectAction_ = SELECT_ICON_ACTION ;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uicomp);
      List<String> filter = new ArrayList<String>() ;
      filter.add("nt:file") ;
      jcrBrowser.setFilter(filter);
      jcrBrowser.setParameters(new String[] { "digital-assets", "/" });
      uicomp.setRenderedSibling(JcrBrowser.class);
    }
  }

  static public class SelectViewsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIDriveForm uiForm = (UIDriveForm) event.getSource();        
      List views = ((UIListStringInput)uiForm.getUIInput(VIEWS)).getValues();        
      UIViewList viewList = (UIViewList) uiForm.getSibling(UIViewList.class);
      viewList.update(views);
      uiForm.setRenderedSibling(UIViewList.class);      
    }
  }
}
