/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.folksonomy.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.jcr.Node;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.services.cms.folksonomy.FolksonomyService;

/**
 * Created by The eXo Platform SAS
 * Author : Xuan Hoa Pham
 *          hoapham@exoplatform.com
 * 					phamvuxuanhoa@gmail.com
 * Dec 11, 2006  
 */
public class UILinkedTags extends UIExoCommand {
  
     
  final private static String TAG_STATUS_PROP = "exo:tagStatus".intern() ;  
  
  public static final String ASCENDING_ORDER = "Ascending".intern();
     
  private FolksonomyService folkService_ ;
  private List<Node> linkedTag_ = new ArrayList<Node>() ;  
  private Node document_ ;
  
  public UILinkedTags(FolksonomyService folkService) throws Exception {
    setId("UITagCloud") ;
    setRendererType("VelocityRenderer") ;
    folkService_ = folkService ;  
    updateLinkedTags() ;        
  }
      
  public List<Node> getLinkedTags() { return linkedTag_ ; }
  public void setLinkedTags(List<Node> tagsList) throws Exception { 
    this.linkedTag_ = sortTags(tagsList) ;
  }
      
  public void setDocument(Node node) throws Exception {
    document_ = node ;
    updateLinkedTags() ;
  }
  
  public void updateLinkedTags() throws Exception{
    List<Node> linkedTags = folkService_.getLinkedTagsOfDocument(document_) ;
    setLinkedTags(linkedTags) ;
  }
  
  public String getHtmlStyleOfTag(Node tagNode) throws Exception {
    String tagStyleName = tagNode.getProperty(TAG_STATUS_PROP).getValue().getString() ;
    return folkService_.getTagStyle(tagStyleName) ;
  }
  
  public List<Node> getDocumentOnTag(String tagPath) throws Exception {
    return folkService_.getDocumentsOnTag(tagPath) ;
  }
  
  public Node getSelectedTag(String tagPath) throws Exception {
    return folkService_.getTag(tagPath) ;    
  }
   
  private List<Node> sortTags(List<Node> tagsList) throws Exception {
    Map<String,Node> sortedNodes = new TreeMap(new AlphaNodeComparator(ASCENDING_ORDER)) ;    
    List<Node> sortedListTags = new ArrayList<Node>() ;
    int j = 0 ;
    for(Node tag :tagsList) {        
      String key = 
        new StringBuffer(tag.getName()).append(j).toString().toLowerCase();
      sortedNodes.put(key, tag);
      j++;
    }               
    Collection<Node> collection = sortedNodes.values() ;
    for(Node tag:collection) {
      sortedListTags.add(tag) ;
    }
    return sortedListTags ;
  }          
}
