/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.cms.i18n.MultiLanguageService;

/**
 * @email: nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIMultiLanguageForm extends UISimpleMulltipartForm implements ChangeNodeListener{

  final static public String      ADD_LANGUAGE_ACTION = "addLanguage";
  final static public String      CHANGE_TYPE_ACTION     = "changeType";
  
  private UISelectBox languageSelect_ ;
  private MultiLanguageService multiLangService_ ;

  public UIMultiLanguageForm(MultiLanguageService multiLangService) throws Exception {
    super("uiMultiLanguageForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    
    multiLangService_ = multiLangService ;
    languageSelect_ = new UISelectBox("language", "", null);
    add(languageSelect_);

    addActionListener(AddLanguageActionListener.class, ADD_LANGUAGE_ACTION);
    addActionListener(ViewActionListener.class, VIEW_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public String getAddParams() { return ADD_LANGUAGE_ACTION ; }
  public String getViewParams() { return VIEW_ACTION ; }
  
  public void update(Node currentNode) throws Exception{
    List<SelectItem> la_options = new ArrayList<SelectItem>();
    List<String> supportedLang = multiLangService_.getSupportedLanguages(currentNode) ;
    boolean hasDefault = false ;
    for(String lang : supportedLang) {
      if(!hasDefault) {
        la_options.add(new SelectItem(lang + "(default)", lang)) ;
        languageSelect_.setValue(lang) ;
        hasDefault = true ;
      }else {
        la_options.add(new SelectItem(lang, lang));
      }      
    }
    languageSelect_.setOptions(la_options) ;    
  }
  
  public boolean isCheckedOut() throws RepositoryException {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    return node.isCheckedOut();
  }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ; 
   }
  
  static public class AddLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMultiLanguageForm uicomp = (UIMultiLanguageForm) event.getSource();
      UIAddLanguageForm addLanguage = (UIAddLanguageForm)uicomp.getSibling(UIAddLanguageForm.class) ;
      addLanguage.update() ;
      uicomp.setRenderedSibling(UIAddLanguageForm.class) ;
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIMultiLanguageForm uiForm = (UIMultiLanguageForm) event.getSource();
        UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
        Node node = uiExplorer.getSelectNode();
        String selectedLang = uiForm.languageSelect_.getValue() ;
        uiForm.multiLangService_.setDefault(node, selectedLang) ;
        uiForm.setRenderedSibling(UIContentManager.class) ;
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  } 
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMultiLanguageForm uicomp = (UIMultiLanguageForm) event.getSource();
      uicomp.setRenderedSibling(UIContentManager.class) ;
    }
  }
  
  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMultiLanguageForm uicomp = (UIMultiLanguageForm) event.getSource();
      UIContentManager documentInfo = (UIContentManager)uicomp.getSibling(UIContentManager.class) ;
      documentInfo.setLanguage(uicomp.languageSelect_.getValue());
      uicomp.setRenderedSibling(UIContentManager.class) ;
    }   
  }
}
