/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.queries.QueryService;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id$
 */
public class UIJCRAdvancedSearch extends UISimpleForm {

  protected UITextArea queryInput_;
  protected UISelectBox selectInput_;
  protected boolean isSQL_ = true;
  private Node node_;
  protected UIStringInput inputName_;
  protected QueryService queryService_;

  public static final String SEARCH_ACTION = "searchAction";
  public static final String CHANGE_QUERY_ACTION = "changeQueryAction";

  private static final String ROOT_SQL_QUERY = "select * from nt:base";
  private static final String SQL_QUERY = "select * from nt:base where jcr:path like '$0/%'";
  private static final String ROOT_XPATH_QUERY = "//*";
  private static final String XPATH_QUERY = "/jcr:root$0//*";

  public UIJCRAdvancedSearch(QueryService queryService) throws Exception {
    super("UIJCRAdvancedSearch", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    queryService_ = queryService;    
    inputName_ = new UIStringInput("name", "");
    add(inputName_);

    List<SelectItem> options = new ArrayList<SelectItem>();
    options.add(new SelectItem("sql", "sql"));
    options.add(new SelectItem("xpath", "xpath"));
    selectInput_ = new UISelectBox("select", "sql", options);
    selectInput_.setUpdateOnChangeAction(CHANGE_QUERY_ACTION);
    add(selectInput_);

    queryInput_ = new UITextArea("query", ROOT_SQL_QUERY);
    add(queryInput_);

    addActionListener(ChangeQueryActionListener.class, CHANGE_QUERY_ACTION);
    addActionListener(SearchActionListener.class, SEARCH_ACTION);
    addActionListener(SaveQueryActionListener.class, SAVE_ACTION);
  }

  public String getSaveAction() { return SAVE_ACTION; }      
  public String getSearchAction() { return SEARCH_ACTION; }

  public void updateQueryField(Node node) throws Exception {
    node_ = node;
    updateQueryField();
  }

  public void updateQueryField() throws Exception {
    String queryText = null;
    if (isSQL_) {
      if ("/".equals(node_.getPath()))
        queryText = ROOT_SQL_QUERY;
      else
        queryText = StringUtils.replace(SQL_QUERY, "$0", node_.getPath());
    } else {
      if ("/".equals(node_.getPath()))
        queryText = ROOT_XPATH_QUERY;
      else
        queryText = StringUtils.replace(XPATH_QUERY, "$0", node_.getPath());
    }
    queryInput_.setText(queryText);
  }

  static public class ChangeQueryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRAdvancedSearch uiForm = (UIJCRAdvancedSearch) event.getSource();
      if ("sql".equals(uiForm.selectInput_.getValue()))
        uiForm.isSQL_ = true;
      else
        uiForm.isSQL_ = false;
      uiForm.updateQueryField();
    }
  }

  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRAdvancedSearch uiForm = (UIJCRAdvancedSearch) event.getSource();
      String queryS = uiForm.queryInput_.getText();

      UIJCRExplorer explorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Session session = explorer.getSession();           
      QueryManager queryManager = session.getWorkspace().getQueryManager();
      Information iprovider = findInformationProvider(uiForm);
      Query query = null;
      try {
        if (uiForm.isSQL_)
          query = queryManager.createQuery(queryS, Query.SQL);
        else
          query = queryManager.createQuery(queryS, Query.XPATH);
        UIResults results = (UIResults) uiForm.getSibling(UIResults.class);        
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = query.execute();
        long duration = System.currentTimeMillis() - beforeTime;                        
        results.resultMap_.clear() ;
        results.setQueryResults(duration, queryResult);
        uiForm.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  } 

  static public class SaveQueryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRAdvancedSearch uiForm = (UIJCRAdvancedSearch) event.getSource();
      String statement = uiForm.queryInput_.getText();
      String language= uiForm.selectInput_.getValue();
      String name = uiForm.inputName_.getValue();
      Information iprovider = findInformationProvider(uiForm);
      try {
        uiForm.queryService_.addQuery(name, statement, language);
        iprovider.addMessage(
            new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UIJCRAdvancedSearch.msg.query-added"))) ;
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }        
    }
  }
}