/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.metadata.MetadataService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;

/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04 
 */
public class UISelectMetadataForm extends UISimpleMulltipartForm  implements ChangeNodeListener{

  final static public String CHANGE_METADATA_ACTION = "changeMetadata" ;
  private String  metadata_ = "dc:elementSet" ;

  private RepositoryService repoService_ ;

  private UISelectBox selectMetadataType ;
  private List<String> propertyList_ = new ArrayList<String>() ;
  private List selected_ = new ArrayList() ;
  private List<SelectItem> options_ ;
  public UISelectMetadataForm(RepositoryService repositoryService, TemplateService tservice,
      MetadataService metaService) throws Exception {
    super("uiMetadataForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    this.repoService_ = repositoryService ;

    selectMetadataType = new  UISelectBox("metadataType", "", null) ;
    selectMetadataType.setUpdateOnChangeAction(CHANGE_METADATA_ACTION) ;    
    options_ = new ArrayList<SelectItem>();
    for(String metaType:metaService.getAllMetadataTypes()) {
      options_.add(new SelectItem(metaType,metaType)) ;
    }
    List documentTemplates = tservice.getDocumentTemplates() ;
    for(int i = 0; i < documentTemplates.size(); i ++) {
      String nodeTypeName = documentTemplates.get(i).toString() ;
      options_.add(new SelectItem(nodeTypeName, nodeTypeName)) ;
    }
    addActionListener(ChangeMetadataActionListener.class, CHANGE_METADATA_ACTION) ;
    addActionListener(SaveMetadataActionListener.class, SAVE_ACTION);
    addActionListener(CancelMetadataActionListener.class, CANCEL_ACTION);
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ;
  }

  public void update(List selected) throws Exception {
    getChildren().clear() ;
    selectMetadataType.setOptions(options_) ;
    selectMetadataType.setValue(metadata_);
    add(selectMetadataType) ;
    UICheckBox checkbox ;
    NodeType nt = repoService_.getRepository().getNodeTypeManager().getNodeType(metadata_) ;
    propertyList_.clear() ;
    PropertyDefinition[] properties = nt.getPropertyDefinitions() ;
    for(int i = 0 ; i < properties.length ; i ++) {
      String name = properties[i].getName() ;
      propertyList_.add(name) ;
      checkbox = new UICheckBox(name, "") ;
      if(selected.contains(name)) checkbox.setSelect(true) ;
      add(checkbox) ;
    }
    selected_ = selected ;
  }

  public List getProperties() {  return propertyList_ ; }

  static public class ChangeMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uicomp = (UISelectMetadataForm) event.getSource() ;
      uicomp.metadata_ = uicomp.selectMetadataType.getValue() ;
      uicomp.update(uicomp.selected_) ;
    }
  }

  static public class SaveMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uiForm = (UISelectMetadataForm) event.getSource();
      List<String> selected = new ArrayList<String>() ;
      for(int i = 0 ; i < uiForm.propertyList_.size() ; i ++) {
        if(uiForm.getUICheckBox(uiForm.propertyList_.get(i)).getSelect()) {
          selected.add(uiForm.propertyList_.get(i)) ;
        }
      }
      UIAdvanceConstraints advanceSearch = (UIAdvanceConstraints)uiForm.getSibling(UIAdvanceConstraints.class) ;
      advanceSearch.updateMetadata(selected) ;
      uiForm.setRenderedSibling(UIAdvanceConstraints.class) ;
    }
  }

  static public class CancelMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uiForm = (UISelectMetadataForm) event.getSource();
      uiForm.setRenderedSibling(UIAdvanceConstraints.class) ;
    }
  }
}