package org.exoplatform.portlets.content.views.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.NameValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.views.ManageViewService;

public class UITemplateForm extends UISimpleForm {
  private static String NAME = "name".intern(), CONTENT = "content".intern(),HOME_TEMPLATE = "homeTemplate".intern() ;
  private static String TEMPLAT_FILE_PROP = "exo:templateFile".intern() ;

  private ManageViewService viewService_ ;
  private UIStringInput nameInput_ ;
  protected UITextArea contentInput_;
  private UISelectBox typeSelectBox_ ;
  private boolean isECMTemplate_ = true ;

  public UITemplateForm(ManageViewService vservice) throws Exception{    
    super("UITemplateForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    viewService_ = vservice ;
    nameInput_ = new UIStringInput(NAME,"") ;
    nameInput_.addValidator(NameValidator.class) ;
    add(nameInput_) ;

    contentInput_ = new UITextArea(CONTENT,"") ;
    contentInput_.setClazz("large") ;    
    add(contentInput_) ;

    List<SelectItem> typeList = new ArrayList<SelectItem>() ;
    typeSelectBox_ = new UISelectBox(HOME_TEMPLATE,"",typeList) ;
    add(typeSelectBox_) ;

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);

  } 

  public void update(boolean isECMTemplate, String templatePath) throws Exception {
    isECMTemplate_ = isECMTemplate ;
    List<SelectItem> typeList = new ArrayList<SelectItem>() ;
    if(isECMTemplate) {              
      Node ecmTemplateHome = viewService_.getTemplateHome(BasePath.ECM_EXPLORER_TEMPLATES) ; 
      typeList.add(new SelectItem(ecmTemplateHome.getName(),ecmTemplateHome.getPath())) ;
      typeSelectBox_.setOptions(typeList) ;
    }else {        
      Node cbTemplateHome = viewService_.getTemplateHome(BasePath.CONTENT_BROWSER_TEMPLATES) ;
      NodeIterator iter = cbTemplateHome.getNodes() ;
      while(iter.hasNext()) {
        Node template = iter.nextNode() ;
        typeList.add(new SelectItem(template.getName(),template.getPath())) ;
      }
      typeSelectBox_.setOptions(typeList) ;      
    }
    
    if(templatePath != null) {
      Node template = viewService_.getTemplate(templatePath) ;
      nameInput_.setText(template.getName()) ;
      nameInput_.setEditable(false) ;
      contentInput_.setText(template.getProperty(TEMPLAT_FILE_PROP).getString()) ;
      typeSelectBox_.setValue(templatePath.substring(0, templatePath.lastIndexOf("/"))) ;
      typeSelectBox_.setReadonly(true) ;
    }else {
      nameInput_.setText("") ;
      nameInput_.setEditable(true) ;
      contentInput_.setText("") ;
      typeSelectBox_.setReadonly(false) ;
    }
  }   

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplateForm uiForm = (UITemplateForm)event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm) ;
      UIECMTemplateList ecmTemplateList = (UIECMTemplateList)uiForm.getSibling(UIECMTemplateList.class) ;
      UICBTemplateList cbTemplateList = (UICBTemplateList)uiForm.getSibling(UICBTemplateList.class) ;
      
      String templateName = uiForm.getUIStringInput(NAME).getValue() ;
      String content = uiForm.contentInput_.getValue();
      String homeTemplate = uiForm.typeSelectBox_.getValue() ;    
      try {
        uiForm.viewService_.addTemplate(templateName, content, homeTemplate) ;
      }catch(Exception e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UIViewTemplateForm.msg.save-template-error"))) ;
        return ;
      }            
      if(uiForm.isECMTemplate_) {                
        ecmTemplateList.refresh();
        uiForm.setRenderedSibling(UIECMTemplateList.class) ; 
      }else {
        cbTemplateList.refresh();
        uiForm.setRenderedSibling(UICBTemplateList.class) ;
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplateForm uiForm = (UITemplateForm)event.getSource() ;
      if(uiForm.isECMTemplate_) {
        uiForm.setRenderedSibling(UIECMTemplateList.class) ; 
      }else {
        uiForm.setRenderedSibling(UICBTemplateList.class) ;
      }
    }
  }
}
