/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.workflow.component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.Repository;
import javax.jcr.Session;

import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.cms.CMSViewComponent;
import org.exoplatform.faces.cms.InputInfo;
import org.exoplatform.faces.core.component.UICalendarInput;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIDateInput;
import org.exoplatform.faces.core.component.UIDateTimeInput;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UIRadioBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.component.UIWYSIWYG;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.workflow.Form;
import org.exoplatform.services.workflow.Process;
import org.exoplatform.services.workflow.Task;
import org.exoplatform.services.workflow.WorkflowFormsService;
import org.exoplatform.services.workflow.WorkflowServiceContainer;

/**
 * Created y the eXo platform team User: Benjamin Mestrallet Date: 17 mai 2004
 */
public class UITask extends UISimpleMulltipartForm implements CMSViewComponent {

  public static final String PREFIX = "prefix";
  public static final String START_PROCESS = PREFIX + "startProcess";
  public static final String END_OF_STATE = PREFIX + "endOfState";
  public static final String MANAGE_TRANSITION = "manageTransition";
  private static final String TEXT = "text";
  private static final String TEXTAREA = "textarea";
  private static final String WYSIWYG = "wysiwyg";
  private static final String DATE = "date";
  private static final String DATE_TIME = "datetime";
  private static final String CALENDAR = "calendar";
  private static final String CALENDAR_TIME = "calendartime";
  private static final String SELECT = "select";
  private static final String UPLOAD = "upload";
  private static final String CHECK_BOX = "checkbox";
  private static final String RADIO_BOX = "radiobox";
  private static final String NODE_TYPE = "nodetype";
  private static final String NODE_VIEW = "nodeview";
  private static final String NODE_EDIT = "nodeedit";
  private static final String LABEL_ENCODING = ".label";
  private static final String NODE_PATH_VARIABLE = "nodePath";
  private static final String WORKSPACE_VARIABLE = "srcWorkspace";
  final static public String CHANGE_LANGUAGE_ACTION = "changeLanguage" ;
  
  public static Parameter[] CHANGE_LANGUAGE_ACTION_PARAMS   = { new Parameter(ACTION,CHANGE_LANGUAGE_ACTION) } ;
  
  private WorkflowFormsService formsService;
  private Form form;
  private boolean isStart;
  private String identification;
  private WorkflowServiceContainer workflowServiceContainer;
  private TemplateService dialogService;
  private RepositoryService jcrService;
  private List inputInfo;
  private boolean isView;
  private boolean isCreatedOrUpdated;
  private CmsService cmsService;
  private CmsConfigurationService cmsConfigService;
  private String dialogPath_;
  private CMSDelegate delegate;
  private String language_ = "default" ;

  public UITask(WorkflowServiceContainer workflowServiceContainer,
	  WorkflowFormsService workflowFormsService, CmsService cmsService,
	  CmsConfigurationService cmsConfigService, TemplateService dialogService,
	  RepositoryService jcrService) throws Exception {
	super("taskForm", "post", null);
	setRendererType("SimpleFormVelocityRenderer");

	this.workflowServiceContainer = workflowServiceContainer;
	this.formsService = workflowFormsService;
	this.dialogService = dialogService;
	this.jcrService = jcrService;
	this.cmsService = cmsService;
	this.cmsConfigService = cmsConfigService;

	inputInfo = new ArrayList();
	delegate = new CMSDelegate(null);

	addActionListener(StartListener.class, START_PROCESS);
	addActionListener(EndListener.class, END_OF_STATE);
	addActionListener(TransitionListener.class, MANAGE_TRANSITION);
	addActionListener(CancelListener.class, CANCEL_ACTION);
  addActionListener(ChangeLanguageActionListener.class,CHANGE_LANGUAGE_ACTION) ;
  }
  
  public Parameter[] getChangeLanguageParams() {return CHANGE_LANGUAGE_ACTION_PARAMS;  }
  
  public String getStartProcess() {
	return START_PROCESS;
  }

  public String getEndOfState() {
	return END_OF_STATE;
  }

  public String getManageTransition() {
	return MANAGE_TRANSITION;
  }

  public String getCancelProcess() {
	return CANCEL_ACTION;
  }

  public String getStateImageURL() {
	try {
	  Locale locale = ((PortalResources) SessionContainer
		  .getComponent(PortalResources.class)).getLocale();
	  if (isStart) {
		Process process = workflowServiceContainer.getProcess(identification);
		form = formsService.getForm(identification,
			process.getStartStateName(), locale);
	  } else {
		Task task = workflowServiceContainer.getTask(identification);
		form = formsService.getForm(task.getProcessId(), task.getTaskName(),
			locale);
	  }
	  return form.getStateImageURL();
	} catch (Exception e) {
	  return "";
	}
  }

  public void updateUITree() throws Exception {
	Locale locale = ((PortalResources) SessionContainer
		.getComponent(PortalResources.class)).getLocale();
	Map variablesForService = new HashMap();
	if (isStart) {
	  Process process = workflowServiceContainer.getProcess(identification);
	  form = formsService.getForm(identification, process.getStartStateName(),
		  locale);
	} else {
	  Task task = workflowServiceContainer.getTask(identification);
	  String processInstanceId = task.getProcessInstanceId();
	  variablesForService = workflowServiceContainer.getVariables(
		  processInstanceId, identification);
	  form = formsService.getForm(task.getProcessId(), task.getTaskName(),
		  locale);
	}
	String workspaceName = (String) variablesForService.get(WORKSPACE_VARIABLE);
	List variables = form.getVariables();
	UIInput input = null;
	int i = 0;
	for (Iterator iter = variables.iterator(); iter.hasNext(); i++) {
	  Map attributes = (Map) iter.next();
	  String name = (String) attributes.get("name");
	  String component = (String) attributes.get("component");
	  boolean editable = true;
	  String editableString = (String) attributes.get("editable");
	  if (editableString != null && !"".equals(editableString))
		editable = new Boolean(editableString).booleanValue();
	  boolean mandatory = false;
	  String mandatoryString = (String) attributes.get("mandatory");
	  if (mandatoryString != null && !"".equals(mandatoryString))
		mandatory = new Boolean(mandatoryString).booleanValue();
	  Object value = variablesForService.get(name);
	  if (NODE_TYPE.equals(component)) {
		dialogPath_ = dialogService.getTemplatePath(true, (String) value);
		isCreatedOrUpdated = true;
	  } else if (NODE_EDIT.equals(component)) {
		Repository repository = jcrService.getRepository();
		String nodePath = (String) variablesForService.get(NODE_PATH_VARIABLE);
    Session session = repository.login(workspaceName) ;    
    Node viewNode = (Node) session.getItem(nodePath);
		delegate.setNode(viewNode);
		String nodetype = viewNode.getPrimaryNodeType().getName();
		dialogPath_ = dialogService.getTemplatePath(true, nodetype);
		isCreatedOrUpdated = true;
    session.logout() ;
	  } else if (NODE_VIEW.equals(component)) {
		Repository repository = jcrService.getRepository();
		String nodePath = (String) variablesForService.get(NODE_PATH_VARIABLE);
    Session session = repository.login(workspaceName) ;    
		Node viewNode = (Node) session.getItem(nodePath);
		delegate.setNode(viewNode);
		isView = true;
    session.logout() ;
	  } else {
		if (component == null || TEXT.equals(component)) {
		  input = new UIStringInput(name, (String) value);
		} else if (TEXTAREA.equals(component)) {
		  input = new UITextArea(name, (String) value);
		} else if (WYSIWYG.equals(component)) {
		  input = new UIWYSIWYG(name, (String) value);
		} else if (DATE.equals(component)) {
		  if (value == null) {
			input = new UIDateInput(name, new Date());
		  } else {
			input = new UIDateInput(name, (Date) value);
		  }
		} else if (DATE_TIME.equals(component)) {
		  if (value == null) {
			input = new UIDateTimeInput(name, new Date());
		  } else {
			input = new UIDateTimeInput(name, (Date) value);
		  }
    } else if (CALENDAR.equals(component) || CALENDAR_TIME.equals(component)) {
      UICalendarInput uicalendar = new UICalendarInput(name, "");
      if (CALENDAR_TIME.equals(component)) {
        uicalendar.setDisplayTime(true);
      }
      if (value != null) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime((Date)value);
        uicalendar.setValue(ISO8601.format(calendar));
      }
      input = uicalendar;
		} else if (SELECT.equals(component)) {
		  String baseKey = name + ".select-";
		  List options = new ArrayList();
		  int j = 0;
		  String select0 = (String) variablesForService.get(baseKey + j);
		  if (select0 == null) {
			ResourceBundle bundle = form.getResourceBundle();
			try {
			  while (true) {
				String property = bundle.getString(baseKey + j);
				options.add(new SelectItem(property, property));
				j++;
			  }
			} catch (MissingResourceException e) {
			}
		  } else {
			while (true) {
			  String property = (String) variablesForService.get(baseKey + j);
			  if (property == null)
				break;
			  options.add(new SelectItem(property, property));
			  j++;
			}
		  }
		  input = new UISelectBox(name, (String) value, options);
		} else if (CHECK_BOX.equals(component)) {
		  ResourceBundle bundle = form.getResourceBundle();
		  String key = name + ".checkbox";
		  input = new UICheckBox(name, bundle.getString(key), Boolean.valueOf(
			  (String) value).booleanValue());
		} else if (UPLOAD.equals(component)) {
		  input = new UIUploadInput(name);
		} else if (RADIO_BOX.equals(component)) {
		  String baseKey = name + ".radiobox-";
		  List options = new ArrayList();
		  int j = 0;
		  String select0 = (String) variablesForService.get(baseKey + j);
		  if (select0 == null) {
			ResourceBundle bundle = form.getResourceBundle();
			try {
			  while (true) {
				String property = bundle.getString(baseKey + j);
				options.add(new SelectItem(property, property));
				j++;
			  }
			} catch (MissingResourceException e) {
			}
		  } else {
			while (true) {
			  String property = (String) variablesForService.get(baseKey + j);
			  if (property == null)
				break;
			  options.add(new SelectItem(property, property));
			  j++;
			}
		  }
		  input = new UIRadioBox(name, (String) value, options);
		}
		input.setEditable(editable);
		ResourceBundle res = form.getResourceBundle();
		inputInfo.add(new InputInfo("", "", res
			.getString(name + LABEL_ENCODING), input, mandatory));
		add(input);
	  }
	}
  }

  public void setIsStart(boolean b) {
	isStart = b;
  }

  public boolean isStart() {
	return isStart;
  }

  public boolean isView() {
	return isView;
  }

  public boolean isCreatedOrUpdated() {
	return isCreatedOrUpdated;
  }

  public String getDialogPath() {
	return dialogPath_;
  }

  public ResourceBundle getWorkflowBundle() {
	return form.getResourceBundle();
  }

  public List getInputInfo() {
	return inputInfo;
  }

  public List getSubmitButtons() {
	return form.getSubmitButtons();
  }

  public boolean isCustomizedView() {
	return form.isCustomizedView();
  }

  public String getCustomizedView() {
	return form.getCustomizedView();
  }

  public void setIdentification(String identification) {
	this.identification = identification;
  }

  public String getIdentification() {
	return identification;
  }

  public VariableMaps prepareVariables() throws Exception {
	VariableMaps maps = prepareWorkflowVariables(form, getUIInputs().values());
	return maps;
  }

  public VariableMaps prepareWorkflowVariables(Form form, Collection inputs) {
	Map workflowVariables = new HashMap();
	Map jcrVariables = new HashMap();
	for (Iterator iter = inputs.iterator(); iter.hasNext();) {
	  UIInput input = (UIInput) iter.next();
	  String name = input.getName();

	  Object value = "";
	  if (input instanceof UIStringInput) {
		value = ((UIStringInput) input).getValue();
	  } else if (input instanceof UIDateInput) {
		Calendar calendar = ((UIDateInput) input).getCalendar();
		value = calendar.getTime();
	  } else if (input instanceof UIDateTimeInput) {
		Calendar calendar = ((UIDateTimeInput) input).getCalendar();
		value = calendar.getTime();
	  } else if (input instanceof UIWYSIWYG) {
		value = ((UIWYSIWYG) input).getValue();
	  } else if (input instanceof UITextArea) {
		value = ((UITextArea) input).getValue();
	  } else if (input instanceof UICheckBox) {
		value = new Boolean(((UICheckBox) input).getSelect()).toString();
	  } else if (input instanceof UISelectBox) {
		value = ((UISelectBox) input).getValue();
	  } else if (input instanceof UIRadioBox) {
		value = ((UIRadioBox) input).getValue();
	  } else if (input instanceof UIUploadInput) {
		value = ((UIUploadInput) input).getContent();
	  }

	  if (value == null)
		value = "";

	  String jcrPath = (String) input.getProperty("jcrPath");
	  if (jcrPath != null) {
		jcrVariables.put(jcrPath, value);
	  } else {
		workflowVariables.put(name, value);
	  }
	}
	return new VariableMaps(workflowVariables, jcrVariables);
  }

  public void clean() {
	isView = false;
	isCreatedOrUpdated = false;
	dialogPath_ = null;
	inputInfo.clear();
  }

  public String getNodeType() throws Exception {
	return delegate.getNodeType();
  }

  public String getTemplatePath() throws Exception {
	return delegate.getTemplatePath();
  }

  public Node getNode() throws Exception {
    return delegate.getNode() ;    
  }
  
  public List getSupportedLocalise() throws Exception {
    List<String> local = new ArrayList<String> () ;
    return local ;
  }

  public void setNode(Node node) {
	delegate.setNode(node);
  }

  public String getPortalName() {
	return delegate.getPortalName();
  }

  public String getWorkspaceName() throws Exception {
	return delegate.getWorkspaceName();
  }

  public boolean isNodeTypeSupported() {
	return delegate.isNodeTypeSupported();
  }
  
  public Object getComponentInstanceOfType(String className) {
    return delegate.getComponentInstanceOfType(className);
  }  

  public WikiEngineService getWikiService() {
	return delegate.getWikiService();
  }

  public WikiContext getWikiContext() {
	return delegate.getWikiContext();
  }

  public List<Node> getRelations() throws Exception {
	return delegate.getRelations();
  }

  public List<Node> getAttachments() throws Exception {
	return delegate.getAttachments();
  }
  
  public String getAttachmentSize(Node node){
  	return delegate.getAttachmentSize(node);
  }  
  
  public boolean hasPropertyContent(Node node, String property) {
    return delegate.hasPropertyContent(node, property);
  }
  
  public String getLargeNodeTypeIcon(Node node) {
    return delegate.getLargeNodeTypeIcon(node);
  }

  public String getPropertyIconByName(String propertyName) {
    return delegate.getPropertyIconByName(propertyName) ;
  }

  public String getPropertyTypeIcon(Property property) {
    return delegate.getPropertyTypeIcon(property);
  }

  public String getSmallNodeTypeIcon(Node node) {
    return delegate.getSmallNodeTypeIcon(node);
  }
  
  public boolean isRssLink() { return delegate.isRssLink() ; }
  public String getRssLink() { return delegate.getRssLink() ; }
  
  public Locale getCurrentLocale() {
    return delegate.getCurrentLocale();
  }
  
  public static class StartListener extends ExoActionListener {
	public void execute(ExoActionEvent event) throws Exception {
	  UITask uiTask = (UITask) event.getComponent();
	  String remoteUser = Utils.getRemoteUser();
	  VariableMaps maps = uiTask.prepareVariables();
	  Map variables = maps.getWorkflowVariables();
	  uiTask.workflowServiceContainer.startProcess(remoteUser,
		  uiTask.identification, variables);
	  uiTask.setRenderedSibling(UIBPDefinition.class);
	  uiTask.clean();
	}
  }

  public static class EndListener extends ExoActionListener {
	public void execute(ExoActionEvent event) throws Exception {
	  UITask uiTask = (UITask) event.getComponent();
	  VariableMaps maps = uiTask.prepareVariables();
	  try {
		Map variables = maps.getWorkflowVariables();
		uiTask.workflowServiceContainer.endTask(uiTask.identification,
			variables);
	  } catch (Exception ex) {
		ex.printStackTrace();
	  }
	  uiTask.setRenderedSibling(UITaskList.class);
	  uiTask.clean();
	}
  }

  public static class CancelListener extends ExoActionListener {
  	public void execute(ExoActionEvent event) throws Exception {
  	  UITask uiTask = (UITask) event.getComponent();
  	  if (uiTask.isStart) {
  		uiTask.setRenderedSibling(UIBPDefinition.class);
  	  } else {
  		uiTask.setRenderedSibling(UITaskList.class);
  	  }
  	  uiTask.clean();
  	  uiTask.removeTransientActionListenerInterceptors();
  	}
    }
  
    public static class TransitionListener extends ExoActionListener {
  
  	public boolean canHandleAction(String action) {
  	  return !action.startsWith(PREFIX);
  	}
  
  	public void execute(ExoActionEvent event) throws Exception {
  	  UITask uiTask = (UITask) event.getComponent();
  	  VariableMaps maps = uiTask.prepareVariables();
  	  List submitButtons = uiTask.form.getSubmitButtons();
  	  for (Iterator iterator = submitButtons.iterator(); iterator.hasNext();) {
    		Map attributes = (Map) iterator.next();
    		String name = (String) attributes.get("name");
    		if (event.getAction().equals(name)) {
    		  String transition = (String) attributes.get("transition");
    		  try {
            Map variables = maps.getWorkflowVariables();
      			uiTask.workflowServiceContainer.endTask(uiTask.identification, variables, transition);
    		  } catch (Exception e) {
    		    e.printStackTrace();
    		  }
    		}
  	  }
  	  uiTask.setRenderedSibling(UITaskList.class);
  	  uiTask.clean();
  	}
  }
  
  static public class ChangeLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITask uicomp = (UITask) event.getComponent();
      uicomp.language_ = event.getParameter(OBJECTID);
    }   
  }  
}
