/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.workflow.component;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.jcr.PathNotFoundException;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.services.workflow.Form;
import org.exoplatform.services.workflow.Task;
import org.exoplatform.services.workflow.WorkflowFormsService;
import org.exoplatform.services.workflow.WorkflowServiceContainer;

/**
 * Created y the eXo platform team User: Benjamin Mestrallet Date: 16 mai 2004
 */
public class UITaskList extends UIExoCommand {
  static final String MANAGE_STATE = "manageState";
  static Parameter[] MANAGE_STATE_PARAMS = { new Parameter(ACTION , MANAGE_STATE)} ;
  private WorkflowServiceContainer workflowServiceContainer;

  private List tasks_ ;
  private WorkflowFormsService workflowFormsService;

  public UITaskList(WorkflowServiceContainer workflowServiceContainer, 
      WorkflowFormsService workflowFormsService) {
    setRendererType("VelocityRenderer");
    setId("TemplateRenderer");
    this.workflowServiceContainer = workflowServiceContainer;
    this.workflowFormsService = workflowFormsService;
    addActionListener(StateListener.class, MANAGE_STATE);
  }
  
  public Parameter[] getManageStateParams() { return MANAGE_STATE_PARAMS ; }
  
  public String getIconURL(Task task) {
    try{
      Locale locale = ((PortalResources) SessionContainer
        .getComponent(PortalResources.class)).getLocale();        
      Form form = workflowFormsService.getForm(task.getProcessId(), task.getTaskName(), locale);
      return form.getIconURL(); 
    }catch(Exception e) {
      return "" ;
    }    
  }
  
  public String getProcessName(Task task) {
    return this.workflowServiceContainer.getProcess(task.getProcessId()).getName();
  }
  
  public List getTasks() throws Exception { 
    String remoteUser = Utils.getRemoteUser();
    List<Task> unsortedTasks = workflowServiceContainer.getAllTasks(remoteUser);
    return selectVisibleTasks(unsortedTasks) ; 
  }
  
  public Map getVariables(Task task) {
    return this.workflowServiceContainer.getVariables(task.getProcessInstanceId(), task.getId());
  }
  
  public Date getProcessInstanceStartDate(Task task) {
    return this.workflowServiceContainer.getProcessInstance(task.getProcessInstanceId()).getStartDate();
  }
  
  private List<Task> selectVisibleTasks(List<Task> all) {
    List<Task> filtered = new ArrayList<Task>();
    Locale locale = ((PortalResources) SessionContainer
        .getComponent(PortalResources.class)).getLocale();    
    for (Iterator iter = all.iterator(); iter.hasNext();) {
      Task task = (Task) iter.next();
      Form form = workflowFormsService.getForm(task.getProcessId(), task.getTaskName(), locale);
      if(!form.isDelegatedView()) {
        filtered.add(task);
      }
    }
    return filtered;
  }  
  
  public static class StateListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	ResourceBundle res = getApplicationResourceBundle();
      UITaskList uicomp = (UITaskList) event.getComponent();
      String tokenId = event.getParameter(OBJECTID);
      UITask start = (UITask) uicomp.getSibling(UITask.class);
      try{
      	start.setIdentification(tokenId);
      	start.updateUITree();
      	uicomp.setRenderedSibling(UITask.class);
    	}catch(PathNotFoundException e){
    		Information iprovider = findInformationProvider(uicomp);
    		iprovider.addMessage(new ExoFacesMessage(
            res.getString("UITaskList.mes.path-not-found")));
    		Task task = uicomp.workflowServiceContainer.getTask(tokenId);
    		String pid = task.getProcessInstanceId();    		
    		uicomp.workflowServiceContainer.deleteProcessInstance(pid);
    	}
    }
  }
}