/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.cms.drives;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.Value;

import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.security.SecurityService;
import org.picocontainer.Startable;
/**
 * Created by The eXo Platform SARL
 * Author : Nguyen Quang Hung
 *          nguyenkequanghung@yahoo.com
 * Feb 27, 2006
 */
public class ManageDriveServiceImpl implements ManageDriveService, Startable {

  private static String WORKSPACE = "exo:workspace".intern() ;
  private static String PERMISSIONS = "exo:permissions".intern() ;
  private static String VIEWS = "exo:views".intern() ;
  private static String ICON = "exo:icon".intern() ;
  private static String PATH = "exo:path".intern() ;
  private static String FOLDER_TYPE = "exo:folderTypes" ;
  private static String VIEW_REFERENCES = "exo:viewPreferences".intern() ;
  private static String VIEW_NON_DOCUMENT = "exo:viewNonDocument".intern() ;
  private static String VIEW_EXPLORER = "exo:viewExplorer".intern() ;
  private static String VIEW_CLIPBOARD = "exo:viewClipboard".intern() ;

  private ManageDrivePlugin drivePlugin_ ;
  private RepositoryService repoService_ ;
  private ExoCache driveCache_ ;
  private CmsConfigurationService cmsConfigService_ ;
  private SecurityService securityService_ ;

  public ManageDriveServiceImpl(RepositoryService jcrService,CacheService cacheService, 
      CmsConfigurationService cmsConfigurationService, SecurityService securityService ) throws Exception{
    repoService_ = jcrService ;
    cmsConfigService_ = cmsConfigurationService ;    
    driveCache_ = cacheService.getCacheInstance(ManageDriveService.class.getName()) ;
    securityService_ = securityService ;    
  }

  public void setManageDrivePlugin(ManageDrivePlugin drivePlugin) {
    drivePlugin_ = drivePlugin ;
  }  

  public Node getDriveHome() throws Exception {    
    String drivesHomePath = cmsConfigService_.getJcrPath(BasePath.EXO_DRIVES_PATH);
    Object cachedDriveNode = driveCache_.get(BasePath.EXO_DRIVES_PATH) ;
    if(cachedDriveNode != null) return (Node)cachedDriveNode ;
    Session session = repoService_.getRepository().getSystemSession(cmsConfigService_.getWorkspace()) ;        
    Node driveHomeNode =  (Node) session.getItem(drivesHomePath);
    driveCache_.put(BasePath.EXO_DRIVES_PATH,driveHomeNode) ;
    session.logout();
    return driveHomeNode ;
  }

  public List<Node> getAllDrives() throws Exception {
    Node driveHome = getDriveHome() ;
    String drivesHomePath = driveHome.getPath() ;
    Object cachedList = driveCache_.get(drivesHomePath) ;
    if(cachedList != null) return (List<Node>)cachedList ;
    List<Node> driveList = new ArrayList<Node>() ;
    for(NodeIterator iter = driveHome.getNodes(); iter.hasNext() ; ) {
      driveList.add(iter.nextNode()) ;
    }        
    driveCache_.put(drivesHomePath,driveList) ;
    return driveList ;
  }

  public Node getDriveByName(String name) throws Exception{  
    Node driveHome = getDriveHome() ;
    String cachedId = driveHome.getPath()+"/" +name ;
    Object cachedDrive = driveCache_.get(cachedId) ;
    if(cachedDrive != null) return (Node)cachedDrive ;
    Node drive = driveHome.getNode(name) ;
    driveCache_.put(cachedId,drive) ;
    return drive ;        
  }

  public String addDrive(String name, String workspace, String[] permissions, String homePath, 
      String[] views, String icon, boolean viewReferences, boolean viewNonDocument, 
      boolean viewExplorer, boolean viewClipboard,String folderTypes[]) throws Exception {
    String drivesHomePath = cmsConfigService_.getJcrPath(BasePath.EXO_DRIVES_PATH);
    Object cachedDriveNode = driveCache_.get(BasePath.EXO_DRIVES_PATH) ;    
    Session session = repoService_.getRepository().getSystemSession(cmsConfigService_.getWorkspace()) ;        
    Node driveHome =  (Node) session.getItem(drivesHomePath);    
    Node newDriveNode  = null ;
    if (!driveHome.hasNode(name)){
      newDriveNode = driveHome.addNode(name, "exo:drive");
      newDriveNode.setProperty(WORKSPACE, workspace) ;
      newDriveNode.setProperty(PERMISSIONS, permissions) ;      
      newDriveNode.setProperty(PATH, homePath) ;      
      newDriveNode.setProperty(VIEWS, views) ;
      newDriveNode.setProperty(FOLDER_TYPE,folderTypes) ;
      newDriveNode.setProperty(ICON, icon) ;
      newDriveNode.setProperty(VIEW_REFERENCES, Boolean.toString(viewReferences)) ;
      newDriveNode.setProperty(VIEW_NON_DOCUMENT, Boolean.toString(viewNonDocument)) ;
      newDriveNode.setProperty(VIEW_EXPLORER, Boolean.toString(viewExplorer)) ;
      newDriveNode.setProperty(VIEW_CLIPBOARD, Boolean.toString(viewClipboard)) ;      
    }else{
      newDriveNode = driveHome.getNode(name);
      newDriveNode.setProperty(WORKSPACE, workspace) ;
      newDriveNode.setProperty(PERMISSIONS, permissions) ;
      newDriveNode.setProperty(PATH, homePath) ;      
      newDriveNode.setProperty(VIEWS, views) ;
      newDriveNode.setProperty(FOLDER_TYPE,folderTypes) ;
      newDriveNode.setProperty(ICON, icon) ;
      newDriveNode.setProperty(VIEW_REFERENCES, Boolean.toString(viewReferences)) ;
      newDriveNode.setProperty(VIEW_NON_DOCUMENT, Boolean.toString(viewNonDocument)) ;
      newDriveNode.setProperty(VIEW_EXPLORER, Boolean.toString(viewExplorer)) ;
      newDriveNode.setProperty(VIEW_CLIPBOARD, Boolean.toString(viewClipboard)) ;      
    }
    driveHome.save() ;
    session.save();
    session.logout() ;
    driveCache_.clearCache() ;
    driveCache_.put(newDriveNode.getPath(),newDriveNode) ;
    return newDriveNode.getPath() ;
  }

  public List<Node> getAllDriveByPermission(String userId) throws Exception {        
    List<Node> driveByPermission = new ArrayList<Node>() ;    
    List<Node> allDrives = getAllDrives() ;       
    for(Node drive:allDrives) {      
      for(Value v:drive.getProperty(PERMISSIONS).getValues()) {
        String acceptPermission = v.getString() ;        
        if(acceptPermission.equalsIgnoreCase(userId) || securityService_.hasMembershipInGroup(userId,acceptPermission)) {          
          if(!driveByPermission.contains(drive)) 
            driveByPermission.add(drive) ;           
        }         
      }
    }
    return driveByPermission ;
  }

  public void removeDrive(String driveName) throws Exception {
    Node driveHome = getDriveHome() ;
    if(driveHome.hasNode(driveName)){
      driveHome.getNode(driveName).remove();      
      driveHome.save() ;      
      driveCache_.clearCache() ;
    }
  }

  public void start() {
    Session session = null ;
    try{
      session = repoService_.getRepository().getSystemSession(cmsConfigService_.getWorkspace()) ;
      String drivesPath = cmsConfigService_.getJcrPath(BasePath.EXO_DRIVES_PATH) ;
      drivePlugin_.initPredefineDrives(session,drivesPath)  ;
      session.logout() ;      
    }catch (Exception e) {
      if(session != null) session.logout() ;
      e.printStackTrace();
    }     
  }

  public void stop() { }

}
