/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: DigitalSignatureService.java 9386 2006-10-09 15:07:53Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

/**
 * eXo Signer main service.
 * 
 * @version $Rev: 9386 $ $Date: 2006-10-09 22:07:53 +0700 (Mon, 09 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public interface DigitalSignatureService {

	/**
	 * Calculate the content digest of the given Node using the corresponding
	 * nodetype DigitalSinger plugin.
	 * 
	 * @param node
	 *            instance to calculate the digest.
	 * @return calculated hash result using a particular algorithm.
	 * @throws IOException
	 *             failed to read node content/stream.
	 * @throws NoSuchAlgorithmException
	 *             digest algorithm not supported.
	 * @throws Exception
	 *             cannot handle that nodetype.
	 */
	String computeDigest(Node node) throws NoSuchAlgorithmException,
	        IOException, Exception;

	/**
	 * Return the configured digest algorithm to the given node as defined by
	 * user. If not configured, return default digest algorithm value.
	 * 
	 * @param user
	 * @param node
	 * @return
	 */
	String getDigestMethod(String user, Node node);

	/**
	 * List supported NodeType names configured to handle the digital signature
	 * with specific DigitalSigner implementation.
	 * 
	 * @return supported nodetype names handled by the digital signature
	 *         service.
	 */
	String[] getSupportedNodeTypeNames();

	/**
	 * Verifies if the service is configured with Signer that support handling
	 * the given node type.
	 *
	 * @param nodeType node type which looks for a signer support
	 * @return <code>true</code> if service supports signature on the node.
	 */
	boolean hasNodeTypeSupport(String nodeType);

	/**
	 * Include the digital signature contents as node mixinType.
	 *
	 * @param node node to generate and add the signature.
	 * @param encodedSignature Base64 encoded signature.
	 * @param encodedCertChain Base64 encoded certification chain.
	 * @throws RepositoryException 
	 */
	void addSignature(Node node, String encodedSignature, String encodedCertChain) throws RepositoryException;
}
