/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: DigitalSigner.java 9514 2006-10-16 21:36:34Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import javax.jcr.Node;

/**
 * @version $Rev: 9514 $ $Date: 2006-10-17 04:36:34 +0700 (Tue, 17 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public interface DigitalSigner {

	/**
	 * Calculates the message (document, properties set, stream, ...) digest of
	 * a given node.
	 * 
	 * @param node
	 *            instance to parse contents and generate hash.
	 * @param algorithm
	 *            message digest algorithm supported by the JCE provider being
	 *            used.
	 * @return calculated hash result using a particular algorithm.
	 * @throws IOException
	 *             can't read content stream
	 * @throws NoSuchAlgorithmException
	 *             digest algorithm not supported
	 * @throws NoSuchProviderException 
	 */
	byte[] computeDigest(Node node, String algorithm) throws NoSuchAlgorithmException, NoSuchProviderException, IOException;

	/**
	 * Get the JCR NodeType names associated to a Signer.
	 * 
	 * @return the associated JCR NodeType names
	 */
	String[] getNodeTypeNames();
}
