/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: DigitalSignatureServiceImpl.java 9982 2006-11-01 13:00:31Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer.impl;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.applet.util.Base64Encoder;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.signer.DigitalSignatureService;

/**
 * @version $Rev: 9982 $ $Date: 2006-11-01 20:00:31 +0700 (Wed, 01 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class DigitalSignatureServiceImpl implements DigitalSignatureService {

	/**
	 * Digital signature service logger.
	 */
	private Log logger;

	/**
	 * Digital signers plugins.
	 */
	private Map<String, BaseDigitalSigner> signers;

	/**
	 * Digital Signature Service constructor.
	 * <p>
	 * Enabled to receive parameters from configuration file (not used yet).
	 */
	public DigitalSignatureServiceImpl(InitParams params, LogService logSvr) {
		super();

		this.signers = new HashMap<String, BaseDigitalSigner>();
		this.logger = logSvr.getLog(DigitalSignatureService.class);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSignatureService#computeDigest(javax.jcr.Node)
	 */
	public String computeDigest(Node node) throws Exception {

		String nodeTypeName = node.getPrimaryNodeType().getName();
		BaseDigitalSigner signer = signers.get(nodeTypeName);

		if (signer != null) {
			/*
			 * Obtain digest algorithm configured by nodetype per user definition.
			 * TODO Specify the user id from the session correctly.
			 */
			String user = "<user>";
			String algorithm = getDigestMethod(user, node);

			byte[] hash = signer.computeDigest(node, algorithm);
			return base64Encode(hash);
		}

		throw new Exception("Cannot handle nodeType: " + nodeTypeName);
	}

	/**
     * @param hash
     * @return
     */
    private String base64Encode(byte[] hash) {

    	String result = null;

		try {
	    	Base64Encoder encoder = new Base64Encoder();
	    	ByteArrayOutputStream ostream = new ByteArrayOutputStream();

	    	encoder.encode(hash, 0, hash.length, ostream);
	        result = ostream.toString();
        } catch (IOException e) {
        	// do nothing
        }

	    return result;
    }

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSignatureService#getDigestMethod(java.lang.String, javax.jcr.Node)
	 */
	public String getDigestMethod(String user, Node node) {
	    // TODO Add functionality to obtain the configured digest algorithm
		String algorithm = "SHA-1";

		logger.debug("No configured digest algorithm is obtained (TODO). Using: " +
				algorithm);

	    return algorithm;
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSignatureService#getSupportedNodeTypeNames()
	 */
	public String[] getSupportedNodeTypeNames() {
		Set<String> keys = signers.keySet();
	    return keys.toArray(new String[keys.size()]);
	}

	/**
	 * Add new nodetype based digital signer plugin.
	 * @param plugin
	 *            a {@link BaseDigitalSigner} subclass to handle a specific
	 *            nodetype.
	 */
	public void addDigitalSigner(ComponentPlugin plugin) {
		BaseDigitalSigner signer = (BaseDigitalSigner) plugin;

		for(String supportedNodeTypeName : signer.getNodeTypeNames()) {
			signers.put(supportedNodeTypeName, signer);
		}
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSignatureService#hasNodeTypeSupport(java.lang.String)
	 */
	public boolean hasNodeTypeSupport(String nodeType) {
	    return signers.containsKey(nodeType);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSignatureService#addSignature(javax.jcr.Node, java.lang.String, java.lang.String)
	 */
	public void addSignature(Node node, String encodedSignature, String encodedCertChain) throws RepositoryException {
		
	    node.addMixin("exo:xmldsig");
	    node.setProperty("exo:signature", encodedSignature);
	    node.save();
	}
}
