/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: NTFileSigner.java 9514 2006-10-16 21:36:34Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer.impl;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.log.LogService;

/**
 * @version $Rev: 9514 $ $Date: 2006-10-17 04:36:34 +0700 (Tue, 17 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class NTFileSigner extends BaseDigitalSigner {

    public NTFileSigner(InitParams params, LogService logSrv) {

	    super(params, logSrv);
    }

	/* (non-Javadoc)
	 * @see org.exoplatform.services.signer.DigitalSigner#computeDigest(javax.jcr.Node)
	 */
	public byte[] computeDigest(Node fileNode, String algorithm) throws NoSuchAlgorithmException, NoSuchProviderException, IOException {

		byte[] digest = null;

		try {

			/*
			 * Retrieve file stream from jcrContent node/jcrData property
			 */

			InputStream data;

			{
				Node jcrContent = fileNode.getNode("jcr:content");
				data = jcrContent.getProperty("jcr:data").getStream();
			}

	        /*
	         * Use HashFunctionService to calculate message digest
	         */
	        digest = calculateDigest(data, algorithm);

	        {
	        	// debug log
		        StringBuffer hexDigest = new StringBuffer();
	
		        for (byte octet : digest) {
		        	if (hexDigest.length() > 0) hexDigest.append(' ');
		        	hexDigest.append(Integer.toHexString(octet));
		        }
	
		        logger.debug("[" + fileNode.getPath() +
		        		"] calculated digest (using [" + algorithm +
		        		"] algorithm) is: " + hexDigest.toString());
	        }

        } catch (RepositoryException e) {
	       logger.error("Unexpected repository exception", e);
        }

		return digest;
	}

}
