package org.exoplatform.services.workflow.impl.oncepi;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Map;
import java.util.WeakHashMap;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import org.apache.commons.collections.ExtendedProperties;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.resource.Resource;
import org.apache.velocity.runtime.resource.ResourceFactory;
import org.exoplatform.services.templates.velocity.ResourceLoaderPlugin;

import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.ConnectionManager;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.ConnectionManagerHome;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.TaskIdentifier;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.TaskInfo;

/**
 * Enables to retrieve .vm files contained by the Business Process Archives
 */
public class PARResourceLoaderImpl extends ResourceLoaderPlugin {
	
	/** Reference to the File Definition service */
	private WorkflowFileDefinitionService fileDefinitionService = null;
	
	/** Prefix of the Resource names */
	public static final String RESOURCE_NAME_PREFIX = "par:";
	
	/** Caches created Resources */
	private Map<String, Resource> resources = null;
	
	/* (non-Javadoc)
	 * @see org.exoplatform.services.templates.velocity.ResourceLoaderPlugin#canLoadResource(java.lang.String)
	 */
	public boolean canLoadResource(String s) {
		
		return s.startsWith(PARResourceLoaderImpl.RESOURCE_NAME_PREFIX);
	}
	
	/* (non-Javadoc)
	 * @see org.apache.velocity.runtime.resource.loader.ResourceLoader#getLastModified(org.apache.velocity.runtime.resource.Resource)
	 */
	public long getLastModified(Resource resource) {
		
		return 0;
	}
	
	/* (non-Javadoc)
	 * @see org.exoplatform.services.templates.velocity.ResourceLoaderPlugin#getResource(java.lang.String, int, java.lang.String)
	 */
	public Resource getResource(String resourceName, int resourceType,
			String encoding) throws ResourceNotFoundException,
			ParseErrorException, Exception {
		
		// Retrieve the Resource from the cache
		Resource resource = resources.get(resourceName);
		
		if (resource == null) {
			// The Resource is not found in the cache. Create a new one.
			String path = resourceName.substring(
					PARResourceLoaderImpl.RESOURCE_NAME_PREFIX.length(),
					resourceName.length());
			resource = ResourceFactory.getResource(path, resourceType);
			resource.setRuntimeServices(rsvc);
			resource.setName(path);
			resource.setEncoding(encoding);
			resource.setResourceLoader(this);
			resource.process();
		}
		
		return resource;
	}
	
	/* (non-Javadoc)
	 * @see org.apache.velocity.runtime.resource.loader.ResourceLoader#getResourceStream(java.lang.String)
	 */
	public synchronized InputStream getResourceStream(String fileLocation)
	throws ResourceNotFoundException {
		
		InputStream inputStream = null;
		
		try {
			// Retrieve the Task identifier
			String[] fileLocationSplit = fileLocation.split(":");
			String taskId = fileLocationSplit[0];
			
			// Get the connectionManger
			ConnectionManager connectionManager = null;
			try {
				InitialContext ctx = new InitialContext();
				ConnectionManagerHome home = (ConnectionManagerHome) PortableRemoteObject
				.narrow(ctx.lookup("ConnectionManager"),
						ConnectionManagerHome.class);
				connectionManager = (ConnectionManager) home.create();
			} catch (Exception ejbExc) {
				connectionManager = null;
				System.out.println("Error in obtainWorklistServer(no psw): "
						+ ejbExc.toString());
				throw ejbExc;
			}
			
			// get the taskinfo from taskid
			TaskIdentifier ti = WorkflowServiceContainerImpl.idMapping
			.getTaskIndentifier(taskId);
			if (ti == null)
				return null;
			TaskInfo task = null;
			try {
				task = connectionManager.getTask(ti);
			} catch (Exception cme) {
				task = null;
			}
			
			// get packageid & workflow id from TaskInfo
			
			String packageid = task.getPackageId();
			String workflowid = task.getProcessTemplateId();
			
			String processModelId = WorkflowServiceContainerImpl.idMapping
			.putProcessIdTuple(packageid, workflowid);
			String taskName = task.getTaskName();
			
			// Retrieve the contents of the file from the File Definition service
			FileDefinition fileDefinition = fileDefinitionService
			.retrieve(processModelId);
			String customizedViewName = fileDefinition
			.getCustomizedView(taskName);
			byte[] customizedViewBytes = fileDefinition
			.getEntry(customizedViewName);
			
			// Create a Stream from the file contents
			inputStream = new ByteArrayInputStream(customizedViewBytes);
		} catch (Exception e) {
			throw new ResourceNotFoundException(
					"Cannot retrieve data in process " + fileLocation
					+ ". Make sure you have a valid location.");
		}
		
		return inputStream;
	}
	
	/* (non-Javadoc)
	 * @see org.apache.velocity.runtime.resource.loader.ResourceLoader#init(org.apache.commons.collections.ExtendedProperties)
	 */
	public void init(ExtendedProperties configuration) {
	}
	
	/* (non-Javadoc)
	 * @see org.apache.velocity.runtime.resource.loader.ResourceLoader#isSourceModified(org.apache.velocity.runtime.resource.Resource)
	 */
	public boolean isSourceModified(Resource resource) {
		
		return false;
	}
	
	/**
	 * Creates a new instance of the Plugin
	 * 
	 * @param fileDefinitionService reference to the File Definition service
	 */
	public PARResourceLoaderImpl(
			WorkflowFileDefinitionService fileDefinitionService)
	throws Exception {
		
		// Cache references to the injected services
		this.fileDefinitionService = fileDefinitionService;
		
		// Create a cache of Resources
		this.resources = new WeakHashMap<String, Resource>();
	}
}
