package org.exoplatform.services.workflow.impl.oncepi;

/**
 * by chencongwu
 * 2006.7.21
 */

import java.io.IOException;
import java.io.InputStream;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.security.auth.login.LoginContext;

import org.apache.log4j.Logger;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserHandler;
import org.exoplatform.services.security.jaas.BasicCallbackHandler;
import org.exoplatform.services.workflow.Form;
import org.exoplatform.services.workflow.Process;
import org.exoplatform.services.workflow.ProcessInstance;
import org.exoplatform.services.workflow.ProcessesConfig;
import org.exoplatform.services.workflow.Task;
import org.exoplatform.services.workflow.Timer;
import org.exoplatform.services.workflow.WorkflowFormsService;
import org.exoplatform.services.workflow.WorkflowServiceContainer;
import org.exoplatform.services.workflow.impl.oncepi.ejb.UserSession;
import org.exoplatform.services.workflow.impl.oncepi.ejb.UserSessionHome;
import org.picocontainer.Startable;

import cn.ac.ios.tcse.isflow.ProcessServer.Client.ISWFClient;
import cn.ac.ios.tcse.isflow.ProcessServer.Engine.ISWFWorkflowContext;
import cn.ac.ios.tcse.isflow.ProcessServer.VersionControl.ISWFVersionControl;
import cn.ac.ios.tcse.isflow.ProcessServer.WorkDataHandle.Parameter;
import cn.ac.ios.tcse.isflow.ProcessServer.WorkDataHandle.ParameterMode;
import cn.ac.ios.tcse.isflow.ProcessServer.WorkDataHandle.ParameterType;
import cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.ISWFDeployPackage;
import cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.template.Template;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.WorkListServer;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.ConnectionManager;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.ConnectionManagerHome;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.TaskIdentifier;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.TaskInfo;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ConnectionManager.TaskTemplateInfo;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ResourceProcess.ResourceProcess;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ResourceProcess.ResourceProcessHome;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.FlowTemplateInfo;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.UserListItem;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.UserWorkItemMap;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.UserWorkItemMapHome;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.WorkItem;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.WorkItemInfo;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.WorkListHome;
import cn.ac.ios.tcse.isflow.WorkList.WorkListServer.Util.WLSConstants;
import cn.ac.ios.tcse.isflow.xpdl.Activities;
import cn.ac.ios.tcse.isflow.xpdl.Activity;
import cn.ac.ios.tcse.isflow.xpdl.DataType;
import cn.ac.ios.tcse.isflow.xpdl.FormalParameter;
import cn.ac.ios.tcse.isflow.xpdl.FormalParameters;
import cn.ac.ios.tcse.isflow.xpdl.ProcessHeader;
import cn.ac.ios.tcse.isflow.xpdl.TypeDeclaration;
import cn.ac.ios.tcse.isflow.xpdl.WorkflowProcess;

public class WorkflowServiceContainerImpl implements WorkflowServiceContainer,
		Startable {

	/**
	 * Only one instance for all containerImpls
	 */
	public static IDMapping idMapping = new IDMappingImpl();

	private ConnectionManager connectionManager = null;

	private ProcessesConfig configuration;

	private WorkflowFileDefinitionService fileDefinitionService = null;

	/** Reference to the Configuration Manager Service */
	private ConfigurationManager configurationManager = null;

	/** Reference to the Organization Service */
	private OrganizationService organizationService = null;

	/** Reference to the Portal Container */
	private PortalContainer portalContainer = null;

	private WorkflowFormsService formsService = null;

	private UserSession userSession = null;

	/** Connection to ProcessServer * */
	ISWFClient psClient = null;

	/**
	 * In methods as delProcess etc, we need a user who is the action provider.
	 * In bonita implementation, it has a UserSession to make this out, but for
	 * PI we mandatorily add it here.
	 * 
	 * What's more, this is the user Name, not user Id. So in many places it
	 * will need another step to mapping the userName to user ID.
	 * 
	 * Use UserSessionBean.getUserName() to keep user infomation updated;
	 * 
	 * defaultUser is now used to check if user has been changed.
	 */
	private String defaultUser = "";

	private cn.ac.ios.tcse.isflow.WorkList.WorkListServer.ScheduleCenter.WorkList workList = null;

	/**
	 * Get a WorkListBean Instance Every bean instance will open a connection to
	 * the Queue, so they will share the same queue.
	 */
	private boolean initWorkList() {
		WorkListHome home;
		try {
			Context initial = new InitialContext();
			home = (WorkListHome) initial.lookup("WorkList");

		} catch (NamingException e) {
			throw new EJBException(WLSConstants.WLS_JNDI_EXCEPTION_STRING);
		}
		try {
			workList = home.create();
		} catch (javax.ejb.CreateException ce) {
			ce.printStackTrace();
		}
		if (workList != null)
			return true;
		return false;
	}

	/**
	 * get User-Workitem Map Bean
	 * 
	 * @return
	 */
	private UserWorkItemMap getUserWorkItemMap() {
		UserWorkItemMapHome home = null;
		try {
			Context initial = new InitialContext();
			Object objref = initial.lookup("UserWorkItemMap");
			home = (UserWorkItemMapHome) objref;
		} catch (NamingException e) {
			throw new EJBException(WLSConstants.WLS_JNDI_EXCEPTION_STRING);
		}
		UserWorkItemMap userWorkItemMap = null;
		try {
			userWorkItemMap = (UserWorkItemMap) home.create();
		} catch (Exception e) {
			throw new EJBException(WLSConstants.WLS_CREATE_EJB_ERROR_STRING);
		}

		return userWorkItemMap;
	}

	private ISWFClient getPSClient() {
		if (psClient == null) {
			try {
				psClient = new ISWFClient(java.net.InetAddress
						.getByName("127.0.0.1"), 7777, "PI", "123");
			} catch (UnknownHostException he) {
				he.printStackTrace();
			}
		}
		if (psClient.isConnected() == false)
			logger.error("ISWFClient not Connected!");
		return psClient;
	}

	private boolean initUserSessionBean() {
		UserSessionHome home;
		try {
			Context initial = new InitialContext();
			home = (UserSessionHome) initial.lookup("UserSession");

		} catch (NamingException e) {
			throw new EJBException(WLSConstants.WLS_JNDI_EXCEPTION_STRING);
		}
		try {
			userSession = home.create();
		} catch (Exception ce) {
			ce.printStackTrace();
		}
		if (userSession != null)
			return true;
		return false;
	}

	private ResourceProcess getResourceProcess() {
		ResourceProcess rp = null;
		ResourceProcessHome home = null;
		try {
			Context initial = new InitialContext();
			Object objref = initial.lookup("ResourceProcess");
			home = (ResourceProcessHome) objref;
		} catch (NamingException e) {
			throw new EJBException(WLSConstants.WLS_JNDI_EXCEPTION_STRING);
		}

		try {
			rp = (ResourceProcess) home.create();
		} catch (Exception rpe) {
			this.logger.error("Resource Process Bean creation failed!");
			rpe.printStackTrace();
		}

		return rp;
	}

	/**
	 * For debug & Testing
	 */
	private Logger logger = LogUtil.getLogger();

	public WorkflowServiceContainerImpl(
			WorkflowFileDefinitionService fileDefinitionService,
			WorkflowFormsService formsService,
			OrganizationService organizationService,
			ConfigurationManager configurationManager,
			PortalContainer portalContainer, InitParams params)
			throws Exception {
		logger.info("Entering WorkflowServiceContainerImpl constructor ");

		// startUpProcessServer();
		startUpWorkListServer();

		initWorkList();

		this.fileDefinitionService = fileDefinitionService;
		this.formsService = formsService;
		this.organizationService = organizationService;
		this.configurationManager = configurationManager;
		this.portalContainer = portalContainer;
		if (params != null)
			this.configuration = (ProcessesConfig) params.getObjectParamValues(
					ProcessesConfig.class).get(0);

	}

	private void startUpWorkListServer() {
		new WorkListServer().startServer();
	}

	/**
	 * Use current logged user info open a connection to the WorkListServer
	 * 
	 * @throws Exception
	 */
	private void initWLSConnectionManager() throws Exception {
		/**
		 * get the pwd from pi
		 */
		logger.info("init WLSConnectionManager");
		ResourceProcess rp = this.getResourceProcess();
		String defaultUser = getDefaultUser();
		String userId = rp.getUserID(defaultUser);
		String pwd = cn.ac.ios.tcse.isflow.ProcessServer.UserManager.User
				.getUserData(userId).Password;
		try {
			InitialContext ctx = new InitialContext();
			ConnectionManagerHome home = (ConnectionManagerHome) PortableRemoteObject
					.narrow(ctx.lookup("ConnectionManager"),
							ConnectionManagerHome.class);
			logger.info("Creat ConnectionManager username: " + defaultUser
					+ "pwd:" + pwd);
			connectionManager = (ConnectionManager) home.create(defaultUser,
					pwd);
		} catch (Exception ejbExc) {
			connectionManager = null;
			logger.error("Error in obtainWorklistServer(no psw): "
					+ ejbExc.toString());
			throw ejbExc;
		}
	}

	private String getDefaultUser() {
		/**
		 * get the default user that is the user who logger in.
		 */
		initUserSessionBean();

		try {
			defaultUser = this.userSession.getUserName();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return defaultUser;
	}

	private boolean isUserChanged() {
		String previousUser = this.defaultUser;
		if (previousUser.equals(this.getDefaultUser()))
			return false;
		return true;
	}

	private ConnectionManager getConnectionManager() {
		/**
		 * In case an previous user has just logged out, new user logger in
		 * since we donnot have a way to figure out when user logged out
		 * 
		 * we have to check if user name has been changed, if so a new
		 * conenction must be opened.
		 * 
		 * Obviously, a good solution is to try to reset connectionManager to
		 * null when the user logged out.
		 */
		if (this.connectionManager == null || this.isUserChanged()) {
			try {
				this.initWLSConnectionManager();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return this.connectionManager;
	}

	public void deployProcess(InputStream iS) throws IOException {
		FileDefinition fileDefinition = new XPDLFileDefinition(iS);
		try {
			String userId = this.getConnectionManager().getUserId(
					this.getDefaultUser());
			fileDefinition.deploy(userId);
			String processName = fileDefinition.getProcessModelName();

			String processId = this.getProcessIDFromName(processName);

			List roleIds = this.getConnectionManager().getRoleIds();
			Iterator iter = roleIds.iterator();
			while (iter.hasNext())
				this.configProcessStartRole(processId, (String) iter.next());
			// Store the File Definition so that it can be retrieved if eXo
			// restarts
			this.fileDefinitionService.store(fileDefinition, processId);
		} catch (Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * Same as deployFlowTemplate in WorkListServer. To config a deployed
	 * process the role that can instantiate it.
	 * 
	 * @param processId
	 * @param roleId
	 */
	private void configProcessStartRole(String processId, String roleId) {
		// Construct FlowTemplateInfo Object
		String[] id = WorkflowServiceContainerImpl.idMapping
				.getProcessIdTuple(processId);
		String packageid = id[0];
		String workflowid = id[1];

		// get workflow process Object
		WorkflowProcess wp = null;

		ISWFDeployPackage deploy = new ISWFDeployPackage("");
		cn.ac.ios.tcse.isflow.xpdl.Package pkg = deploy
				.getProcessTemplate(packageid);

		TypeDeclaration[] tdArr = null;
		if (pkg.getTypeDeclarations() != null)
			tdArr = pkg.getTypeDeclarations().getTypeDeclaration();

		WorkflowProcess[] wparray = pkg.getWorkflowProcesses()
				.getWorkflowProcess();
		if (wparray == null) {
			/**
			 * cannot find workflows in this package
			 */
			return;
		}
		for (int index = 0; index < wparray.length; index++) {
			if (workflowid.equals(wparray[index].getId())) {
				wp = wparray[index];
				break;
			}
		}

		String applicationId = wp.getApplicationID();
		ProcessHeader pHeader = wp.getProcessHeader();
		String processDesc = pHeader.getDescriptionText();

		// Get start activity
		Activities as = wp.getActivities();
		Activity[] actArr = as.getActivity();
		String activityID = new String("");

		for (int actCount = 0; actCount < actArr.length; actCount++) {
			if ("Start".equals(actArr[actCount].getType())) {
				activityID = actArr[actCount].getId();
				break;
			}
		}

		// parameter processing
		FormalParameters fps = wp.getFormalParameters();

		ArrayList paras = new ArrayList();

		if (fps != null) {

			FormalParameter[] fpArr = fps.getFormalParameter();

			if (fpArr != null) {
				for (int paraCount = 0; paraCount < fpArr.length; paraCount++) {
					// Mode
					ParameterMode mode = null;
					if ("IN".equals(fpArr[paraCount].getMode()))
						mode = ParameterMode.IN;
					else if ("OUT".equals(fpArr[paraCount].getMode()))
						mode = ParameterMode.OUT;
					else
						mode = ParameterMode.INOUT;

					// »Type
					ParameterType pType = null;
					DataType dt = fpArr[paraCount].getDataType();
					String basicType = null;
					if (dt.getBasicType() != null) {
						basicType = dt.getBasicType().getType();
					} else {
						// declared type
						String dtId = dt.getDeclaredType().getId();
						if (tdArr == null)
							basicType = "";
						else {
							for (int tdCount = 0; tdCount < tdArr.length; tdCount++) {
								if (dtId.equals(tdArr[tdCount].getId())) {
									basicType = tdArr[tdCount].getBasicType()
											.getType();
									break;
								}
							}
						}
					}
					if (basicType.equals("STRING"))
						pType = ParameterType.STR;
					else if (basicType.equals("FLOAT"))
						pType = ParameterType.FLO;
					else if (basicType.equals("INTEGER"))
						pType = ParameterType.INT;
					else if (basicType.equals("DOUBLE"))
						pType = ParameterType.DOU;
					else if (basicType.equals("LONG"))
						pType = ParameterType.LON;
					else if (basicType.equals("BOOLEAN"))
						pType = ParameterType.BOO;
					else
						pType = ParameterType.BYT;

					Parameter para = new Parameter(fpArr[paraCount].getId(),
							null, pType, mode);
					paras.add(para);
				}
			}
		}
		FlowTemplateInfo info = new FlowTemplateInfo(packageid, workflowid,
				activityID);
		info.setParas(paras);
		info.setPkgName(pkg.getName());
		info.setProcessName(wp.getName());
		info.setProcessDesc(processDesc);
		info.setAppID(applicationId);

		try {
			logger.info("try to config roleid= " + roleId
					+ " with start priviledge of the process " + processId);
			this.getConnectionManager().deployFlowTemplate(info, roleId);
		} catch (Exception re) {
			re.printStackTrace();
		}

	}

	public List<Process> getProcesses() {

		/**
		 * Bonita implementation: By calling this method, we voluntarily return
		 * the models which can be instantiated by the user.
		 * 
		 * Duplicates must be eliminated! Use process id to identified it
		 * currently.
		 */

		logger.info("Entering method: getProcess()");
		HashMap<String, FlowTemplateInfo> temp = new HashMap<String, FlowTemplateInfo>();
		List<Process> result = new ArrayList<Process>();
		FlowTemplateInfo templateinfo = null;
		try {
			List roles = this.getConnectionManager().getRoleIds();
			Iterator iter = roles.iterator();
			while (iter.hasNext()) {
				String roleid = (String) iter.next();
				List startableTemplates = this.getConnectionManager()
						.getStartableProcessTemplates(roleid);
				Iterator templateIter = startableTemplates.iterator();
				while (templateIter.hasNext()) {
					templateinfo = (FlowTemplateInfo) templateIter.next();
					temp.put(templateinfo.getWpID(), templateinfo);

				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		// now templateinfos in the hash map donnot have dupicate entries,
		// process them here.
		Iterator iter = temp.values().iterator();
		while (iter.hasNext()) {
			templateinfo = (FlowTemplateInfo) iter.next();
			String packageid = templateinfo.getPkgID();
			String workflowid = templateinfo.getWpID();
			String processid = idMapping.putProcessIdTuple(packageid,
					workflowid);
			result.add(this.getProcess(processid));
			logger.info("A startable process is found : " + processid);
		}
		return result;
	}

	public Process getProcess(String processId) {
		String packageid = null;
		String workflowid = null;
		String[] data = idMapping.getProcessIdTuple(processId);
		if (data == null) {
			/**
			 * Cannot find the key in our map failes
			 */
			logger.info("Cannot find the key in our map! in method getProcess");
			return null;
		}
		packageid = data[0];
		workflowid = data[1];

		WorkflowProcess wp = null;

		ISWFDeployPackage deploy = new ISWFDeployPackage("");
		cn.ac.ios.tcse.isflow.xpdl.Package pkg = deploy
				.getProcessTemplate(packageid);
		WorkflowProcess[] wparray = pkg.getWorkflowProcesses()
				.getWorkflowProcess();
		if (wparray == null) {
			/**
			 * cannot find workflows in this package
			 */
			logger
					.info("cannot find the workflow corresponding to the package in method getProcess");
			return null;
		}
		for (int index = 0; index < wparray.length; index++) {
			if (workflowid.equals(wparray[index].getId())) {
				wp = wparray[index];
				break;
			}
		}

		if (wp == null)
			return null;
		return new ProcessData(wp, processId);
	}

	public boolean hasStartTask(String processId) {

		/**
		 * Workflow process in oncepi always has a start activity
		 */
		return true;
	}

	public List<ProcessInstance> getProcessInstances(String processId) {
		String packageid = null;
		String workflowid = null;
		String[] data = idMapping.getProcessIdTuple(processId);
		if (data == null) {
			/**
			 * Cannot find the key int our map failes
			 */
			return null;
		}
		packageid = data[0];
		workflowid = data[1];

		List<ProcessInstance> result = new ArrayList<ProcessInstance>();
		List wfcontexts = null;

		try {
			wfcontexts = this.getPSClient().listInstance(workflowid);
		} catch (Exception e) {
			e.printStackTrace();
		}

		Iterator iter = wfcontexts.iterator();
		while (iter.hasNext()) {
			ISWFWorkflowContext wc = (ISWFWorkflowContext) iter.next();
			result.add(new ProcessInstanceData(wc, processId));
		}

		return result;

	}

	public ProcessInstance getProcessInstance(String processInstance) {
		ISWFWorkflowContext wc = null;
		try {
			wc = this.getPSClient().getInstanceProperties(processInstance);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (wc == null)
			logger
					.error("Cannot get PRocess WorkflowContext in method getProcessInstance!");
		String packageid = wc.getPkgID();
		String workflowid = wc.getWpID();
		String processid = idMapping.putProcessIdTuple(packageid, workflowid);
		return new ProcessInstanceData(wc, processid);
	}

	public Map getVariables(String processInstanceId, String taskId) {
		/**
		 * Returns variables corresponding to specific task instance. It is used
		 * to show task properties page. According to the form, get needed form
		 * varible values from Workflow Context Data field which are stored in
		 * setVaribables method.
		 */
		Map<Object, Object> variables = new HashMap<Object, Object>();

		// get workflow context object

		ISWFWorkflowContext wc = null;
		try {
			ISWFClient client = new ISWFClient(java.net.InetAddress
					.getByName("127.0.0.1"), 7777, "PI", "123");
			// wc=this.getPSClient().getInstanceProperties(processInstanceId);
			wc = client.getInstanceProperties(processInstanceId);
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (wc == null) {
			logger
					.error("Cannot find corresponding workflow context in method getVariables, processInstanceId: "
							+ processInstanceId);
			logger.error(" We will manually repair it here..");
			/**
			 * In case that a process instance already not existed, however
			 * workitems and user-workitem map still existed. We have to delete
			 * these undated ones here.
			 */
			this.deleteWorkitems(processInstanceId);
			variables
					.put("ERROR DETECTED!",
							"The task maybe not existed already, refresh this page to recover");
			return variables;
		}

		Task task = this.getTask(taskId);
		String processid = task.getProcessId();

		HashMap workflowRelevantData = wc.getWorkflowRelevantData();
		// convert hashmap<name, parameter> to hashMap<name, value>
		logger.info("Convert workflowRelevantData to Variables");
		Iterator iter = workflowRelevantData.keySet().iterator();
		Object value;
		Parameter para;
		while (iter.hasNext()) {
			Object key = iter.next();
			para = (Parameter) workflowRelevantData.get(key);
			value = para.getValue();
			/**
			 * Only return string type, currently
			 */
			if (para.getType().getValue() == ParameterType.STRING_TYPE)
				variables.put(key, value);
		}

		logger.info("Get Form variable definition");
		Form form = this.formsService.getForm(processid, task.getTaskName(),
				Locale.getDefault());
		List<Map<String, Object>> formVariables = form.getVariables();

		logger.info("Change form variable type");
		for (Map<String, Object> formVariable : formVariables) {
			String key = (String) formVariable.get("name");
			String component = (String) formVariable.get("component");
			variables.put(key, WorkflowServiceContainerHelper.stringToObject(
					(String) variables.get(key), component));
		}

		return variables;
	}

	public List<Task> getTasks(String processInstanceId) {
		/**
		 * Get all user tasks first, and then iterate through them to find any
		 * tasks that belong to this specific process instance. What's more,
		 * redundancy must be eliminated.
		 */
		HashMap<String, Task> allTasks = new HashMap<String, Task>();

		LinkedList allUserTasks = null;
		try {
			allUserTasks = this.getConnectionManager().getAllUserWorkItemList();
			Iterator userIter = allUserTasks.iterator();
			UserListItem item;
			while (userIter.hasNext()) {
				item = (UserListItem) userIter.next();
				List workIterms = item.getWorkList();
				Iterator itemIter = workIterms.iterator();
				WorkItemInfo wii;
				while (itemIter.hasNext()) {
					wii = (WorkItemInfo) itemIter.next();
					WorkItem wi = this.workList.getWorkItem(wii);
					if (wi.getProcessInstanceID().equals(processInstanceId)) {
						TaskIdentifier id;
						TaskInfo temp;
						id = new TaskIdentifier(wii);
						try {
							temp = this.getConnectionManager().getTask(id);
						} catch (Exception e) {
							temp = null;
						}
						TaskData td = null;
						if (temp != null) {
							td = new TaskData(temp);
							idMapping.putTaskIndentifier(td.getId(), id);
						}

						// put it to the hash table
						allTasks.put(wi.getActivityInstanceID(), td);
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return new ArrayList<Task>(allTasks.values());
	}

	public Task getTask(String taskId) {
		TaskIdentifier ti = idMapping.getTaskIndentifier(taskId);
		if (ti == null)
			return null;
		TaskInfo task = null;
		try {
			task = this.getConnectionManager().getTask(ti);
		} catch (Exception cme) {
			task = null;
		}
		return new TaskData(task);

	}

	public List<Task> getAllTasks(String user) throws Exception {
		List<Task> allTasks = new java.util.ArrayList<Task>();
		logger.info("Task: getUserTaskList userName" + user);
		allTasks.addAll(getUserTaskList(user));
		logger.info("Task: getGroupTaskList");
		allTasks.addAll(getGroupTaskList(user));
		logger.info("Task return");
		return allTasks;
	}

	public List<Task> getUserTaskList(String user) {
		/**
		 * The input is UserName.
		 */
		logger.info("Enter GetUserTaskList");

		List<WorkItemInfo> workiteminfos = null;
		try {
			String username = this.getDefaultUser();
			String userid = this.getConnectionManager().getUserId(username);
			workiteminfos = this.getConnectionManager().getUserTask(userid);
		} catch (Exception e) {
			e.printStackTrace();
		}

		List<Task> tasks = new LinkedList<Task>();

		if (workiteminfos == null) {
			logger.info("no task has been found");
			return tasks;
		}

		TaskInfo temp;
		TaskIdentifier id;
		for (WorkItemInfo wii : workiteminfos) {
			id = new TaskIdentifier(wii);
			try {
				temp = this.getConnectionManager().getTask(id);
			} catch (Exception e) {
				e.printStackTrace();
				temp = null;
			}
			if (temp != null) {
				TaskData td = new TaskData(temp);
				tasks.add(td);
				idMapping.putTaskIndentifier(td.getId(), id);
			}

		}
		return tasks;
	}

	public List<Task> getGroupTaskList(String user) throws Exception {
		return new ArrayList<Task>();
	}

	public List<Timer> getTimers() {
		// TODO Auto-generated method stub
		return null;
	}

	public void startProcess(String processId) {
		startProcess(null, processId, new HashMap());
	}

	/**
	 * remoteUser is ignored, since we donnot need that. What's more how to
	 * handle the map of variables in exo to a list of parameters to our PI is
	 * not decied yet.
	 */
	public void startProcess(String remoteUser, String processId, Map variables) {
		String packageid = null;
		String workflowid = null;
		String[] data = idMapping.getProcessIdTuple(processId);
		if (data == null) {
			/**
			 * Cannot find the key int our map failes
			 */
			return;
		}
		packageid = data[0];
		workflowid = data[1];

		logger.info("Step1: find process template ready to be startted");
		FlowTemplateInfo templateinfo = null;
		FlowTemplateInfo result = null;
		String role = null;
		try {
			List roles = this.getConnectionManager().getRoleIds();
			Iterator iter = roles.iterator();
			while (iter.hasNext()) {
				String roleid = (String) iter.next();
				List startableTemplates = this.getConnectionManager()
						.getStartableProcessTemplates(roleid);
				Iterator templateIter = startableTemplates.iterator();
				while (templateIter.hasNext()) {
					templateinfo = (FlowTemplateInfo) templateIter.next();
					if (templateinfo.getPkgID().equals(packageid)) {
						logger.info("find flow template info.");
						result = templateinfo;
						role = roleid;
						break;
					}

				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (result == null) {
			logger
					.error("Cannot find process flow template when trying to start it");
		}

		logger.info("Step2: instantiate the process");

		try {
			this.getConnectionManager().instantiateProcess(result, role);
		} catch (Exception re) {
			re.printStackTrace();
		}

		logger.info("Step3: take care of the parameters");
		ArrayList para = new ArrayList();
		Form form = this.formsService.getForm(processId,
				ProcessData.START_STATE_NAME, Locale.getDefault());
		List<Map<String, Object>> formVariableList = form.getVariables();
		// Retrieve Form variables and put them in a HashMap to access them
		// easily

		HashMap<String, Map<String, Object>> formVariables = new HashMap<String, Map<String, Object>>();
		for (Map<String, Object> formVariable : formVariableList) {
			formVariables.put((String) formVariable.get("name"), formVariable);
		}

		// Process each variable to set
		for (Object keyObject : variables.keySet()) {
			// Retrieve the variable from the Form. It may not exist.
			String key = (String) keyObject;
			Map<String, Object> formVariable = formVariables.get(key);

			// Determine the Component name
			String componentName = null;
			if (formVariable != null) {
				// The variable exists in the Form
				componentName = (String) formVariable.get("component");
			}
			if (componentName == null) {
				/*
				 * The variable does not exist in the Form or the component is
				 * unset. The default in that case is "text".
				 */
				componentName = "text";
			}

			// The value can be any Object. Convert it to a String
			String value = WorkflowServiceContainerHelper.objectToString(
					variables.get(keyObject), componentName);

			// construct Parameter Instance
			Parameter paraData = new Parameter(key, value, ParameterType.STR);
			para.add(paraData);
			logger.info("Added parameter: Name=" + paraData.getName()
					+ "Value= " + paraData.getValue());
		}

		logger.info("Step4: startup process");
		boolean success = false;
		try {
			success = this.getConnectionManager().processResponse(result, role,
					para);
		} catch (Exception e) {
			e.printStackTrace();
		}

		logger.info("Process start success= " + new Boolean(success));
	}

	public void startProcessFromName(String remoteUser, String processName,
			Map variables) {
		String processid = this.getProcessIDFromName(processName);
		if (processid == null)
			return;
		this.startProcess(remoteUser, processid, variables);
	}

	private String getProcessIDFromName(String processName) {
		cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.template.TemplateList result = null;
		try {
			cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.ISWFDeployPackage deploy = new cn.ac.ios.tcse.isflow.ProcessServer.processdeploy.ISWFDeployPackage(
					"");
			result = deploy.getAllTemplate();
		} catch (Exception e) {
			e.printStackTrace();
		}
		String packagefind = null;
		String workflowfind = null;

		// iterate all templates to find our process
		Template[] list = result.getTemplate();
		for (int i = 0; i < list.length; i++) {
			Template tem = list[i];
			String packageid = tem.getPackageID();
			packagefind = packageid;
			ISWFDeployPackage deploy = new ISWFDeployPackage("");
			cn.ac.ios.tcse.isflow.xpdl.Package pkg = deploy
					.getProcessTemplate(packageid);
			WorkflowProcess[] wparray = pkg.getWorkflowProcesses()
					.getWorkflowProcess();
			for (int index = 0; index < wparray.length; index++) {
				if (wparray[index].getName().equalsIgnoreCase(processName)) {
					workflowfind = wparray[index].getId();
					break;
				}
			}
		}
		if (workflowfind == null) {
			/**
			 * Not find!
			 */
			logger
					.error("getProcessIdFromName, cannot find any process with this name! "
							+ processName);
			return null;
		}
		String processid = idMapping.putProcessIdTuple(packagefind,
				workflowfind);
		return processid;
	}

	/**
	 * According to the bonita implemention: it first starts the task, set the
	 * variables to the task, and then terminate the task.
	 */
	public void endTask(String taskId, Map variables) {
		Task task = this.getTask(taskId);
		TaskIdentifier tid = idMapping.getTaskIndentifier(taskId);
		try {
			TaskTemplateInfo tmi = this.getConnectionManager().taskExecute(tid,
					task.getActorId());
			logger.info("set variables to process instace in method endTask");
			logger.info(variables.toString());
			// not needed now, replaced by the following out/in-out parameters
			// this.setVariables(task.getProcessInstanceId(), taskId,
			// variables);

			ArrayList paras = tmi.getParameters();
			Iterator iter = paras.iterator();
			Parameter para;
			ParameterMode mode;
			while (iter.hasNext()) {
				para = (Parameter) iter.next();
				mode = para.getMode();
				if (mode.getValue() == ParameterMode.INOUT_TYPE
						|| mode.getValue() == ParameterMode.OUT_TYPE) {
					logger.info("setting out type parameter values in endTask");
					logger.info("variables name is " + para.getName());
					logger.info("variable old value is " + para.getValue()
							+ "new value is " + variables.get(para.getName()));
					para.setValue(variables.get(para.getName()));
				}
			}
			this.getConnectionManager().taskResponse(tid, task.getActorId(),
					paras);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("TaskResponse in endTask Error! \n");
		}

	}

	public void endTask(String taskId, Map variables, String transition) {
		endTask(taskId, variables);
	}

	public void deleteProcess(String processId) {
		logger.info("delete process : " + processId
				+ "in method deletePorcess@WorkflowContainerImpl");
		String[] tuple = idMapping.getProcessIdTuple(processId);
		String packageid = tuple[0];
		String workflowid = tuple[1];

		/**
		 * remove all process instances
		 */
		List<ProcessInstance> instances = this.getProcessInstances(processId);
		Iterator iter = instances.iterator();
		while (iter.hasNext()) {
			String instanceId = ((ProcessInstance) iter.next())
					.getProcessInstanceId();
			this.deleteProcessInstance(instanceId);
		}

		/**
		 * undeploy process's starting role
		 * 
		 */
		try {
			List<FlowTemplateInfo> allTemplates = this.getConnectionManager()
					.getAllDeployedTemplates();
			iter = allTemplates.iterator();
			FlowTemplateInfo result = null, temp = null;
			while (iter.hasNext()) {
				temp = (FlowTemplateInfo) iter.next();
				if (temp.getWpID().equals(workflowid)
						&& temp.getPkgID().equals(packageid)) {
					result = temp;
					break;
				}
			}

			if (result == null) {
				logger
						.error("Cannot find corresponding flowtemplate to undeploy while deleting process");
			}
			this.getConnectionManager().deleteDeployFlowTemplate(result);

			// then delete it from the db
			String userId = this.getConnectionManager().getUserId(
					getDefaultUser());
			ISWFVersionControl.delPackage(userId, packageid);
		} catch (Exception e) {
			e.printStackTrace();
		}

		this.formsService.removeForms(processId);
		this.fileDefinitionService.remove(processId);
	}

	/**
	 * Delete all workitems and user-workitem map corresponding to this
	 * specified processintace.
	 * 
	 * @param processInstaceId
	 */
	private void deleteWorkitems(String processInstanceId) {
		// 0. find all workitems from current processInstance, same as
		// getTasks(String )
		HashMap<String, WorkItem> workItems = new HashMap<String, WorkItem>();
		LinkedList allUserTasks = null;
		try {
			allUserTasks = this.getConnectionManager().getAllUserWorkItemList();
			Iterator userIter = allUserTasks.iterator();
			UserListItem item;
			while (userIter.hasNext()) {
				item = (UserListItem) userIter.next();
				List workIterms = item.getWorkList();
				Iterator itemIter = workIterms.iterator();
				WorkItemInfo wii;
				while (itemIter.hasNext()) {
					wii = (WorkItemInfo) itemIter.next();
					WorkItem wi = this.workList.getWorkItem(wii);
					if (wi.getProcessInstanceID().equals(processInstanceId)) {
						// put it to the hash table
						workItems.put(item.getUserName(), wi);
					}
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		// Iterate through the list, to apply step 1
		Iterator keyIter = workItems.keySet().iterator();
		WorkItem item = null;
		while (keyIter.hasNext()) {
			String userName = (String) keyIter.next();
			item = workItems.get(userName);
			// 1. delete user-workitem map entry
			UserWorkItemMap map = this.getUserWorkItemMap();
			if (map == null)
				logger.error("Error, cannot get user-workitem Map");
			map.deleteWorkItem(userName, item.getWorkItemInfo(), item
					.getExecuteProperty());
			// 2 delete workitem
			try {
				this.workList.deleteWorkItem(item.getWorkItemInfo());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void deleteProcessInstance(String processInstanceId) {

		// delete workitems first
		this.deleteWorkitems(processInstanceId);
		// delete workflow context in the ps
		System.out.println("Now remove workflow context in PS");
		this.getPSClient().removeInstance(processInstanceId);
	}

	public void start() {
		LoginContext lc = null;

		try {
			// Retrieve the already deployed Processes. No need to be logged in
			// yet.
			// BnProjectLocalHome projectHome = BnProjectUtil.getLocalHome();
			// Collection<BnProjectLocal> projects = projectHome.findAll();

			/**
			 * in bonita impl it first find wheather the projects has been
			 * deployed if not it will deploy now.
			 */
			// The predefined Processes need to be deployed
			java.util.HashSet predefinedProcesses = configuration
					.getPredefinedProcess();
			String processLoc = configuration.getProcessLocation();

			/*
			 * Account used to deploy the predefined Processes. TODO Specify it
			 * in the Service configuration.
			 */
			final String account = "exoadmin";

			/*
			 * Retrieve the corresponding password from the Organization
			 * Service. Currently the password needs the "@portalname" suffix so
			 * that the target portal is specified to the Login Module.
			 */
			UserHandler userHandler = this.organizationService.getUserHandler();
			User user = userHandler.findUserByName(account);
			final char[] password = (user.getPassword() + "@" + ((PortalContainer) portalContainer)
					.getPortalContainerInfo().getContainerName()).toCharArray();

			/*
			 * As the Application Server is being started, there is no logged in
			 * user. It is therefore required to login programmatically.
			 */
			BasicCallbackHandler handler = new BasicCallbackHandler(account,
					password);
			lc = new LoginContext("tomcat", handler);
			lc.login();

			if (predefinedProcesses == null) {
				lc.logout();
				return;
			}

			// Deploy each predefined Process
			for (Iterator iter = predefinedProcesses.iterator(); iter.hasNext();) {
				String parFile = (String) iter.next();
				InputStream iS;
				try {
					iS = this.configurationManager.getInputStream(processLoc
							+ parFile);
					this.deployProcess(iS);
				} catch (Exception e) {
					// Process does not exist
					e.printStackTrace();
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				/*
				 * Logout. This does not hurt if it fails as Exceptions are
				 * ignored.
				 */
				lc.logout();
			} catch (Exception ignore) {
			}
		}
	}

	public void stop() {
		/**
		 * close ProcessServer Client Connection
		 */
		if (this.psClient != null)
			psClient.close();
		/**
		 * shut down WorkList Server
		 */
		new WorkListServer().stopServer();
	}
}
