/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.benchmark.jcr;

import java.util.ArrayList;

import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;

import com.sun.japex.JapexDriverBase;
import com.sun.japex.TestCase;

/**
 * Created by The eXo Platform SARL Author : Vitaliy Obmanjuk
 * vitaliy.obmanjuk@exoplatform.com.ua 07.03.2007
 */
public class JCRDriverMethods extends JapexDriverBase {

  private static final String NUMBER_OF_NODES      = "numberOfNodes";

  private static final String NUMBER_OF_PROPERTIES = "numberOfProperties";

  private static final String MIXIN_TYPE           = "mixinType";
  
  private static final String SAVE_SESSION         = "saveSession";

  public static final String  TESTCASE_METHOD_NAME = "methodName";
  
  protected Repository        repository;

  public void run(final TestCase tc) {
    Session session = null;
    System.out.println("\n===JCRDriverMethods.java, run");
    try {
      session = repository.login(new SimpleCredentials("admin", "admin".toCharArray()), "ws");
      // checking the method to execute
      if (tc.hasParam(TESTCASE_METHOD_NAME)) {
        if (tc.getParam(TESTCASE_METHOD_NAME).equals("doAddNodesOfTypeNtUnstructured"))
          doAddNodesOfTypeNtUnstructured(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doSetProperties"))
          doSetProperties(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doRemoveNodes"))
          doRemoveNodes(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doRemoveProperties"))
          doRemoveProperties(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doAddMixin"))
          doAddMixin(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doCheckin"))
          doCheckin(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doCheckout"))
          doCheckout(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doLock"))
          doLock(tc, session);
        else if (tc.getParam(TESTCASE_METHOD_NAME).equals("doUnlock"))
          doUnlock(tc, session);
      }
      // the method has been executed
    } catch (Throwable exception) {
      exception.printStackTrace();
      throw new RuntimeException(exception.getMessage(), exception);
    } finally {
      session.logout();
    }
  }
  
  private void doAddNodesOfTypeNtUnstructured(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doAddNodesOfTypeNtUnstructured");
    Node doAddNodesOfTypeNtUnstructuredNode = session.getRootNode().addNode("doAddNodesOfTypeNtUnstructuredNode" + String.valueOf(Math.random()), "nt:unstructured");
    long start = System.nanoTime();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      doAddNodesOfTypeNtUnstructuredNode.addNode("testNode" + String.valueOf(Math.random()), "nt:unstructured");
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }

  private void doSetProperties(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doSetProperties");
    Node doSetPropertyNode = session.getRootNode().addNode("doSetPropertyNode" + String.valueOf(Math.random()), "nt:unstructured");
    long start = System.nanoTime();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_PROPERTIES); i++) {
      doSetPropertyNode.setProperty("testProperty" + i, String.valueOf(Math.random()));
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_PROPERTIES));
    //to calculate average time of execution of each operation 
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doRemoveNodes(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doRemoveNodes");
    Node doRemoveNodesNode = session.getRootNode().addNode(
        "doRemoveNodes" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doRemoveNodesNode.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.remove();
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doRemoveProperties(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doRemoveProperties");
    Node doRemovePropertiesNode = session.getRootNode().addNode(
        "doRemovePropertiesNode" + String.valueOf(Math.random()), "nt:unstructured");
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_PROPERTIES); i++) {
      doRemovePropertiesNode.setProperty("testProperty" + i, String.valueOf(Math.random()));
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long start = System.nanoTime();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_PROPERTIES); i++) {
      doRemovePropertiesNode.getProperty("testProperty" + i).remove();
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_PROPERTIES));
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doAddMixin(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doAddMixin");
    Node doAddMixinNode = session.getRootNode().addNode(
        "doAddMixinNode" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doAddMixinNode.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.addMixin(tc.getParam(MIXIN_TYPE));
      if (tc.getBooleanParam(SAVE_SESSION)) {
        session.save();
      }
    }
    long end = System.nanoTime();
    // ? testAddMixReferenceableToNodeOfTypeNtBase.remove();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    //to calculate average time of execution of each operation    
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doCheckin(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doCheckin");
    Node doCheckin = session.getRootNode().addNode(
        "doCheckin" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doCheckin.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      tmpNode.addMixin(tc.getParam(MIXIN_TYPE));
      session.save();
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.checkin();
    }
    long end = System.nanoTime();
    //to make node removeable
    for (Node tmpNode : nodesList) {
      tmpNode.checkout();
    }    
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doCheckout(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doCheckout");
    Node doCheckout = session.getRootNode().addNode(
        "doCheckout" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doCheckout.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      tmpNode.addMixin(tc.getParam(MIXIN_TYPE));
      session.save();      
      tmpNode.checkin();
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.checkout();
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doLock(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doLock");
    Node doLock = session.getRootNode().addNode(
        "doLock" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doLock.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      tmpNode.addMixin(tc.getParam(MIXIN_TYPE));
      session.save();
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.lock(true,true);
    }
    long end = System.nanoTime();
    //to make node removeable
    for (Node tmpNode : nodesList) {
      tmpNode.unlock();
    }    
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }
  
  private void doUnlock(TestCase tc, Session session) throws Exception {
    System.out.println("\n===JCRDriverMethods.java, doUnlock");
    Node doUnlock = session.getRootNode().addNode(
        "doUnlock" + String.valueOf(Math.random()), "nt:unstructured");
    ArrayList<Node> nodesList = new ArrayList<Node>();
    for (int i = 1; i < tc.getIntParam(NUMBER_OF_NODES); i++) {
      Node tmpNode = doUnlock.addNode("NodeOfTypeNtUnstructured#" + i,
           "nt:unstructured");
      tmpNode.addMixin(tc.getParam(MIXIN_TYPE));
      session.save();
      tmpNode.lock(true,true);
      nodesList.add(tmpNode);
    }
    long start = System.nanoTime();
    for (Node tmpNode : nodesList) {
      tmpNode.unlock();
    }
    long end = System.nanoTime();
    double result = (end-start)/(1000000.0 * tc.getIntParam(NUMBER_OF_NODES));
    tc.setDoubleParam("japex.resultValue", result);
  }
}
