/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.storage.swap;

import java.io.File;
import java.util.List;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.BinarySwapEntry;
import org.exoplatform.services.jcr.util.StringNumberParser;
import org.exoplatform.services.jcr.util.UUIDGenerator;
import org.exoplatform.services.jcr.util.io.BLOBUtil;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: BinaryValueSwapStorage.java 9751 2006-10-25 12:00:47Z peterit $
 */

public class BinaryValueSwapStorage {
  
  private static Log log = ExoLogger.getLogger("jcr.BinaryValueSwapStorage");
  
  private File rootDir = null; 
  private int minFileSize = -1;
  private boolean copyLocalFiles = false;

  public BinaryValueSwapStorage(String rootDirPath, String minFileSizeText, String copyLocalFilesText) {
    
    if (rootDirPath == null || minFileSizeText == null)
      throw new IllegalArgumentException("BinaryValueSwapStorage configuration error - null parameters can't be used."
          + " rootDirPath: " + rootDirPath + ", minFileSizeText: " +  minFileSizeText);
    
    rootDir = new File(rootDirPath);
    if(rootDir.exists()) {
      File[] files = rootDir.listFiles();
      if (files != null)
        for (File f: files) {
          try {
            f.delete();
          } catch(Throwable th) {
            log.warn("Can't delete swap file: " + (f != null ? f.getAbsolutePath() : "[null]"), th);
          } 
        }
      rootDir.delete();
    }
    rootDir.mkdirs();
    try {
      minFileSize = StringNumberParser.parseNumber(minFileSizeText).intValue();
    } catch(Throwable th) {
      log.warn("Can't parse minimal swap file size: " + minFileSizeText 
          + ". Will use " + BLOBUtil.BLOB_MIN_SIZE + " bytes threshold.", th);
      minFileSize = BLOBUtil.BLOB_MIN_SIZE;
    }
    
    copyLocalFiles = Boolean.parseBoolean(copyLocalFilesText);
  }
  
  public BinaryValueSwapStorage(BinarySwapEntry bsConfig) {
    //BinarySwapEntry bsConfig = wsConfig.getBinarySwap();
    this(bsConfig.getDirectoryPath(), bsConfig.getMinFileSize(), bsConfig.getCopyLocalFiles());
  }
  
  public String generatePath() {
    // [PN] 03.10.06 It may doesn't work if some parent dirs is not existed in workspace init-time
    // So, re-check existense.
    if(!rootDir.exists()) {
      rootDir.mkdirs();
    }
    return rootDir.getAbsoluteFile() + File.separator + UUIDGenerator.generate();
  }
  
//  public PersistedValueData createPersistedValueData(LazyInputStream stream, int orderNumber) throws IOException {
//    return new PersistedValueData(stream, orderNumber, generatePath(), minFileSize);
//  }

  public int getMinFileSize() {
    return minFileSize;
  }

  public boolean isCopyLocalFiles() {
    return copyLocalFiles;
  }
  
}
