/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.ItemVisitor;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.access.AccessControlEntry;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.PermissionValue;
import org.exoplatform.services.jcr.impl.core.value.ValueConstraintsMatcher;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author Gennady Azarenkov
 * @version $Id: PropertyImpl.java 9750 2006-10-25 11:39:50Z geaz $
 */

public class PropertyImpl extends ItemImpl implements Property {

  protected int type;
  
//  /**
//   * @deprecated
//   */
  private PropertyDefinitions definitions;
  
//  private PropertyDefinition definition;

  
  /**
   * just to simplify operations
   */
  private TransientPropertyData propertyData;

  // new impl
  PropertyImpl(ItemData data, SessionImpl session) throws RepositoryException, ConstraintViolationException {
    super(data, session);
    //initDefinition();
    loadData(data);
  }

  PropertyImpl(TransientPropertyData data, NodeImpl parentNode, int state, SessionImpl session
      ) throws RepositoryException, ConstraintViolationException {

    super(data, parentNode, state, session);
    // propertyData = (PropertyData)data;
    loadData(data);
  }
  
  

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.ItemImpl#loadData(org.exoplatform.services.jcr.datamodel.ItemData)
   */
  void loadData(ItemData data) throws RepositoryException, ConstraintViolationException {

    if (!(data instanceof TransientPropertyData))
      throw new RepositoryException("Load data: TransientPropertyData is expected, but have "+data);

    this.data = data;
    this.propertyData = (TransientPropertyData)data;
    this.type = propertyData.getType();
    initDefinitions();
//    initDefinition();
   
  }

//  private void initDefinition() throws RepositoryException, ConstraintViolationException {
//    NodeImpl parent = (NodeImpl) getParent();
//    this.definition = parent.findChildPropertyDefinition(propertyData.getQPath().getName());
////    this.definition = parent().findChildPropertyDefinition(propertyData.getQPath().getName());
//  }
  

  /**
   * @throws RepositoryException
   * @throws ConstraintViolationException
   */
  private void initDefinitions() throws RepositoryException, ConstraintViolationException {

    NodeImpl parent = (NodeImpl) getParent();
    NodeType[] nodeTypes = parent.getAllNodeTypes();
    PropertyDefinitions defs = null;
    this.definitions = null;
    for (int i = 0; i < nodeTypes.length; i++) {
      
      defs = ((NodeTypeImpl) nodeTypes[i]).getPropertyDefinitions(getName());
      
      if (defs.getAnyDefinition() != null) { // includes residual set
        definitions = defs;
        if (!((PropertyDefinitionImpl) defs.getAnyDefinition()).isResidualSet())
          break;
      }
    }

    if (definitions == null)
      throw new ConstraintViolationException("Definition for property " + getPath() + " not found.");
  }


  public TransientItemData getActualItemData() throws RepositoryException {
    return propertyData;
  }

  /**
   * @see javax.jcr.Property#getValue
   */
  public Value getValue() throws ValueFormatException, RepositoryException {

    if (getDefinition().isMultiple())
      throw new ValueFormatException("The property " + getPath() + " is multi-valued (6.2.4)");

    
    if (propertyData.getValues() != null && propertyData.getValues().size() == 0)
      throw new ValueFormatException("The single valued property " + getPath() + " is empty");
    
    return valueFactory.loadValue((TransientValueData)propertyData.getValues().get(0), propertyData.getType());


  }

  /**
   * @see javax.jcr.Property#getValues
   */
  public Value[] getValues() throws ValueFormatException, RepositoryException {
    
    // Check property definition and life-state flag both
    if (!getDefinition().isMultiple() && !isMultiValued()) {
      throw new ValueFormatException("The property is single-valued (6.2.4)");
    }

    // The array returned is a copy of the stored values
    return getValueArray();
  }

  /**
   * @see javax.jcr.Property#getString
   */
  public String getString() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getString();
    } catch (ValueFormatException e) {
      throw new ValueFormatException("PropertyImpl.getString() for " + getPath() + " failed: " + e);
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getString() for " + getPath() + " failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getDouble
   */
  public double getDouble() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getDouble();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getDouble() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getLong
   */
  public long getLong() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getLong();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getLong() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getStream
   */
  public InputStream getStream() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getStream();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getStream() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getDate
   */
  public Calendar getDate() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getDate();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getDate() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getBoolean
   */
  public boolean getBoolean() throws ValueFormatException, RepositoryException {
    try {
      return getValue().getBoolean();
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getBoolean() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getNode
   */
  public Node getNode() throws ValueFormatException, RepositoryException {
    try {
      String uuid = ((BaseValue) getValue()).getReference();
      return session.getNodeByUUID(uuid);
    } catch (IllegalStateException e) {
      throw new ValueFormatException("PropertyImpl.getNode() failed: " + e);
    }
  }

  /**
   * @see javax.jcr.Property#getLength
   */
  public long getLength() throws ValueFormatException, RepositoryException {

    return ((BaseValue) getValue()).getLength();
    //return propertyData.getValues().get(0).getLength();
  }

  /**
   * @see javax.jcr.Property#getLengths
   */
  public long[] getLengths() throws ValueFormatException, RepositoryException {

    Value[] thisValues = getValues();
    
    long[] lengths = new long[thisValues.length];
    for (int i = 0; i < lengths.length; i++) {
      lengths[i] =  ((BaseValue) thisValues[i]).getLength();
    }
    return lengths;
  }

  /**
   * @see javax.jcr.Property#getDefinition
   */
  public PropertyDefinition getDefinition() throws RepositoryException {

    PropertyDefinition def = null;

    if (propertyData.getValues() == null || propertyData.getValues().size() == 0) {
      def = definitions.getAnyDefinition();
    }
    
    if (def == null) { // && values.length == 1
      if (propertyData.getValues().size() == 1)
        def = definitions.getDefinition(false);
      if (def == null)
        def = definitions.getDefinition(true);
    }

    if (def == null) {
      throw new RepositoryException("FATAL: property definition is NULL " + getPath() + " "
          + propertyData.getValues());
    } else {
      return def;
    }
  }

  /**
   * @see javax.jcr.Property#getType
   */
  public int getType() {
    return type;
  }

  /**
   * @see javax.jcr.Item#isModified
   */
  public boolean isModified() {
    return (getState() == ItemChangeState.UPDATED);
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Value value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value[] newValues;
    int type = PropertyType.UNDEFINED;
    if (value != null) {
      newValues = new Value[1];
      newValues[0] = value;
      type = value.getType();
    } else {
      newValues = null;
    }
    doSetValue(newValues, false, type, ItemChangeState.UPDATED, true, true);

  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Value[] values) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {

    int type = PropertyType.UNDEFINED;
    if (values != null && values.length > 0) {
      if (values[0] != null)
        type = values[0].getType();
      else
        type = PropertyType.UNDEFINED;
    }
    doSetValue(values, true, type, ItemChangeState.UPDATED, true, true);
  }
  
  // registerInDataManager,
  // doExternalValidation
  // TODO: Add ConstraintViolationException throw if the change would violate a
  // node type
  public void doSetValue(Value[] propertyValues, boolean multiValue, int expectedType, int changeState,
      boolean registerInDataManager, boolean doExternalValidation) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    // log.debug("Definitions "+getPath()+" "+values.length+" "+definitions.dump());
    
    if (doExternalValidation) {
      NodeImpl parent = (NodeImpl) getParent();

      if (getDefinition().isProtected())
        throw new ConstraintViolationException("Can not set protected property " + getPath());

      if (!parent.isCheckedOut())
        throw new VersionException("Node " + parent.getPath()
            + " or its nearest ancestor is checked-in");

      // Check locking
      if (!parent.checkLocking())
        throw new LockException("Node " + parent.getPath() + " is locked ");
    }    

    // no such definition
    if (isMultiValued() != multiValue ) { 
      if (multiValue) {        
        throw new ValueFormatException("Can not assign multiple-values Value to a single-valued property " + getPath());
      } else {
        throw new ValueFormatException("Can not assign single-value Value to a multiple-valued property " + getPath());
      }
    }

//    if (definitions.getDefinition(multiValue) == null)
//      throw new ValueFormatException("Can not set value due to multiplicity constraint "
//          + getPath());
    

    // Remove this property if values == null
    if (propertyValues == null) {
      if (getDefinition().isMandatory() || getDefinition().isProtected())
        throw new ConstraintViolationException(
            "Can not remove (by setting null value) protected or mandatory property " + getPath());

      remove();
      if (log.isDebugEnabled())
        log.debug("Set null value (" + getPath() + ", multivalued: " + multiValue + "), property removed");
      return;
    }

    
    //  cast to required type if neccessary
    int requiredType = getDefinition().getRequiredType();
    int propType = requiredType;
    if (propertyValues.length > 0) {
      if (requiredType == PropertyType.UNDEFINED) {
        if (expectedType == PropertyType.UNDEFINED) {
          for(Value val: propertyValues) {
            if(val != null) {
              expectedType = val.getType();
              break;
            }
          }
        }
        propType = expectedType;
      }
    }

    // prepare TransientValueData list 
    List <ValueData> valueDataList = new ArrayList <ValueData>(propertyValues.length);
    // fill datas and also remove null values and reorder values
    for (Value value: propertyValues) {
      if (value != null) {
        valueDataList.add(valueData(value, propType));
      } else {
        if (log.isDebugEnabled())
          log.debug("Set null value (" + getPath() + ", multivalued: " + multiValue + ")");
      }
    }
    

    // Check value constraints
    checkValueConstraints(valueDataList, propType);
    
    propertyData.setValues(valueDataList);
    propertyData.setType(propType);
    
    loadData(propertyData);
    
    
/////////////////////////////////////////////////////    

    setState(changeState);
    
    // [PN] 20.07.06 add to the parent
    if (changeState != ItemChangeState.DELETED)
      parent.addChildProperty(this);
    
    if (registerInDataManager) {
      dataManager.update(this);
    }
  }
  /**
   * @return multiValued property of data field (PropertyData) it's a life-state
   *         property field which contains multiple-valued flag for value(s)
   *         data. Can be set in property creation time or from persistent
   *         storage.
   */
  public boolean isMultiValued() {
    return ((PropertyData) data).isMultiValued();
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(String value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(value));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(InputStream stream) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(stream));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(double number) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(number));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(long number) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(number));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Calendar date) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(date));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(boolean b) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    setValue(valueFactory.createValue(b));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(Node value) throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    // Uuid uuid = new Uuid(value.getUUID());
    setValue(valueFactory.createValue(value));
  }

  /**
   * @see javax.jcr.Property#setValue
   */
  public void setValue(String[] values) throws ValueFormatException, VersionException,
      LockException, ConstraintViolationException, RepositoryException {

    Value[] strValues = null;
    if (values != null) {
      strValues = new Value[values.length];
      for (int i = 0; i < values.length; i++)
        strValues[i] = valueFactory.createValue(values[i]);
      // new StringValue(values[i]);
    }
    setValue(strValues);
  }

  // ////////////////// Item implementation ////////////////////

  /**
   * @see javax.jcr.Item#accept
   */
  public void accept(ItemVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  /**
   * @see javax.jcr.Item#isNode
   */
  public boolean isNode() {
    return false;
  }

  // ////////////////////////////////

  /**
   * @return
   * @throws RepositoryException
   */
  public Value[] getValueArray() throws RepositoryException {
    
    // [PN] 09.04.06
    //loadProperty();
    Value[] values = new Value[propertyData.getValues().size()];
    for(int i=0; i<values.length; i++) {
      values[i] = valueFactory.loadValue((TransientValueData)propertyData.getValues().get(i), propertyData.getType());
    }
    return values; 
  }

  public String dump() {
    String vals = "Property " + getPath() + " values: ";
    try {  
      for (int i = 0; i < getValueArray().length; i++) {
        vals += new String(((BaseValue) getValueArray()[i]).getInternalData().getAsByteArray()) + ";";
      }
    } catch(Exception e) {
      e.printStackTrace();
    }
    return vals;
  }

  private void checkValueConstraints(List <ValueData> newValues, int type) throws ConstraintViolationException, RepositoryException {

    // [PN] 20.09.06
    ValueConstraintsMatcher constraints = new ValueConstraintsMatcher(getDefinition().getValueConstraints(), session);

    for (ValueData value: newValues) {
      if (!constraints.match(value, type)) {
        String strVal = null;
        try {
          strVal = ((TransientValueData)value).getString();
        } catch(Throwable e) {
          log.error("Error of value read: " + e.getMessage(), e);
        }
        throw new ConstraintViolationException("Can not set value '" + strVal + "' to " + getPath()
            + " due to value constraints ");
      }
    }  
  }
  
  //----------------------- Object.equals() ----------------------

  /**
   * @author [PN] 18.04.06
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof PropertyImpl) {
      try {
        // by path
        return getLocation().equals(((PropertyImpl) obj).getLocation());
      } catch(Exception e) {
        return false;
      }
    }
    return false;
  }  
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    try {
      return getLocation().getAsString(false).hashCode();
    } catch(Exception e) {
      return super.hashCode();
    }
  }   
  
  private ValueData valueData(Value value, int type)
      throws RepositoryException, ValueFormatException {

    if (value == null)
      return null;
    
    switch (type) {
    case PropertyType.STRING:
      return new TransientValueData(value.getString());
    case PropertyType.BINARY:
      return new TransientValueData(value.getStream());
    case PropertyType.BOOLEAN:
      return new TransientValueData(value.getBoolean());
    case PropertyType.LONG:
      return new TransientValueData(value.getLong());
    case PropertyType.DOUBLE:
      return new TransientValueData(value.getDouble());
    case PropertyType.DATE:
      return new TransientValueData(value.getDate());
    case PropertyType.PATH:
      InternalQPath pathValue = locationFactory.parseJCRPath(value.getString()).getInternalPath(); 
      return new TransientValueData(pathValue);
    case PropertyType.NAME:
      InternalQName nameValue = locationFactory.parseJCRName(value.getString()).getInternalName();  
      return new TransientValueData(nameValue);
    case PropertyType.REFERENCE:
      Uuid uuid = new Uuid(value.getString());
      return new TransientValueData(uuid);
    case ExtendedPropertyType.PERMISSION:
      PermissionValue permValue = (PermissionValue) value;
      AccessControlEntry ace = new AccessControlEntry(
          permValue.getIdentity(), permValue.getPermission());
      return new TransientValueData(ace);
    default:
      throw new ValueFormatException(
          "ValueFactory.convert() unknown or unconvertable type " + type);
    }
  }
}