/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author Gennady Azarenkov
 * @version $Id: ItemDataChangesLog.java 7657 2006-08-07 13:23:11Z peterit $
 * 
 * Stores collection of ItemStates
 */
public class ItemDataChangesLog {
  
  protected final List <ItemState> items;
  protected String sessionId;
  
  // temporary!
  protected SessionImpl session;
  
  /**
   * @deprecated
   */
  public ItemDataChangesLog(List <ItemState> items, SessionImpl session) {
    this.items = items;
    this.sessionId = null;
    this.session = session;
  }

  /**
   * @deprecated
   */
  public ItemDataChangesLog(ItemState change, SessionImpl session ) {
    this(new ArrayList <ItemState> (), session);
    items.add(change);
  }
  
  public ItemDataChangesLog(List <ItemState> items, String sessionId) {
    this.items = items;
    this.sessionId = sessionId;
  }
  
  public ItemDataChangesLog(ItemState change, String sessionId) {
    this(new ArrayList <ItemState> (), sessionId);
    items.add(change);
  }
  

 /**
  * @return all states
  */
  public List <ItemState> getAllStates() {
    return items;
  }
  
  /**
   * @return number of stored states
   */
  public int getSize() {
    return items.size();
  }

  /**
   * @param id
   * @return current state (i.e. last change) of item data with UUID == id
   * or null if not found
   */
  public ItemState getItemState(String itemUuid) {
    ItemState lastState = null;
    for (ItemState state: getAllStates()) {
      if (state.getData().getUUID().equals(itemUuid)) {
        lastState = state;
      }
    }
    return lastState;
  }

  /**
  * @param qpath
  * @return current state (i.e. last change) of item data with qpath
  * or null if not found
  */
  public ItemState getItemState(InternalQPath itemPath) {
    ItemState lastState = null;
    for (ItemState state: getAllStates()) {
      if (state.getData().getQPath().equals(itemPath)) {
        lastState = state;
      }
    }
    return lastState;
  }
  
  /**
   * returns all states for with uuid == itemUuid
   * @param itemUuid
   * @return
   */
  public List <ItemState> getItemStates(String itemUuid) {
    List <ItemState> states = new ArrayList <ItemState>();
    for (ItemState state: getAllStates()) {
      if (state.getData().getUUID().equals(itemUuid)) {
        states.add(state);
      }
    }
    return states;
  }

  
  /**
   * @return sessionId
   */
  public String getSessionId() {
    return sessionId;
  }
  
  /**
   * @deprecated
   */
  public SessionImpl getSession() {
    return session;
  }
  
  public String dump() {
    String str = "ChangesLog: \n";
    for(int i=0; i<items.size(); i++)
      str+=" "+items.get(i).getData().getQPath().getAsString()+" "+ItemState.getStateName(items.get(i).getState())+" \n";
    return str;
  }
}
