/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.dataflow.session;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.dataflow.ItemDataVisitor;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.SessionDataManager;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: SessionDataTraversingVisitor.java 9660 2006-10-21 16:14:41Z peterit $
 */

public abstract class SessionDataTraversingVisitor implements ItemDataVisitor {

  protected final int maxLevel;

  protected int currentLevel;
  
  protected final SessionDataManager dataManager;

  public SessionDataTraversingVisitor(SessionDataManager dataManager, int  maxLevel) {
    this.maxLevel = maxLevel;
    this.dataManager = dataManager;
  }
  
  public SessionDataTraversingVisitor(SessionDataManager dataManager) {
    this.maxLevel = -1;
    this.dataManager = dataManager;
  }

  public void visit(PropertyData property) throws RepositoryException {
    entering(property, currentLevel);
    leaving(property, currentLevel);
  }

  public void visit(NodeData node) throws RepositoryException {
    try {
      entering(node, currentLevel);
      if (maxLevel == -1 || currentLevel < maxLevel) {
        currentLevel++;
        // TODO use session scope item data list instead transact manager's list
        for(PropertyData data : dataManager.getTransactManager().getChildPropertiesData(node))
          data.accept(this);
        for(NodeData data : dataManager.getTransactManager().getChildNodesData(node))
          data.accept(this);
        currentLevel--;
      }
      leaving(node, currentLevel);
    } catch (RepositoryException re) {
      currentLevel = 0;
      throw re; 
    }

  }

  protected abstract void entering(PropertyData property, int level) throws RepositoryException;

  protected abstract void entering(NodeData node, int level) throws RepositoryException;

  protected abstract void leaving(PropertyData property, int level) throws RepositoryException;

  protected abstract void leaving(NodeData node, int level) throws RepositoryException;


}
