/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.ext.metadata;

import java.io.InputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.Map.Entry;

import javax.jcr.PathNotFoundException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.ValueFormatException;

import org.apache.commons.chain.Context;
import org.exoplatform.commons.utils.QName;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.command.action.Action;
import org.exoplatform.services.document.DocumentReaderService;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.JCRName;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;

/**
 * Created by The eXo Platform SARL        .
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class AddMetadataAction implements Action {

  public boolean execute(Context ctx) throws Exception {

    //NodeImpl parent = (NodeImpl)ctx.get("jcr:parent");
    PropertyImpl property = (PropertyImpl)ctx.get("currentItem");
    NodeImpl parent = (NodeImpl)property.getParent();
    if(!parent.isNodeType("nt:resource"))
      throw new Exception("incoming node is not nt:resource type");
    InputStream data = null;
    String mimeType = null;
    
    if(property.getInternalName().equals(Constants.JCR_DATA)) {
      data = ((TransientPropertyData)property.getData()).getValues().get(0).getAsStream();
        try {
          mimeType = parent.getProperty("jcr:mimeType").getString();
        } catch (PathNotFoundException e) {
          return false;
        }
    } else if(property.getInternalName().equals(Constants.JCR_MIMETYPE)) {
      mimeType = property.getString();
      try {
        PropertyImpl propertyImpl = (PropertyImpl)parent.getProperty("jcr:data");
        data = ((TransientPropertyData)propertyImpl.getData()).getValues().get(0).getAsStream();
        //data = parent.getProperty("jcr:data").getStream();
      } catch (PathNotFoundException e) {
        return false;
      }

    } else {
      //System.out.println("return  >>> "+property.getName()); 
      return false;
    }

    if(!parent.isNodeType("dc:elementSet"))
      parent.addMixin("dc:elementSet");
    
//    InputStream is = resource.getProperty("jcr:data").getStream();
//    String mimeType = resource.getProperty("jcr:mimeType").getString();
    
    DocumentReaderService readerService = (DocumentReaderService)
    ((ExoContainer)ctx.get("exocontainer")).getComponentInstanceOfType(DocumentReaderService.class);
    
    Properties props = readerService.getDocumentReader(mimeType).getProperties(data);
    Iterator entries = props.entrySet().iterator();
    while(entries.hasNext()) {
      Entry entry = (Entry)entries.next();
      QName qname = (QName)entry.getKey();
      JCRName jcrName = property.getSession().getLocationFactory().createJCRName(new InternalQName(qname.getNamespace(), qname.getName()));
      
      PropertyDefinitions pds = parent.getSession().getWorkspace().getNodeTypeManager().findPropertyDefinitions(
          jcrName.getInternalName(), 
          ((NodeData) parent.getData()).getPrimaryTypeName(), 
          ((NodeData) parent.getData()).getMixinTypeNames());
      if(pds.getDefinition(true) != null) {
        Value[] values = {createValue(entry.getValue(), property.getSession().getValueFactory())};
        parent.setProperty(jcrName.getAsString(), values);
      } else {
        parent.setProperty(jcrName.getAsString(), createValue(entry.getValue(), property.getSession().getValueFactory()));
      }
    }

    return false;
  }

  
  private Value createValue(Object obj, ValueFactory factory) throws ValueFormatException {
    if(obj instanceof String)
      return factory.createValue((String)obj);
    else if(obj instanceof Calendar)
      return factory.createValue((Calendar)obj);
    else if(obj instanceof Date) {
      Calendar cal = Calendar.getInstance();
      cal.setTime((Date)obj);
      return factory.createValue(cal);
    } else {
      throw new ValueFormatException("Unsupported value type "+obj.getClass());
    }
  }

}
