/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.ext.audit;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.jcr.ext.BaseStandaloneTest;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.observation.ExtendedEvent;

/**
 * Created by The eXo Platform SARL        .
 * @author Gennady Azarenkov
 * @version $Id: AuditServiceTest.java 11944 2007-01-16 16:20:24Z ksm $
 *
 * JCR Use Case test sample
 */

public class AuditServiceTest extends BaseStandaloneTest {


  public void testIfAuditServiceConfigured() throws Exception {

    assertNotNull(container.getComponentInstanceOfType(AuditService.class));
    assertNotNull(repository.getNodeTypeManager().getNodeType("exo:auditable"));
    assertNotNull(repository.getNodeTypeManager().getNodeType("exo:auditRecord"));
    assertNotNull(repository.getNodeTypeManager().getNodeType("exo:auditHistory"));
    assertNotNull(repository.getNodeTypeManager().getNodeType("exo:auditStorage"));
  }
  
  public void testIfAuditStorageCreated() {

    AuditService service = (AuditService)container.getComponentInstanceOfType(AuditService.class);
    ExtendedNode node = null;
    try {
      node = (ExtendedNode)session.getRootNode().addNode("auditable");
      node.addMixin("exo:auditable");
    } catch (RepositoryException e) {
      fail("Fail to add node or add mixin");
    }
    try {
      service.createHistory(node);
      session.save();
      assertNotNull(session.getNodeByUUID(AuditService.AUDIT_STORAGE_ID));
    } catch (RepositoryException e) {
      fail("Fail to create AUDIT_STORAGE");
    }
    
    try {
      assertNotNull(session.getNodeByUUID(node.getProperty("exo:auditHistory").getString()));
    } catch (RepositoryException e) {
      fail("Fail to create AUDITHISTORY");
    }
  }
  
  public void testAddInfoToAuditStorage() throws RepositoryException {
    AuditService service = (AuditService)container.getComponentInstanceOfType(AuditService.class);
    ExtendedNode node = null;
    try {
      node = (ExtendedNode)session.getRootNode().addNode("auditable");
      node.addMixin("exo:auditable");
      service.createHistory(node);
      session.save();
    } catch (RepositoryException e) {
      fail("Fail to init AuditStorage");
    }
    Property property1 = node.setProperty("property1","value1");
    service.addRecord(node,ExtendedEvent.NODE_ADDED);
    service.addRecord(property1,ExtendedEvent.PROPERTY_ADDED);
    
    assertEquals(2,session.getRootNode().getNode("auditable").getNodes().getSize());
   
  }

}
