/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.dataflow.DataManager;
import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.MutableItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;
import org.exoplatform.services.jcr.impl.dataflow.session.SessionChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.version.VersionHistoryDataHelper;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author Gennady Azarenkov
 * @version $Id: VersionImpl.java 11939 2007-01-16 12:21:53Z peterit $
 */

public class VersionImpl extends VersionStorageDescendantNode implements Version {
  
  
  // new impl
  public VersionImpl(NodeData data, SessionImpl session)
      throws PathNotFoundException, RepositoryException {

    super(data, session);
    
    if (!this.isNodeType(Constants.NT_VERSION))
      throw new RepositoryException("Node " + getLocation().getAsString(true)
          + " is not nt:version type");
  }


  public VersionHistoryImpl getContainingHistory() throws RepositoryException {
    
    checkValid();
    
    VersionHistoryImpl vhistory = (VersionHistoryImpl) dataManager.getItem(getInternalPath().makeParentPath(), true);
    
    if (vhistory == null)
      throw new VersionException("Version history item is not found for version " + getPath());
    
    return vhistory;
  }

  public Calendar getCreated() throws RepositoryException {
    
    checkValid();
    
    InternalQPath createdPath = InternalQPath.makeChildPath(getData().getQPath(), Constants.JCR_CREATED);
    
    PropertyData pdata = (PropertyData) dataManager.getItemData(createdPath);
    
    if (pdata == null)
      throw new VersionException("jcr:created property is not found for version " + getPath());
    
    Value created = session.getValueFactory().loadValue((TransientValueData) pdata.getValues().get(0), pdata.getType());
    
    return created.getDate();
    
    //return getProperty("jcr:created").getDate();
  }

  /* (non-Javadoc)
   * @see javax.jcr.version.Version#getSuccessors()
   */
  public Version[] getSuccessors() throws RepositoryException {
    
    checkValid();
    
    InternalQPath successorsPath = InternalQPath.makeChildPath(getData().getQPath(), Constants.JCR_SUCCESSORS);
    
    PropertyData successorsData = (PropertyData) dataManager.getItemData(successorsPath);
    
    if (successorsData == null)
      return new Version[0];
    
    List<ValueData> successorsValues = successorsData.getValues();
    Version[] successors = new Version[successorsValues.size()];
    
    try {
      for (int i=0; i<successorsValues.size(); i++) {
        String vuuid = new String(successorsValues.get(i).getAsByteArray());
        VersionImpl version = (VersionImpl) dataManager.getItemByUUID(vuuid, false);
        if (version != null)
          successors[i] = version;
        else
          throw new RepositoryException("Successor version is not found " + vuuid + ", this version " + getPath());
      }
    } catch (IOException e) {
      throw new RepositoryException("Successor value read error " + e, e);
    }
    
    return successors;
    
//    Value[] refs = new Value[0];
//    try {
//      refs = getProperty("jcr:successors").getValues();
//    } catch (PathNotFoundException e) {
//    }
//    Version[] successors = new Version[refs.length];
//    for (int i = 0; i < refs.length; i++) {
//      successors[i] = (Version) session.getNodeByUUID(refs[i].getString());
//    }
//    return successors;
  }

  /* (non-Javadoc)
   * @see javax.jcr.version.Version#getPredecessors()
   */
  public Version[] getPredecessors() throws RepositoryException {
    
    checkValid();
    
    InternalQPath predecessorsPath = InternalQPath.makeChildPath(getData().getQPath(), Constants.JCR_PREDECESSORS);
    
    PropertyData predecessorsData = (PropertyData) dataManager.getItemData(predecessorsPath);
    
    if (predecessorsData == null)
      return new Version[0];
    
    List<ValueData> predecessorsValues = predecessorsData.getValues();
    Version[] predecessors = new Version[predecessorsValues.size()];
    
    try {
      for (int i=0; i<predecessorsValues.size(); i++) {
        String vuuid = new String(predecessorsValues.get(i).getAsByteArray());
        VersionImpl version = (VersionImpl) dataManager.getItemByUUID(vuuid, false);
        if (version != null)
          predecessors[i] = version;
        else
          throw new RepositoryException("Predecessor version is not found " + vuuid + ", this version " + getPath());
      }
    } catch (IOException e) {
      throw new RepositoryException("Predecessor value read error " + e, e);
    }
    
    return predecessors;
    
//    Value[] refs = new Value[0];
//    try {
//      refs = getProperty("jcr:predecessors").getValues();
//    } catch (PathNotFoundException e) {
//    }
//    Version[] ps = new Version[refs.length];
//    for (int i = 0; i < refs.length; i++) {
//      ps[i] = (Version) session.getNodeByUUID(refs[i].getString());
//    }
//    return ps;
  }

  public void addSuccessor(String successorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    ValueData successorRef = new TransientValueData(new Uuid(successorUuid));
    
    InternalQPath successorsPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_SUCCESSORS);
    TransientPropertyData successorsProp = (TransientPropertyData) dataManager.getItemData(successorsPath);
    
    if (successorsProp == null) {
      // create a property now
      List<ValueData> successors = new ArrayList<ValueData>(); 
      successors.add(successorRef);
      successorsProp = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_SUCCESSORS, PropertyType.REFERENCE, true, successors);
      changesLog.add(ItemState.createAddedState(successorsProp));
    } else {
      // add successor in existed one
      TransientPropertyData newSuccessorsProp = successorsProp.cloneWithData();
      newSuccessorsProp.getValues().add(successorRef);
      changesLog.add(ItemState.createUpdatedState(newSuccessorsProp));
    }


//    PropertyImpl successors = null;
//    try {
//      successors = (PropertyImpl) getProperty("jcr:successors");
//    } catch (PathNotFoundException e) {
//    }
//    Value successorRef = session.getValueFactory().createValue(successor);
//    if (successors != null) {
//      Value[] refs = successors.getValues();
//      Value[] successorsRefs = new Value[refs.length + 1];
//      for (int i = 0; i < refs.length; i++)
//        successorsRefs[i] = refs[i];
//      successorsRefs[refs.length] = successorRef;
//      successors = createChildProperty("jcr:successors", successorsRefs, PropertyType.REFERENCE);
//
//    } else {
//      Value[] refs = new Value[] {successorRef};
//      successors = createChildProperty("jcr:successors", refs, PropertyType.REFERENCE);
//    }
//    
//    // [PN] 07.04.06
//    session.getTransientNodesManager().getTransactManager().saveItem(successors);
    
//    throw new RepositoryException("addSuccessor()");
  }

  public void addPredecessor(String predeccessorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    
    ValueData predeccessorRef = new TransientValueData(new Uuid(predeccessorUuid));
    
    InternalQPath predeccessorssPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_PREDECESSORS);
    TransientPropertyData predeccessorsProp = (TransientPropertyData) dataManager.getItemData(predeccessorssPath);
    
    if (predeccessorsProp == null) {
      List<ValueData> predeccessors = new ArrayList<ValueData>(); 
      predeccessors.add(predeccessorRef);
      predeccessorsProp = TransientPropertyData.createPropertyData(
          nodeData(), Constants.JCR_PREDECESSORS, PropertyType.REFERENCE, true, predeccessors);
      changesLog.add(ItemState.createAddedState(predeccessorsProp));
    } else {
      // add successor in existed one
      TransientPropertyData newPredeccessorsProp = predeccessorsProp.cloneWithData();
      newPredeccessorsProp.getValues().add(predeccessorRef);
      changesLog.add(ItemState.createUpdatedState(newPredeccessorsProp));
    }
    
//  PropertyImpl predecessors = null;
//  try {
//    predecessors = (PropertyImpl) getProperty("jcr:predecessors");
//  } catch (PathNotFoundException e) {
//  }
//  Value predecessorRef = session.getValueFactory().createValue(predecessor);
//  if (predecessors != null) {
//    Value[] refs = predecessors.getValues();
//    Value[] successorsRefs = new Value[refs.length + 1];
//    for (int i = 0; i < refs.length; i++)
//      successorsRefs[i] = refs[i];
//    successorsRefs[refs.length] = predecessorRef;
//    predecessors = createChildProperty("jcr:predecessors", successorsRefs, PropertyType.REFERENCE);
//  } else {
//    Value[] refs = new Value[] { predecessorRef };
//    predecessors = createChildProperty("jcr:predecessors", refs, PropertyType.REFERENCE);
//  }
//
//  // [PN] 12.04.06
//  session.getTransientNodesManager().getTransactManager().saveItem(predecessors);
//    throw new RepositoryException("addPredecessor()");
  }
  
  void removeSuccessor(String successorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    InternalQPath successorsPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_SUCCESSORS);
    PropertyData successorsProp = (PropertyData) dataManager.getItemData(successorsPath);
    
    if (successorsProp != null) {
      List<ValueData> newSuccessors = new ArrayList<ValueData>();
      
      try {
        for (ValueData sdata: successorsProp.getValues()) {
          if (!successorUuid.equals(new String(sdata.getAsByteArray())))
            newSuccessors.add(sdata);
        }
      } catch (IOException e) {
        throw new RepositoryException("A jcr:successors property read error " + e, e);
      }
      
      TransientPropertyData newSuccessorsProp = new TransientPropertyData(
          InternalQPath.makeChildPath(nodeData().getQPath(), Constants.JCR_SUCCESSORS, successorsProp.getQPath().getIndex()),
          successorsProp.getUUID(),
          successorsProp.getPersistedVersion(), 
          PropertyType.REFERENCE,
          nodeData().getUUID(), 
          true);
      newSuccessorsProp.setValues(newSuccessors);
      changesLog.add(ItemState.createUpdatedState(newSuccessorsProp));
    } else {
      throw new RepositoryException("A jcr:successors property is not found, version " + getPath());
    }
  }
  
  void removeAddSuccessor(String removedSuccessorUuid, String addedSuccessorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    InternalQPath successorsPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_SUCCESSORS);
    PropertyData successorsProp = (PropertyData) dataManager.getItemData(successorsPath);
    
    if (successorsProp != null) {
      List<ValueData> newSuccessors = new ArrayList<ValueData>();
      
      try {
        for (ValueData sdata: successorsProp.getValues()) {
          if (!removedSuccessorUuid.equals(new String(sdata.getAsByteArray())))
            newSuccessors.add(sdata);
        }
      } catch (IOException e) {
        throw new RepositoryException("A jcr:successors property read error " + e, e);
      }

      newSuccessors.add(new TransientValueData(new Uuid(addedSuccessorUuid)));
      
      TransientPropertyData newSuccessorsProp = new TransientPropertyData(
          InternalQPath.makeChildPath(nodeData().getQPath(), Constants.JCR_SUCCESSORS, successorsProp.getQPath().getIndex()),
          successorsProp.getUUID(),
          successorsProp.getPersistedVersion(), 
          PropertyType.REFERENCE,
          nodeData().getUUID(), 
          true);
      newSuccessorsProp.setValues(newSuccessors);
      changesLog.add(ItemState.createUpdatedState(newSuccessorsProp));
    } else {
      throw new RepositoryException("A jcr:successors property is not found, version " + getPath());
    }
  }
  
  void removePredecessor(String predecessorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    InternalQPath predeccessorsPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_PREDECESSORS);
    PropertyData predeccessorsProp = (PropertyData) dataManager.getItemData(predeccessorsPath);
    
    if (predeccessorsProp != null) {
      List<ValueData> newPredeccessors = new ArrayList<ValueData>();
      
      try {
        for (ValueData sdata: predeccessorsProp.getValues()) {
          if (!predecessorUuid.equals(new String(sdata.getAsByteArray())))
            newPredeccessors.add(sdata);
        }
      } catch (IOException e) {
        throw new RepositoryException("A jcr:predecessors property read error " + e, e);
      }
      
      TransientPropertyData newPredecessorsProp = new TransientPropertyData(
          InternalQPath.makeChildPath(nodeData().getQPath(), Constants.JCR_PREDECESSORS, predeccessorsProp.getQPath().getIndex()),
          predeccessorsProp.getUUID(),
          predeccessorsProp.getPersistedVersion(), 
          PropertyType.REFERENCE,
          nodeData().getUUID(), 
          true);
      newPredecessorsProp.setValues(newPredeccessors);
      changesLog.add(ItemState.createUpdatedState(newPredecessorsProp));
    } else {
      throw new RepositoryException("A jcr:predecessors property is not found, version " + getPath());
    }
  }
  
  void removeAddPredecessor(String removedPredecessorUuid, String addedPredecessorUuid, MutableItemDataChangesLog changesLog) throws RepositoryException {
    InternalQPath predeccessorsPath = InternalQPath.makeChildPath(this.getInternalPath(), Constants.JCR_PREDECESSORS);
    PropertyData predeccessorsProp = (PropertyData) dataManager.getItemData(predeccessorsPath);
    
    if (predeccessorsProp != null) {
      List<ValueData> newPredeccessors = new ArrayList<ValueData>();
      
      try {
        for (ValueData sdata: predeccessorsProp.getValues()) {
          if (!removedPredecessorUuid.equals(new String(sdata.getAsByteArray())))
            newPredeccessors.add(sdata);
        }
      } catch (IOException e) {
        throw new RepositoryException("A jcr:predecessors property read error " + e, e);
      }
      
      newPredeccessors.add(new TransientValueData(new Uuid(addedPredecessorUuid)));
      
      TransientPropertyData newPredecessorsProp = new TransientPropertyData(
          InternalQPath.makeChildPath(nodeData().getQPath(), Constants.JCR_PREDECESSORS, predeccessorsProp.getQPath().getIndex()),
          predeccessorsProp.getUUID(),
          predeccessorsProp.getPersistedVersion(), 
          PropertyType.REFERENCE,
          nodeData().getUUID(), 
          true);
      newPredecessorsProp.setValues(newPredeccessors);
      changesLog.add(ItemState.createUpdatedState(newPredecessorsProp));
    } else {
      throw new RepositoryException("A jcr:predecessors property is not found, version " + getPath());
    }
  }
    
  public SessionChangesLog restoreLog(NodeData nodeData, VersionHistoryDataHelper historyData, 
      SessionImpl restoreSession, boolean removeExisting, SessionChangesLog delegatedLog) throws RepositoryException {

    if (log.isDebugEnabled())
      log.debug("Restore: " + nodeData.getQPath().getAsString() + ", removeExisting=" + removeExisting);
    
    DataManager dmanager = restoreSession.getTransientNodesManager().getTransactManager();
    
    NodeData parentData = (NodeData) dmanager.getItemData(nodeData.getParentUUID());
    
    InternalQPath frozenPath = InternalQPath.makeChildPath(getData().getQPath(), Constants.JCR_FROZENNODE);
    NodeData frozenData = (NodeData) dmanager.getItemData(frozenPath);
    
    ItemDataRestoreVisitor restoreVisitor = new ItemDataRestoreVisitor(
        parentData, 
        nodeData.getQPath().getName(),
        historyData, 
        restoreSession, 
        removeExisting,
        delegatedLog);
    
    frozenData.accept(restoreVisitor);
    
    return restoreVisitor.getRestoreChanges();
  }
  
  public void restore(NodeImpl node, boolean removeExisting) throws RepositoryException {

    // use restored node session
    SessionImpl restoreSession = node.getSession();
    NodeData nodeData = (NodeData) node.getData();
    VersionHistoryDataHelper historyData = node.getVersionHistory().getData();
    
    SessionChangesLog changesLog = restoreLog(nodeData, historyData, restoreSession, removeExisting, null);
    
    //log.info(changesLog.dump());
    
    restoreSession.getTransientNodesManager().getTransactManager().save(changesLog);
    
    
    // [PN] 19.12.06 not used
    //return (NodeImpl) dataManager.getItemByUUID(node.getInternalUUID(), true);
      
//    NodeImpl frozenNode = (NodeImpl) getNode("jcr:frozenNode");
//
//    SessionImpl nodeSession = (SessionImpl) node.getSession();
//    SessionDataManager sdm = nodeSession.getTransientNodesManager();
//    WorkspaceStorageDataManagerProxy wdm = sdm.getWorkspaceDataManager();
//    
//    if (!removeExisting) {
//      // [PN] 13.04.06 fast check first
//      try {
//        // same UUID node
//        NodeImpl sameUUIDNode = (NodeImpl) nodeSession.getNodeByUUID(frozenNode.getProperty("jcr:frozenUuid").getString());
//        if (!sameUUIDNode.getLocation().equals(node.getLocation())) {
//          throw new ItemExistsException("Item with the same UUID as restored node "+node.getPath()
//              + " already exists and removeExisting = false. Existed "+sameUUIDNode.getPath() + " " + sameUUIDNode.getInternalUUID());
//        }
//      } catch(ItemNotFoundException e) {
//        // hasn't same uuid node
//      }
//    }
//    
//    // make new node from frozen
//
//    // Manually reloading data to actual state,
//    // force lazy loading of childs - they will be used in previous node
//    // removing (inside WDM)
//    node.setRefreshToPersistedVersion(true);
//    List<NodeImpl> nodeChilds = node.loadChildNodes();
//    node.loadChildProperties();
//    node.setRefreshToPersistedVersion(false);
//
//    // get an actual data of the target node and clean all childs in data,
//    // we will load them in visitor regarding the frozen node childs
//    TransientNodeData nodeData = (TransientNodeData) node.getActualItemData();
//    nodeData.getChildNodes().clear();
//    nodeData.getChildProperties().clear();
//
//    NodeImpl nodeParent = (NodeImpl) node.getParent();
//    InternalQName nodeName = nodeData.getQName();
//
//    String ptNameStr = frozenNode.getProperty("jcr:frozenPrimaryType").getString();
//    InternalQName ptName = ((NameValue) frozenNode.getProperty("jcr:frozenPrimaryType").getValue()).getQName();
//    InternalQName[] mtNames = {};
//    Value[] mtValues = null;
//    if (frozenNode.hasProperty("jcr:frozenMixinTypes")) {
//      mtValues = frozenNode.getProperty("jcr:frozenMixinTypes").getValues();
//      mtNames = new InternalQName[mtValues.length];
//      for (int i = 0; i < mtNames.length; i++)
//        mtNames[i] = ((NameValue) mtValues[i]).getQName();
//    }
//    String uuid = null;
//    if (frozenNode.hasProperty("jcr:frozenUuid")) {
//      uuid = frozenNode.getProperty("jcr:frozenUuid").getString();
//    }
//
//    // [PN] 06.05.06 TODO It's same as node but with status DELETED. Fix it in the refactoring.
//    NodeImpl removedNode = (NodeImpl) nodeParent.getNode(node.getName());
//    
//    NodeImpl restoredNode = nodeParent.createChildNodeInmemory(nodeName, ptName, mtNames, uuid,
//        node.getIndex(), false, false, false);
//    restoredNode.loadData(nodeData);
//
//    // copy childs/properties with OnParentVersionAction.IGNORE to the restored node
//    NodeCopyIgnoredVisitor copyIgnoredVisitor = new NodeCopyIgnoredVisitor(restoredNode, true, false);
//    node.accept(copyIgnoredVisitor);
//
//    try {
//      // ----------- BEGIN SESSION BATCH ----------    
//      wdm.beginSessionBatch(nodeSession);
//      
//      // visitor accept frozenNode traverse and save state in WorkspaceDataManager
//      RestoreVisitor visitor = new RestoreVisitor(restoredNode, removeExisting);
//      frozenNode.accept(visitor);
//  
//      // [PN] 03.08.06 Using InternalQName...
//      restoredNode.createChildPropertyInmemory(
//          Constants.JCR_PRIMARYTYPE,
//          PropertyType.NAME, 
//          new Value[] { session.getValueFactory().createValue(ptNameStr, PropertyType.NAME) },
//          false,  // registerInDataManager 
//          false); // doExternalValidation
//      //restoredNode.createChildProperty("jcr:primaryType", new Value[] { session.getValueFactory().createValue(ptNameStr, PropertyType.NAME) }, PropertyType.NAME);
//      
//      restoredNode.createChildPropertyInmemory(
//          Constants.JCR_UUID,
//          PropertyType.STRING, 
//          new Value[] { session.getValueFactory().createValue(uuid) },
//          false,  // registerInDataManager 
//          false); // doExternalValidation
//      //restoredNode.createChildProperty("jcr:uuid", new Value[] { session.getValueFactory().createValue(uuid) }, PropertyType.REFERENCE); // STRING --> REFERENCE
//      
//      if (mtValues != null)
//        //restoredNode.createChildProperty("jcr:mixinTypes", mtValues, PropertyType.NAME);
//        restoredNode.createChildPropertyInmemory(
//          Constants.JCR_MIXINTYPES,
//          PropertyType.NAME, 
//          mtValues,
//          false,  // registerInDataManager 
//          false); // doExternalValidation
//  
//      restoredNode.createChildPropertyInmemory(
//          Constants.JCR_BASEVERSION,
//          PropertyType.REFERENCE, 
//          new Value[] { session.getValueFactory().createValue(this) },
//          false,  // registerInDataManager 
//          false); // doExternalValidation
//      //restoredNode.createChildProperty("jcr:baseVersion", new Value[] { session.getValueFactory().createValue(this) }, PropertyType.REFERENCE);
//      
//      restoredNode.createChildPropertyInmemory(
//          Constants.JCR_ISCHECKEDOUT,
//          PropertyType.BOOLEAN, 
//          new Value[] { session.getValueFactory().createValue(false) },
//          false,  // registerInDataManager 
//          false); // doExternalValidation
//      //restoredNode.createChildProperty("jcr:isCheckedOut", new Value[] { session.getValueFactory().createValue(false) }, PropertyType.BOOLEAN);
//      
//      sdm.getTransactManager().saveItem(removedNode); // delete
//      // [PN] 04.07.06 For restored node we disabling validation (Caused by restore 'nt:file' error)
//      sdm.getTransactManager().saveItem(restoredNode, false); // add
//      
//      wdm.endSessionBatch(nodeSession);
//      //----------- END SESSION BATCH ----------
//      
//      sdm.resetItemState(restoredNode);
//      
//      sdm.reloadCache(restoredNode.getLocation(), true);
//     
//    
//    } catch(ItemExistsException e) {
//      // rollback some changes
//      nodeParent.loadData(wdm.getItemData(nodeParent.getInternalUUID())); // rollback node's parent to stored state
//      throw e;
//    }
//    return restoredNode;
    
    //throw new RepositoryException("restore()");
  }
  
  public boolean isSuccessorOrSameOf(VersionImpl anotherVersion) 
  throws RepositoryException {
    Version[] prds = getPredecessors();
    for(int i=0; i<prds.length; i++) {
      if(prds[i].getUUID().equals(anotherVersion.getUUID()) ||
         ((VersionImpl)prds[i]).isSuccessorOrSameOf(anotherVersion))
        return true;
    }
    return false;
  }
 
}