/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.xml;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeType;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.xml.sax.ContentHandler;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 22.09.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: ImporteBase.java 9060 2006-09-22 22:11:39Z peterit $
 */
abstract public class ImporteBase implements ContentHandler {

  protected int uuidBehavior = -1;
    
  protected SessionImpl       session;
  @Deprecated
  protected NodeImpl          rootNode;
  /**
   * The NodeTypeManager
   */
  protected NodeTypeManagerImpl ntManager;
  
  /**
   * The list of added item states
   */
  protected List<ItemState> itemStatesList;

  protected LocationFactory locationFactory;

  ImporteBase(NodeImpl parent, int uuidBehavior) {
  
    this.session = (SessionImpl) parent.getSession();
    this.ntManager = (NodeTypeManagerImpl) ((RepositoryImpl) this.session.getRepository()).getNodeTypeManager();
    locationFactory = session.getLocationFactory();
    this.uuidBehavior = uuidBehavior;
    itemStatesList = new ArrayList<ItemState>();
  }
  
  protected PropertyDefinition getPropertyDefinition(String propertyName, List<ExtendedNodeType> nodeTypes) {
    PropertyDefinition pdResidual = null;
    for (ExtendedNodeType nt: nodeTypes){
      PropertyDefinitions pds = nt.getPropertyDefinitions(propertyName);
      PropertyDefinition pd = pds.getAnyDefinition();
      if ( pd != null) {
        if (((PropertyDefinitionImpl)pd).isResidualSet()) {
          pdResidual = pd;
        } else {
          return pd;
        }
      }
    }
    return pdResidual;
  }
  
  protected boolean isReferenceable(List<ExtendedNodeType> nodeTypes) {
    
    for (ExtendedNodeType nt: nodeTypes){
      if (nt.isNodeType(Constants.MIX_REFERENCEABLE)) {
        return true;
      }
    }
    return false;
  }
  public List<ItemState> getItemStatesList() {
    return itemStatesList;
  }
  public void setItemStatesList(List<ItemState> itemStatesList) {
    this.itemStatesList = itemStatesList;
  }
 
}