/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.xml;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.List;

import javax.jcr.NamespaceException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.codec.binary.Base64;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.dataflow.ItemDataConsumer;
import org.exoplatform.services.jcr.dataflow.ItemDataTraversingVisitor;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;
import org.exoplatform.services.jcr.util.io.BLOBUtil;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * @author <a href="mailto:Sergey.Kabashnyuk@gmail.com">Sergey Kabashnyuk</a>
 * @version
 */
public class SysExportXmlVisior extends ExportXmlVisitor {


  private final String    SV_NAMESPACE_URI;

  private ContentHandler  contentHandler;


  /**
   * @param handler
   * @param session
   * @param dataManager
   * @param noRecurse
   * @throws NamespaceException
   * @throws RepositoryException
   */
  public SysExportXmlVisior(ContentHandler handler, Session session, ItemDataConsumer dataManager,
      boolean skipBinary, boolean noRecurse) throws NamespaceException, RepositoryException {
    super(handler, session,dataManager, skipBinary, noRecurse);

    if (handler == null)
      throw new java.lang.IllegalArgumentException("ContentHandler can't be null");
    this.contentHandler = handler;
    if (session == null)
      throw new java.lang.IllegalArgumentException("session can't be null");

    SV_NAMESPACE_URI = session.getNamespaceURI("sv");
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.dataflow.ItemDataTraversingVisitor#entering(org.exoplatform.services.jcr.datamodel.PropertyData,
   *      int)
   */
  @Override
  protected void entering(PropertyData property, int level) throws RepositoryException {
    try {
      // set name and type of property
      AttributesImpl atts = new AttributesImpl();
      atts.addAttribute(SV_NAMESPACE_URI, "name", "sv:name", "CDATA", locationFactory
          .createJCRName(property.getQPath().getName()).getAsString());
      atts.addAttribute(SV_NAMESPACE_URI, "type", "sv:type", "CDATA", ExtendedPropertyType
          .nameFromValue(property.getType()));

      contentHandler.startElement(SV_NAMESPACE_URI, "property", "sv:property", atts);
      List<ValueData> values = property.getValues();
      for (ValueData valueData : values) {
        contentHandler.startElement(SV_NAMESPACE_URI, "value", "sv:value", new AttributesImpl());

        char[] charValue = "".toCharArray();
        switch (property.getType()) {
        case PropertyType.BINARY:
          if (getBinaryConduct() == BINARY_SKIP)
            break;
          else if (getBinaryConduct() == BINARY_EMPTY) {
            try {
              charValue = new BinaryValue("").getString().toCharArray();
            } catch (IOException e) {
              throw new RepositoryException(e);
            }
          } else {
            charValue = new String(Base64.encodeBase64(BLOBUtil.readValue(valueData)))
                .toCharArray();
          }

          break;

        case PropertyType.NAME:
        case PropertyType.DATE:
        case PropertyType.PATH:
          charValue = ((ValueFactoryImpl) session.getValueFactory()).loadValue(
              (TransientValueData) valueData, property.getType()).getString().toCharArray();
          break;
        default:
          charValue = new String(valueData.getAsByteArray()).toCharArray();
          break;
        }

        contentHandler.characters(charValue, 0, charValue.length);
        contentHandler.endElement(SV_NAMESPACE_URI, "value", "sv:value");
      }
    } catch (SAXException e) {
      throw new RepositoryException(e);
    } catch (IOException e) {
      throw new RepositoryException("Can't export value to string: " + e.getMessage(), e);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.dataflow.ItemDataTraversingVisitor#entering(org.exoplatform.services.jcr.datamodel.NodeData,
   *      int)
   */
  @Override
  protected void entering(NodeData node, int level) throws RepositoryException {
    try {
      // set name of node as sv:name attribute
      AttributesImpl atts = new AttributesImpl();
      atts.addAttribute(SV_NAMESPACE_URI, "name", "sv:name", "CDATA", locationFactory
          .createJCRName(node.getQPath().getName()).getAsString());

      contentHandler.startElement(SV_NAMESPACE_URI, "node", "sv:node", atts);
    } catch (SAXException e) {
      throw new RepositoryException(e);
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.dataflow.ItemDataTraversingVisitor#leaving(org.exoplatform.services.jcr.datamodel.PropertyData,
   *      int)
   */
  @Override
  protected void leaving(PropertyData property, int level) throws RepositoryException {
    try {
      contentHandler.endElement(SV_NAMESPACE_URI, "property", "sv:property");
    } catch (SAXException e) {
      throw new RepositoryException(e);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.dataflow.ItemDataTraversingVisitor#leaving(org.exoplatform.services.jcr.datamodel.NodeData,
   *      int)
   */
  @Override
  protected void leaving(NodeData node, int level) throws RepositoryException {

    try {
      contentHandler.endElement(SV_NAMESPACE_URI, "node", "sv:node");
    } catch (SAXException e) {
      throw new RepositoryException(e);
    }

  }

  /**
   * Return the current content handler
   */
  public ContentHandler getContentHandler() {
    return contentHandler;
  }

  /**
   * Allow an application to register a content event handler. If the
   * application does not register a content handler, all content events
   * reported by the SAX parser will be silently ignored. Applications may
   * register a new or different handler in the middle of a parse, and the SAX
   * parser must begin using the new handler immediately.
   * 
   * @param handler ContentHandler The content handler
   * @throws NullPointerException If the handler argument is null.
   */
  public void setContentHandler(ContentHandler handler) {
    if (handler == null)
      throw new java.lang.IllegalArgumentException("ContentHandler can't be null");
    // this.customerContentHandler = handler;
    this.contentHandler = handler;// new ContentHandlerWrapper(handler);
  }
}
