/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.usecases.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ObjectParameter;
import org.exoplatform.services.command.action.ActionMatcher;
import org.exoplatform.services.command.action.Condition;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.ext.action.ActionConfiguration;
import org.exoplatform.services.jcr.impl.ext.action.AddActionsPlugin;
import org.exoplatform.services.jcr.impl.ext.action.SessionActionCatalog;
import org.exoplatform.services.jcr.impl.ext.action.SessionEventMatcher;
import org.exoplatform.services.jcr.impl.ext.action.AddActionsPlugin.ActionsConfig;
import org.exoplatform.services.jcr.observation.ExtendedEvent;
import org.exoplatform.services.jcr.usecases.BaseUsecasesTest;

public class TestSessionActionCatalog extends BaseUsecasesTest {

  public void testIfServicePresent() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);

    assertNotNull("No SessionActionCatalog configured!", catalog);
    Iterator <ActionMatcher> matchers = catalog.getAllEntries().keySet().iterator();
    System.out.println(">>testIfServicePresent");
    while(matchers.hasNext()) {
      System.out.println(((SessionEventMatcher)matchers.next()).dump());
    }

  }
  
  public void testPluginConfiguration() throws Exception {
    ActionConfiguration ac = new ActionConfiguration(
        "org.exoplatform.services.jcr.usecases.action.DummyAction", 
        "addNode,addProperty", "/test,/exo:test1", true, 
        "nt:base,", "nt:base", null);
    List actionsList = new ArrayList();
    ActionsConfig actions = new ActionsConfig();
    actions.setActions(actionsList);
    actionsList.add(ac);
    InitParams params = new InitParams();
    ObjectParameter op = new ObjectParameter();
    op.setObject(actions);
    op.setName("actions");
    params.addParameter(op);
    AddActionsPlugin aap = new AddActionsPlugin(params);
    
    SessionActionCatalog catalog = (SessionActionCatalog)
      container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();

    catalog.addPlugin(aap);
    assertEquals(1, aap.getActions().size());
    assertEquals(1, catalog.getAllActions().size());
    ActionConfiguration ac1 = (ActionConfiguration)aap.getActions().get(0);
    assertEquals("org.exoplatform.services.jcr.usecases.action.DummyAction", ac1.getActionClassName());
    assertEquals("/test,/exo:test1", ac1.getPath());
    
    Condition cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);

    assertEquals(1, catalog.getActions(cond).size());
    
    Iterator <ActionMatcher> matchers = catalog.getAllEntries().keySet().iterator();
    
    while(matchers.hasNext()) {
      System.out.println(((SessionEventMatcher)matchers.next()).dump());
    }
  }

  
  public void testMatchEventType() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();
    assertEquals(0, catalog.getAllActions().size());


    //((NodeTypeImpl)node.getPrimaryNodeType()).getQName()
    //node.getInternalPath()
//  cond.put(SessionEventMatcher.PATH_KEY, node.getInternalPath());
//  cond.put(SessionEventMatcher.NODETYPE_KEY, ((NodeTypeImpl)node.getPrimaryNodeType()).getQName());
    
    // test by event type
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, null, 
        true, null, null, null);
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);
    assertEquals(1, catalog.getActions(cond).size());
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.ADD_MIXIN);
    assertEquals(0, catalog.getActions(cond).size());
    
  }
  
  public void testMatchDeepPath() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();
    NodeImpl node = (NodeImpl)root.addNode("test");


    // test by path 
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, new InternalQPath[] {node.getInternalPath()}, 
        true, null, null, null);
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();

    cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);
    cond.put(SessionEventMatcher.PATH_KEY, node.getInternalPath());
    
    // test for this path
    assertEquals(1, catalog.getActions(cond).size());
    
    cond.put(SessionEventMatcher.PATH_KEY, Constants.ROOT_PATH);
    assertEquals(0, catalog.getActions(cond).size());
    
    // test for child path
    InternalQPath child = InternalQPath.makeChildPath(node.getInternalPath(), Constants.EXO_PREFIX); 
    cond.put(SessionEventMatcher.PATH_KEY, child);
    assertEquals(1, catalog.getActions(cond).size());

    // test for grandchild path - found as deep = true
    InternalQPath grandchild = InternalQPath.makeChildPath(child, Constants.EXO_PREFIX); 
    cond.put(SessionEventMatcher.PATH_KEY, grandchild);
    assertEquals(1, catalog.getActions(cond).size());

  }
  
  public void testMatchNotDeepPath() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();
    NodeImpl node = (NodeImpl)root.addNode("test");


    // test by path 
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, new InternalQPath[] {((NodeImpl)root).getInternalPath()}, 
        false, null, null, null);
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();

    cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);
    
    // test for this path
    cond.put(SessionEventMatcher.PATH_KEY, Constants.ROOT_PATH);
    assertEquals(1, catalog.getActions(cond).size());
    
    // test for child path
    cond.put(SessionEventMatcher.PATH_KEY, node.getInternalPath());
    assertEquals(1, catalog.getActions(cond).size());

    // test for grandchild path - not found as deep = false
    InternalQPath child = InternalQPath.makeChildPath(node.getInternalPath(), Constants.EXO_PREFIX); 
    cond.put(SessionEventMatcher.PATH_KEY, child);
    assertEquals(0, catalog.getActions(cond).size());

  }

  
  public void testMatchNodeType() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();

    // test by path 
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, null, 
        true, new InternalQName[] {Constants.NT_UNSTRUCTURED}, null, null);
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);

    // test for this nodetype
    cond.put(SessionEventMatcher.NODETYPE_KEY, Constants.NT_UNSTRUCTURED);
    assertEquals(1, catalog.getActions(cond).size());
    
    cond.put(SessionEventMatcher.NODETYPE_KEY, Constants.NT_NODETYPE);
    assertEquals(0, catalog.getActions(cond).size());
  }
  
  public void testMatchParentNodeType() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();

    // test by path 
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, null, 
        true, null, new InternalQName[] {Constants.NT_UNSTRUCTURED}, null);
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);

    // test for this nodetype
    cond.put(SessionEventMatcher.PARENT_NODETYPE_KEY, 
        Constants.NT_UNSTRUCTURED);
    assertEquals(1, catalog.getActions(cond).size());
    
    cond.put(SessionEventMatcher.PARENT_NODETYPE_KEY, 
        Constants.NT_NODETYPE);
    assertEquals(0, catalog.getActions(cond).size());

  }

  public void testMatchWorkspace() throws Exception {
    SessionActionCatalog catalog = (SessionActionCatalog)
    container.getComponentInstanceOfType(SessionActionCatalog.class);
    catalog.clear();

    // 
    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, null, 
        true, null, null, new String[]{"production"});
    catalog.addAction(matcher, new DummyAction());
    Condition cond = new Condition();
    cond.put(SessionEventMatcher.EVENTTYPE_KEY, ExtendedEvent.NODE_ADDED);

    // test for this ws
    cond.put(SessionEventMatcher.WORKSPACE_KEY, 
        "production");
    assertEquals(1, catalog.getActions(cond).size());
    
    cond.put(SessionEventMatcher.WORKSPACE_KEY, 
        "draft");
    assertEquals(0, catalog.getActions(cond).size());

  }

  public void testDumpMatcher() throws Exception {
    NodeImpl node = (NodeImpl)root.addNode("test");
    NodeImpl node1 = (NodeImpl)root.addNode("test1");
    InternalQPath[] paths = new InternalQPath[] {
        node.getInternalPath(), node1.getInternalPath()};
    

    SessionEventMatcher matcher = new SessionEventMatcher(
        ExtendedEvent.NODE_ADDED, paths, 
        true, 
        new InternalQName[] {Constants.NT_BASE, Constants.NT_QUERY}, 
        new InternalQName[] {Constants.NT_UNSTRUCTURED, Constants.NT_QUERY},
        null);
    System.out.println(matcher.dump());
  }
  
}
