/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.persistent;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.jcr.JcrImplBaseTest;
import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedNodeData;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedPropertyData;
import org.exoplatform.services.jcr.dataflow.persistent.WorkspaceStorageCache;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.util.UUIDGenerator;

public class TestWorkspaceStorageCache extends JcrImplBaseTest {

  private InternalQPath nodePath1 = InternalQPath.makeChildPath(Constants.ROOT_PATH, new InternalQName(Constants.NS_EXO_PREFIX,"node 1"));
  private InternalQPath nodePath2 = InternalQPath.makeChildPath(Constants.ROOT_PATH, new InternalQName(Constants.NS_EXO_PREFIX,"node 2"));
  private InternalQPath nodePath3 = InternalQPath.makeChildPath(Constants.ROOT_PATH, new InternalQName(Constants.NS_EXO_PREFIX,"node 3"));
  
  private InternalQPath nodePath31 = InternalQPath.makeChildPath(nodePath3, new InternalQName(Constants.NS_EXO_PREFIX,"node 3.1"));
  private InternalQPath nodePath32 = InternalQPath.makeChildPath(nodePath3, new InternalQName(Constants.NS_EXO_PREFIX,"node 3.2"));
  
  private InternalQPath propertyPath11 = InternalQPath.makeChildPath(nodePath1, new InternalQName(Constants.NS_EXO_PREFIX,"property 1.1"));
  private InternalQPath propertyPath12 = InternalQPath.makeChildPath(nodePath1, new InternalQName(Constants.NS_EXO_PREFIX,"property 1.2"));
  private InternalQPath propertyPath21 = InternalQPath.makeChildPath(nodePath2, new InternalQName(Constants.NS_EXO_PREFIX,"property 2.1"));
  private InternalQPath propertyPath22 = InternalQPath.makeChildPath(nodePath2, new InternalQName(Constants.NS_EXO_PREFIX,"property 2.2"));
  
  private InternalQPath propertyPath311 = InternalQPath.makeChildPath(nodePath31, new InternalQName(Constants.NS_EXO_PREFIX,"property 3.1.1"));
  private InternalQPath propertyPath312 = InternalQPath.makeChildPath(nodePath31, new InternalQName(Constants.NS_EXO_PREFIX,"property 3.1.2"));
  
  private String rootUuid;
  
  private String nodeUuid1;
  private String nodeUuid2;
  private String nodeUuid3;
  private String nodeUuid31;
  private String nodeUuid32;
  
  private String propertyUuid11;
  private String propertyUuid12;
  private String propertyUuid21;
  private String propertyUuid22;
  
  private String propertyUuid311;
  private String propertyUuid312;
  
  private NodeData nodeData1;
  private NodeData nodeData2;
  private NodeData nodeData3;
  private NodeData nodeData31;
  private NodeData nodeData32;
  
  private PropertyData propertyData11;
  private PropertyData propertyData12;
  private PropertyData propertyData21;
  private PropertyData propertyData22;
  private PropertyData propertyData311;
  private PropertyData propertyData312;
  
  private WorkspaceStorageCacheImpl cache;
  
  public void setUp() throws Exception {
    super.setUp();
    
    rootUuid = UUIDGenerator.generate();
    
    nodeUuid1 = UUIDGenerator.generate();
    nodeUuid2 = UUIDGenerator.generate();
    nodeUuid3 = UUIDGenerator.generate();
    nodeUuid31 = UUIDGenerator.generate();
    nodeUuid32 = UUIDGenerator.generate();
    
    propertyUuid11 = UUIDGenerator.generate();
    propertyUuid12 = UUIDGenerator.generate();
    propertyUuid21 = UUIDGenerator.generate();
    propertyUuid22 = UUIDGenerator.generate();
    
    propertyUuid311 = UUIDGenerator.generate();
    propertyUuid312 = UUIDGenerator.generate();
    
    WorkspaceStorageCache cacheProbe = (WorkspaceStorageCacheImpl) session.getContainer().getComponentInstanceOfType(WorkspaceStorageCacheImpl.class);
    assertNotNull("Cache is unaccessible (check access denied or configuration)", cacheProbe);
    assertTrue("Cache is disabled", cacheProbe.isEnabled());
    
    // new instance
    cache = new WorkspaceStorageCacheImpl(
        (CacheService) session.getContainer().getComponentInstanceOfType(CacheService.class),
        (WorkspaceEntry) session.getContainer().getComponentInstanceOfType(WorkspaceEntry.class)
        );
    assertNotNull("Cache is disabled (test cache)", cache);
  }
  
  private void initNodesData() {
    nodeData1 = new PersistedNodeData(nodeUuid1, nodePath1, rootUuid, 1, 0, Constants.NT_UNSTRUCTURED, null, new AccessControlList(true));
    nodeData2 = new PersistedNodeData(nodeUuid2, nodePath2, rootUuid, 1, 1, Constants.NT_UNSTRUCTURED, null, new AccessControlList(true));
    nodeData3 = new PersistedNodeData(nodeUuid3, nodePath3, rootUuid, 1, 2, Constants.NT_UNSTRUCTURED, null, new AccessControlList(true));
    nodeData31 = new PersistedNodeData(nodeUuid31, nodePath31, nodeUuid3, 1, 0, Constants.NT_UNSTRUCTURED, null, new AccessControlList(true));
    nodeData32 = new PersistedNodeData(nodeUuid32, nodePath32, nodeUuid3, 1, 1, Constants.NT_UNSTRUCTURED, null, new AccessControlList(true));
    
    //nodeData31 = new TransientNodeData(nodePath31, nodeUuid31, 1, Constants.NT_UNSTRUCTURED, null, 1, nodeUuid3, new AccessControlList(true));
  }  
  
  private void initDataAsPersisted() {
    propertyData11 = new PersistedPropertyData(propertyUuid11, propertyPath11, nodeUuid1, 1, PropertyType.STRING, false);
    List<ValueData> stringData = new ArrayList<ValueData>(); 
    stringData.add(new ByteArrayPersistedValueData("property data 1".getBytes(), 0));
    stringData.add(new ByteArrayPersistedValueData("property data 2".getBytes(), 1));
    stringData.add(new ByteArrayPersistedValueData("property data 3".getBytes(), 2));
    try {
      ((PersistedPropertyData) propertyData11).setValues(stringData);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    propertyData12 = new PersistedPropertyData(propertyUuid12, propertyPath12, nodeUuid1, 1, PropertyType.BINARY, false);
    List<ValueData> binData = new ArrayList<ValueData>(); 
    binData.add(new ByteArrayPersistedValueData("property data bin 1".getBytes(), 0));
    try {
      ((PersistedPropertyData) propertyData12).setValues(binData);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    propertyData21 = new PersistedPropertyData(propertyUuid21, propertyPath21, nodeUuid2, 1, PropertyType.STRING, true);
    List<ValueData> stringData1 = new ArrayList<ValueData>(); 
    stringData1.add(new ByteArrayPersistedValueData("property data 1".getBytes(), 0));
    stringData1.add(new ByteArrayPersistedValueData("property data 2".getBytes(), 1));
    stringData1.add(new ByteArrayPersistedValueData("property data 3".getBytes(), 2));
    try {
      ((PersistedPropertyData) propertyData21).setValues(stringData1);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    propertyData22 = new PersistedPropertyData(propertyUuid22, propertyPath22, nodeUuid2, 1, PropertyType.BOOLEAN, false);
    List<ValueData> booleanData = new ArrayList<ValueData>(); 
    booleanData.add(new ByteArrayPersistedValueData("true".getBytes(), 0));
    try {
      ((PersistedPropertyData) propertyData22).setValues(booleanData);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    propertyData311 = new PersistedPropertyData(propertyUuid311, propertyPath311, nodeUuid31, 1, PropertyType.LONG, false);
    List<ValueData> longData = new ArrayList<ValueData>(); 
    longData.add(new ByteArrayPersistedValueData(new Long(123456).toString().getBytes(), 0));
    try {
      ((PersistedPropertyData) propertyData311).setValues(longData);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    propertyData312 = new PersistedPropertyData(propertyUuid312, propertyPath312, nodeUuid31, 1, PropertyType.REFERENCE, true);
    List<ValueData> refData = new ArrayList<ValueData>(); 
    refData.add(new ByteArrayPersistedValueData(nodeUuid1.getBytes(), 0));
    refData.add(new ByteArrayPersistedValueData(nodeUuid2.getBytes(), 1));
    refData.add(new ByteArrayPersistedValueData(nodeUuid3.getBytes(), 2));
    try {
      ((PersistedPropertyData) propertyData312).setValues(refData);
    } catch (RepositoryException e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
  }

  public void testGetItem_Persisted() throws Exception {

    initNodesData();
    initDataAsPersisted();
    
    cache.put(nodeData1);
    cache.put(nodeData2);
    cache.put(propertyData12);
    
    assertEquals("Cached node " + nodeData1.getQPath().getAsString() + " is not equals", cache.get(nodePath1), nodeData1);
    assertEquals("Cached node " + nodeData2.getQPath().getAsString() + " is not equals", cache.get(nodePath2), nodeData2);
    
    assertEquals("Cached node " + nodeData1.getUUID() + " is not equals", cache.get(nodeUuid1), nodeData1);
    assertEquals("Cached node " + nodeData2.getUUID() + " is not equals", cache.get(nodeUuid2), nodeData2);
    
    assertEquals("Cached property " + propertyData12.getQPath().getAsString() + " is not equals", cache.get(propertyPath12), propertyData12);
    assertEquals("Cached property " + propertyData12.getUUID() + " is not equals", cache.get(propertyUuid12), propertyData12);
  }
  
  public void testGetItems_Persisted() throws Exception {

    initNodesData();
    initDataAsPersisted();
    
    List<NodeData> nodes = new ArrayList<NodeData>();
    nodes.add(nodeData31);
    nodes.add(nodeData32);
    cache.addChildNodes(nodeData3, nodes);
    
    cache.put(nodeData1);
    cache.put(nodeData2);
    cache.put(propertyData12);
    
    List<PropertyData> properties2 = new ArrayList<PropertyData>();
    properties2.add(propertyData21);
    properties2.add(propertyData22);
    cache.addChildProperties(nodeData2, properties2);
    
    List<PropertyData> properties1 = new ArrayList<PropertyData>();
    properties1.add(propertyData11);
    properties1.add(propertyData12);
    cache.addChildProperties(nodeData1, properties1);
    
    // prev stuff
    assertEquals("Cached " + nodeData1.getQPath().getAsString() + " is not equals", cache.get(nodePath1), nodeData1);
    assertEquals("Cached " + nodeData2.getQPath().getAsString() + " is not equals", cache.get(nodePath2), nodeData2);
    assertEquals("Cached " + propertyData12.getQPath().getAsString() + " is not equals", cache.get(propertyPath12), propertyData12);
    
    // childs...
    // nodes
    assertEquals("Cached child node " + nodeData31.getQPath().getAsString() + " is not equals", cache.get(nodePath31), nodeData31);
    assertEquals("Cached child node " + nodeData31.getUUID() + " is not equals", cache.get(nodeUuid31), nodeData31);
    
    assertEquals("Cached child node " + nodeData32.getQPath().getAsString() + " is not equals", cache.get(nodePath32), nodeData32);
    assertEquals("Cached child node " + nodeData32.getUUID() + " is not equals", cache.get(nodeUuid32), nodeData32);
    
    assertTrue("Cached child node " + nodeData31.getQPath().getAsString() + " is not in the childs list", cache.getChildNodes(nodeData3).contains(nodeData31));
    assertTrue("Cached child node " + nodeData32.getQPath().getAsString() + " is not in the childs list", cache.getChildNodes(nodeData3).contains(nodeData32));
    
    assertEquals("Cached child nodes count is wrong", cache.getChildNodes(nodeData3).size(), 2);
    
    // props
    assertEquals("Cached child property " + propertyData11.getQPath().getAsString() + " is not equals", cache.get(propertyPath11), propertyData11);
    assertEquals("Cached child property " + propertyData11.getUUID() + " is not equals", cache.get(propertyUuid11), propertyData11);
    assertEquals("Cached child property " + propertyData12.getQPath().getAsString() + " is not equals", cache.get(propertyPath12), propertyData12);
    assertEquals("Cached child property " + propertyData12.getUUID() + " is not equals", cache.get(propertyUuid12), propertyData12);
    
    assertEquals("Cached child property " + propertyData21.getQPath().getAsString() + " is not equals", cache.get(propertyPath21), propertyData21);
    assertEquals("Cached child property " + propertyData21.getUUID() + " is not equals", cache.get(propertyUuid21), propertyData21);
    assertEquals("Cached child property " + propertyData22.getQPath().getAsString() + " is not equals", cache.get(propertyPath22), propertyData22);
    assertEquals("Cached child property " + propertyData22.getUUID() + " is not equals", cache.get(propertyUuid22), propertyData22);
    
    assertTrue("Cached child property " + propertyData11.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData11));
    assertTrue("Cached child property " + propertyData12.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData12));
    assertTrue("Cached child property " + propertyData21.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData21));
    assertTrue("Cached child property " + propertyData22.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData22));
    
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData1).size(), 2);
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData2).size(), 2);
  }
  
  public void testGetProperty_Persisted() throws Exception {

    initNodesData();
    initDataAsPersisted();
    
    List<NodeData> nodes = new ArrayList<NodeData>();
    nodes.add(nodeData31);
    nodes.add(nodeData32);
    cache.addChildNodes(nodeData3, nodes);
    
    cache.put(nodeData1);
    cache.put(nodeData2);
    cache.put(propertyData12);
    
    List<PropertyData> properties2 = new ArrayList<PropertyData>();
    properties2.add(propertyData21);
    properties2.add(propertyData22);
    cache.addChildProperties(nodeData2, properties2);
    
    List<PropertyData> properties1 = new ArrayList<PropertyData>();
    properties1.add(propertyData11);
    properties1.add(propertyData12);
    cache.addChildProperties(nodeData1, properties1);
    
    // props, prev stuff
    assertEquals("Cached child property " + propertyData11.getQPath().getAsString() + " is not equals", cache.get(propertyPath11), propertyData11);
    assertEquals("Cached child property " + propertyData11.getUUID() + " is not equals", cache.get(propertyUuid11), propertyData11);
    assertEquals("Cached child property " + propertyData12.getQPath().getAsString() + " is not equals", cache.get(propertyPath12), propertyData12);
    assertEquals("Cached child property " + propertyData12.getUUID() + " is not equals", cache.get(propertyUuid12), propertyData12);
    
    assertEquals("Cached child property " + propertyData21.getQPath().getAsString() + " is not equals", cache.get(propertyPath21), propertyData21);
    assertEquals("Cached child property " + propertyData21.getUUID() + " is not equals", cache.get(propertyUuid21), propertyData21);
    assertEquals("Cached child property " + propertyData22.getQPath().getAsString() + " is not equals", cache.get(propertyPath22), propertyData22);
    assertEquals("Cached child property " + propertyData22.getUUID() + " is not equals", cache.get(propertyUuid22), propertyData22);
    
    assertTrue("Cached child property " + propertyData11.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData11));
    assertTrue("Cached child property " + propertyData12.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData12));
    assertTrue("Cached child property " + propertyData21.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData21));
    assertTrue("Cached child property " + propertyData22.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData22));
    
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData1).size(), 2);
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData2).size(), 2);
    
    // remove
    cache.remove(propertyData12);
    
    // check
    assertEquals("Cached child property " + propertyData11.getQPath().getAsString() + " is not equals", cache.get(propertyPath11), propertyData11);
    assertEquals("Cached child property " + propertyData11.getUUID() + " is not equals", cache.get(propertyUuid11), propertyData11);

    // here
    assertNull("Child property " + propertyData12.getQPath().getAsString() + " is not in the cache", cache.get(propertyPath12));
    assertNull("Child property " + propertyData12.getQPath().getAsString() + " is not in the cache", cache.get(propertyUuid12));
    
    assertEquals("Cached child property " + propertyData21.getQPath().getAsString() + " is not equals", cache.get(propertyPath21), propertyData21);
    assertEquals("Cached child property " + propertyData21.getUUID() + " is not equals", cache.get(propertyUuid21), propertyData21);
    assertEquals("Cached child property " + propertyData22.getQPath().getAsString() + " is not equals", cache.get(propertyPath22), propertyData22);
    assertEquals("Cached child property " + propertyData22.getUUID() + " is not equals", cache.get(propertyUuid22), propertyData22);
    
    assertTrue("Cached child property " + propertyData11.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData11));
    
    // here
    assertFalse("Cached child property " + propertyData12.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData1).contains(propertyData12));

    assertTrue("Cached child property " + propertyData21.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData21));
    assertTrue("Cached child property " + propertyData22.getQPath().getAsString() + " is not in the childs list", cache.getChildProperties(nodeData2).contains(propertyData22));
    
    // and here
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData1).size(), 1);
    
    assertEquals("Cached child properties count is wrong", cache.getChildProperties(nodeData2).size(), 2);
  }

}