/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.property.dav;

import java.util.ArrayList;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.version.Version;

import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.property.DavProperty;
import org.exoplatform.services.webdav.resource.DavResource;
import org.exoplatform.services.webdav.resource.VersionResource;
import org.exoplatform.services.webdav.response.DavStatus;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: SuccessorSetProp.java 12525 2007-02-02 12:26:47Z gavrikvetal $
 */

public class SuccessorSetProp extends AbstractDAVProperty {

  protected ArrayList<String> versionSuccessors = new ArrayList<String>();
  
  public SuccessorSetProp() {
    super(DavProperty.SUCCESSORSET);
  }
  
  @Override
  protected boolean initialize(DavResource resource) throws RepositoryException {    
    if (!(resource instanceof VersionResource)) {
      return false;
    }
    
    Node node = ((VersionResource)resource).getNode();
    
    if (!(node instanceof Version)) {
      return false;
    }
    
    Version []successors = ((Version)node).getSuccessors();

    while (successors.length > 0) {
      versionSuccessors.add(resourceHref + DavConst.DAV_VERSIONPREFIX + successors[0].getName());
      successors = successors[0].getSuccessors();
    }      
    status = DavStatus.OK;

    return true;
  }
  
  @Override
  public void serialize(Document rootDoc, Element parentElement) {
    super.serialize(rootDoc, parentElement);
    if (status != DavStatus.OK) {
      return;
    }
    for (int i = 0; i < versionSuccessors.size(); i++) {
      Element hrefEl = rootDoc.createElement(DavConst.DAV_PREFIX + DavProperty.HREF);
      propertyElement.appendChild(hrefEl);
      hrefEl.setTextContent(versionSuccessors.get(i));
    }
  }  
  
}
