/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: ExoLauncherApplet.java 10464 2006-11-21 18:55:33Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.launcher;

import java.applet.Applet;
import java.applet.AppletStub;
import java.awt.CardLayout;
import java.awt.Label;

import org.exoplatform.applet.ExoAppletException;
import org.exoplatform.applet.Invokable;
import org.exoplatform.applet.deploy.LauncherConfiguration;
import org.exoplatform.applet.install.DownloadItem;
import org.exoplatform.applet.install.Installer;
import org.exoplatform.applet.install.Loader;
import org.exoplatform.applet.install.event.InstallEvent;
import org.exoplatform.applet.install.event.InstallListener;
import org.exoplatform.applet.logging.Level;
import org.exoplatform.applet.logging.Logger;
import org.exoplatform.applet.logging.file.FileLoggerFactory;
import org.exoplatform.applet.plugin.AppletPlugin;
import org.exoplatform.applet.plugin.PluginManager;
import org.exoplatform.applet.scripting.JavaScriptBrowser;
import org.exoplatform.applet.scripting.ScriptableBrowserSupport;
import org.exoplatform.applet.security.RestrictedAction;
import org.exoplatform.applet.security.RestrictedActionFactory;
import org.exoplatform.applet.security.RestrictedActor;
import org.exoplatform.applet.util.Base64Serializer;
import org.exoplatform.applet.util.ExoAppletUtils;

/**
 * @version $Rev: 10464 $ $Date: 2006-11-22 01:55:33 +0700 (Wed, 22 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class ExoLauncherApplet extends Applet implements Invokable {

	private class Counter {

		private long completedDownload;

		private long totalDownloadSize;

		public Counter(long totalDownloadSize) {
			super();
			this.totalDownloadSize = totalDownloadSize;
			this.completedDownload = 0;
		}

		/**
		 * @param downloadCount
		 */
		public int getProgressLevel(int downloadCount) {
			long count = (completedDownload + downloadCount);
			int level = (int) ((count * 100) / totalDownloadSize);
			return level;
		}

		/**
		 * @param downloadCount
		 */
		public void setCompletedDownloadSize(int downloadCount) {
			completedDownload = +downloadCount;
		}
	}

	/**
	 * Serialization object version uid.
	 */
	private static final long serialVersionUID = 1L;

	private static final String ERROR_LAYOUT_CARD_NAME = "error".intern();

//	private static final String DOWNLOAD_LAYOUT_CARD_NAME = "download";

	private static final String REAL_APPLET_LAYOUT_CARD_NAME = "real".intern();

	private static final String LOG_SOURCE_CLASS = "ExoLauncherApplet".intern();

	private CardLayout layoutMgr;

	/**
	 * The launching Applet.
	 */
	private Applet realApplet;

	private RestrictedAction ra;

	private AppletParamHelper arguments;

	private PluginManager pluginMgr;

	private LauncherConfiguration configuration;

	private boolean realAppletStarted;

	private Logger logger;

	public ExoLauncherApplet() {
		super();

		layoutMgr = new CardLayout();
		this.setLayout(layoutMgr);
	}

	public void init() {

		final String log_method = "init".intern();

		if (realApplet != null) {
			logger.finest("Initing Real Applet.");
			realApplet.init();
			return;
		}

		initLogging();
		logger.entering(LOG_SOURCE_CLASS, log_method, realApplet);

		try {

			// check applet is signed (to support needed restricted actions)
			if (!isSignedApplet()) {
				throw new ExoAppletException(
	        		"Unable to execute. This is not a signed applet.");
			}

			// load applet configuration
			configuration = getLauncherConfiguration();

			// do installation or updates
			if (configuration != null) {
//				prepareDownloadGUI();
				doInstall();
			}

		} catch (ExoAppletException e) {
			doFailure(e);
		}

		logger.exiting(LOG_SOURCE_CLASS, log_method);
	}

	public void destroy() {
		if (realApplet != null) {
			realApplet.destroy();
			realApplet = null;
		}

		if (pluginMgr != null)
			pluginMgr.shutdownLifeCyclePlugins();
	}

	public Object invoke(String methodName, String args) {

		logger.entering(LOG_SOURCE_CLASS, "invoke", methodName + " :: " + args);

		String[] params = (args == null ? new String[0] : args.split(","));

		Object result = invoke(methodName, params);

		logger.exiting(LOG_SOURCE_CLASS, "invoke", result);

		return result;
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.Invokable#invoke(java.lang.String, java.lang.String[])
	 */
	public Object invoke(String methodName, String[] args) {
		Object result = null;

		try {
			if ((realApplet != null) && (realApplet instanceof Invokable))
				result = ((Invokable) realApplet).invoke(methodName, args);
		}
		catch (Throwable t) {
			logger.throwing(LOG_SOURCE_CLASS, "invoke", t);
		}

		return result;
	}

	private void doFailure(ExoAppletException e) {
		doFailure(e.getMessage());
	}

	private void doFailure(String error) {

		final String log_method = "doFailure".intern();

		logger.entering(LOG_SOURCE_CLASS, log_method, error);

		Label label = new Label(error);
		this.add(label, ERROR_LAYOUT_CARD_NAME);
		layoutMgr.show(this, ERROR_LAYOUT_CARD_NAME);

		logger.exiting(LOG_SOURCE_CLASS, log_method);
	}

	private void doInstall() throws ExoAppletException {

		final String log_method = "doInstall".intern();

		logger.entering(LOG_SOURCE_CLASS, log_method);

		String baseUrl = getCodeBaseUrl();

		Installer installer = new Installer(baseUrl, configuration);
		final Counter counter = new Counter(installer.getTotalDownloadSize());

		installer.addInstallListener(new InstallListener() {
			public void onAction(InstallEvent event) {

				DownloadItem dl;

				switch (event.getAction()) {

//				case InstallEvent.DOWNLOAD_PROGRESS:
//					download = (DownloadItem) event.getSource();
//					int level = counter.getProgressLevel(download
//					        .getDownloadCount());
//					setDownloadProgressLevel(level);
//					break;

				case InstallEvent.DOWNLOAD_COMPLETE:
					dl = (DownloadItem) event.getSource();
					counter.setCompletedDownloadSize(dl.getDownloadCount());
					break;

				case InstallEvent.LOAD_APPLET:
					Applet target = (Applet) event.getSource();

					doRealAppletInit(target);
					break;

				case InstallEvent.LOAD_PLUGIN:
					AppletPlugin plugin = (AppletPlugin) event.getSource();

					onPluginLoad(plugin);
					break;

				case InstallEvent.LOAD_COMPLETE:
					Loader loader = (Loader) event.getSource();
					initPluginManager(loader);
					break;

				case InstallEvent.LOAD_FAILURE:
					doFailure("Applet loading failed. Cannot bootstrap components from real applet package.");
					break;

				case InstallEvent.DOWNLOAD_FAILURE:
					dl = (DownloadItem) event.getSource();
					doFailure("Failed to download " + dl.getDownloadUrl() + ".");
					break;
				}
			}

		});

		Thread t = installer.createThread();
		t.start();

		logger.exiting(LOG_SOURCE_CLASS, log_method);
	}

	private void onPluginLoad(AppletPlugin plugin) {
		if (plugin instanceof RestrictedActor)
			((RestrictedActor) plugin).setTrustAccess(getRestrictedAction());
	}

	private void doRealAppletInit(Applet real) {

		final String log_method = "doRealAppletInit".intern();
		logger.entering(LOG_SOURCE_CLASS, log_method, real);

		this.realApplet = real;

		AppletStub stub = new DelegationAppletStub(this);
		realApplet.setStub(stub);

		if (realApplet instanceof ScriptableBrowserSupport) {
			((ScriptableBrowserSupport) realApplet).setBrowser(new JavaScriptBrowser(this));
		}

		showRealApplet();

		logger.exiting(LOG_SOURCE_CLASS, log_method);
	}

	protected AppletParamHelper getArguments() {
		if (arguments == null)
			arguments = new AppletParamHelper(this);
		return arguments;
	}

	private String getCodeBaseUrl() {

		final String log_method = "getCodeBaseUrl".intern();

		logger.entering(LOG_SOURCE_CLASS, log_method);

		String baseUrl = arguments.getCodeBaseUrlParam();

		if (baseUrl == null)
			baseUrl = ExoAppletUtils.getCodeBase(this);

		logger.exiting(LOG_SOURCE_CLASS, log_method, baseUrl);

		return baseUrl;
	}

	private LauncherConfiguration getLauncherConfiguration()
	        throws ExoAppletException {

		final String log_method = "getLauncherConfig";

		logger.entering(LOG_SOURCE_CLASS, log_method);

		LauncherConfiguration cfg = null;

		AppletParamHelper args = getArguments();
		String encodedConfig = args.getLauncherConfigurationParam();

		if (encodedConfig == null)
			throw new ExoAppletException(
			        "Init failed. Launcher Applet configuration parameter not found.");

		try {
			cfg = (LauncherConfiguration) Base64Serializer
			        .deserialize(encodedConfig.getBytes());
		} catch (Exception e) {
			throw new ExoAppletException(
			        "Invalid launcher applet configuration parameter.", e);
		}

		logger.exiting(LOG_SOURCE_CLASS, log_method, cfg);

		return cfg;
	}

	protected RestrictedAction getRestrictedAction() {
		if (ra == null)
			this.ra = RestrictedActionFactory.getInstance();
		return ra;
	}

	public Object getRealApplet() {

		final String log_method = "getRealApplet".intern();

		logger.exiting(LOG_SOURCE_CLASS, log_method, realApplet);

		return realApplet;
	}

    private void initLogging() {

    	Level loggingLevel;

    	String slevel = getArguments().getLoggingLevelParam();

    	if (slevel == null || "".equals(slevel.trim()))
    		loggingLevel = Level.WARNING;
    	else
    		loggingLevel = Level.parse(slevel);

    	FileLoggerFactory factory = FileLoggerFactory.getInstance();
    	factory.configureLevel(loggingLevel);

    	this.logger = factory.getLogger(ExoLauncherApplet.class.getName());

    }

	private void initPluginManager(Loader loader) {

		Applet target = loader.getRealApplet();
		AppletPlugin[] plugins = loader.getPlugins();

		if ((target == null) || (plugins == null))
			return;

		pluginMgr = new PluginManager(target, plugins);

		pluginMgr.initLifeCyclePlugins();
	}

	private boolean isSignedApplet() {
		try {
			getRestrictedAction().getUserHome();
			return true;
		} catch (SecurityException se) {
			return false;
		}
	}

	private void showRealApplet() {
		this.add(realApplet, REAL_APPLET_LAYOUT_CARD_NAME);
		realApplet.init();

		layoutMgr.show(this, REAL_APPLET_LAYOUT_CARD_NAME);

		if (!realAppletStarted)
			start();
	}

	public void start() {
		if (realApplet != null) {
			realAppletStarted = true;
			realApplet.start();
		}

		if (pluginMgr != null)
			pluginMgr.startLifeCyclePlugins();
	}

	public void stop() {
		if (realApplet != null)
			realApplet.stop();

		if (pluginMgr != null)
			pluginMgr.stopLifeCyclePlugins();
	}

}
