/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: Level.java 8955 2006-09-18 13:46:06Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.logging;

import java.io.Serializable;

/**
 * @version $Rev: 8955 $ $Date: 2006-09-18 20:46:06 +0700 (Mon, 18 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public final class Level implements Serializable {

    private static final long serialVersionUID = 1L;

    private static final String ALL_LEVEL_NAME = "All";
    private static final String CONFIG_LEVEL_NAME = "Config";
    private static final String FINE_LEVEL_NAME = "Fine";
    private static final String FINER_LEVEL_NAME = "Finer";
    private static final String FINEST_LEVEL_NAME = "Finest";
    private static final String INFO_LEVEL_NAME = "Info";
    private static final String OFF_LEVEL_NAME = "Off";
    private static final String SEVERE_LEVEL_NAME = "Severe";
    private static final String WARNING_LEVEL_NAME = "Warning";

    private static final byte ALL_LEVEL_VALUE = Byte.MAX_VALUE;
    private static final byte CONFIG_LEVEL_VALUE = 4;
    private static final byte FINE_LEVEL_VALUE = 3;
    private static final byte FINER_LEVEL_VALUE = 2;
    private static final byte FINEST_LEVEL_VALUE = 1;
    private static final byte INFO_LEVEL_VALUE = 5;
    private static final byte OFF_LEVEL_VALUE = 0;
    private static final byte SEVERE_LEVEL_VALUE = 7;
    private static final byte WARNING_LEVEL_VALUE = 6;

    public static final Level ALL = new Level(ALL_LEVEL_NAME, ALL_LEVEL_VALUE);
	public static final Level CONFIG = new Level(CONFIG_LEVEL_NAME, CONFIG_LEVEL_VALUE);
	public static final Level FINE = new Level(FINE_LEVEL_NAME, FINE_LEVEL_VALUE);
	public static final Level FINER = new Level(FINER_LEVEL_NAME, FINER_LEVEL_VALUE);
	public static final Level FINEST = new Level(FINEST_LEVEL_NAME, FINEST_LEVEL_VALUE);
	public static final Level INFO = new Level(INFO_LEVEL_NAME, INFO_LEVEL_VALUE);
	public static final Level OFF = new Level(OFF_LEVEL_NAME, OFF_LEVEL_VALUE);
	public static final Level SEVERE = new Level(SEVERE_LEVEL_NAME, SEVERE_LEVEL_VALUE);
	public static final Level WARNING = new Level(WARNING_LEVEL_NAME, WARNING_LEVEL_VALUE);

	private static Level[] definedLevels;

	private String name;

	private byte value;

    protected Level(String name, byte value) {
    	super();
    	this.name = name;
    	this.value = value;
    }

    protected static Level[] getDefinedLevels() {
    	if (definedLevels == null)
    		definedLevels = new Level[] {ALL, CONFIG, FINE, FINER, FINEST, INFO,
    			OFF, SEVERE, WARNING};
    	return definedLevels;
    }

    public String getName() {
        return name;
    }
    
    public int intValue() {
        return value;
    }

    public static Level parse(String name) {

    	Level found = null;

    	Level[] ls = getDefinedLevels();
    	for (int i=0; i<ls.length; i++) {
    		if (name.equalsIgnoreCase(ls[i].getName())) {
    			found = ls[i];
    			break;
    		}
    	}

    	return found;
    }

    public static Level parse(byte value) {
    	
    	Level found = null;

    	Level[] ls = getDefinedLevels();
    	for (int i=0; i<ls.length; i++) {
    		if (value == ls[i].intValue()) {
    			found = ls[i];
    			break;
    		}
    	}

    	return found;
    }

    public String toString() {
        return getName();
    }

}
