/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: BrowserDetect.java 10468 2006-11-22 00:20:32Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.appletconfig;

/**
 * @version $Rev: 10468 $ $Date: 2006-11-22 07:20:32 +0700 (Wed, 22 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class BrowserDetect {

    private static final String GECKO = "gecko".intern();
    private static final String SAFARI = "safari".intern();
    private static final String NETSCAPE = "netscape".intern();
    private static final String MOZILLA = "mozilla".intern();
    private static final String OPERA = "opera".intern();
    private static final String WEBTV = "webtv".intern();
    private static final String HOTJAVA = "hotjava".intern();
    private static final String MSIE = "msie".intern();
    private static final String KONQUEROR = "konqueror".intern();
    private static final String ICAB = "icab".intern();
    private static final String AOL = "aol".intern();

//    private static final String WIN = "win".intern();
//    private static final String MAC = "mac".intern();
//    private static final String UNIX = "unix".intern();
//    private static final String LINUX = "linux".intern();
//    private static final String SUNOS = "sunos".intern();
//    private static final String BSD = "bsd".intern();
//    private static final String X11 = "x11".intern();

    private static final String SPOOFER = "spoofer".intern();
    private static final String COMPATIBLE = "compatible".intern();

    /**
     * User Agent info.
     */
    private String ua;

    /**
     * @param userAgentInfo
     */
    public BrowserDetect(String userAgentInfo) {
        super();
        this.ua = userAgentInfo.toLowerCase();
    }

    public boolean isGecko() {
        return (ua.indexOf(GECKO) != -1 && ua.indexOf(SAFARI) == -1);
    }

    public boolean isMozilla() {
        return (this.isGecko() && (ua.indexOf(GECKO + "/") + 14 == ua.length()));
    }

    public boolean isNS() {
        return (this.isGecko() ? (ua.indexOf(NETSCAPE) != -1) :
            ((ua.indexOf(MOZILLA) != -1) &&
                    (ua.indexOf(SPOOFER) == -1) &&
                    (ua.indexOf(COMPATIBLE) == -1) &&
                    (ua.indexOf(OPERA) == -1) &&
                    (ua.indexOf(WEBTV) == -1) &&
                    (ua.indexOf(HOTJAVA) == -1)));
    }

    public boolean isIE() {
        return ((ua.indexOf(MSIE) != -1) &&
                (ua.indexOf(OPERA) == -1) &&
                (ua.indexOf(WEBTV) == -1)); 
    }

    public boolean isSafari() {
        return (ua.indexOf(SAFARI) != - 1);
    }

    public boolean isOpera() {
        return (ua.indexOf(OPERA) != -1);
    }

    public boolean isKonqueror() {
        return (ua.indexOf(KONQUEROR) != -1 && !this.isSafari());
    }

    public boolean isIcab() {
        return (ua.indexOf(ICAB) != -1);
    }

    public boolean isAol() {
        return (ua.indexOf(AOL) != -1);
    }

    public boolean isIECompatible() {
        return ((ua.indexOf(MSIE) != -1) && !this.isIE());
    }

    public boolean isNSCompatible() {
        return ((ua.indexOf(MOZILLA) != -1) && !this.isNS() && !this.isMozilla());
    }

//    public boolean isWin() {
//        return (ua.indexOf(WIN) != -1);
//    }
//
//    public boolean isMac() {
//        return (ua.indexOf(MAC) != -1);
//    }
//
//    public boolean isUnix() {
//        return (ua.indexOf(UNIX) != -1 ||
//                ua.indexOf(SUNOS) != -1 ||
//                ua.indexOf(BSD) != -1 ||
//                ua.indexOf(X11) != -1);
//    }
//
//    public boolean isLinux() {
//        return (ua.indexOf(LINUX) != -1);
//    }
}
