/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: AppletConfigurationServiceImpl.java 10589 2006-11-24 04:57:38Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.appletconfig.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.appletconfig.AppletConfigurationService;
import org.exoplatform.services.appletconfig.BrowserDetect;
import org.exoplatform.services.appletconfig.impl.embed.EmbeddingStrategy;
import org.exoplatform.services.appletconfig.impl.embed.EmbeddingStrategyFactory;
import org.exoplatform.services.log.LogService;

/**
 * 
 * @version $Rev: 10589 $ $Date: 2006-11-24 11:57:38 +0700 (Fri, 24 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class AppletConfigurationServiceImpl implements
        AppletConfigurationService {

    private static final String LAUNCHER_CONFIG_PARAM_NAME = "launcherConfiguration";

    private Map<String, DistributionConfiguration> configCache;

    private Log logger;

    public AppletConfigurationServiceImpl(LogService loggingService) {

        super();
        this.logger = loggingService.getLog(AppletConfigurationServiceImpl.class);
        this.configCache = new HashMap<String, DistributionConfiguration>();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.exoplatform.services.appletconfig.AppletConfigurationService#getHTMLAppletEmbedding(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public String getAppletConfigHTMLEmbed(String configName, String objectId,
            String userAgentInfo) {
        return getAppletConfigHTMLEmbed(configName, objectId, userAgentInfo,
                null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.exoplatform.services.appletconfig.AppletConfigurationService#getHTMLAppletEmbedding(java.lang.String,
     *      java.lang.String, java.lang.String, java.util.Map)
     */
    public String getAppletConfigHTMLEmbed(String configName, String objectId,
            String userAgentInfo, Map<String, String> additionalParams) {

        logger.debug("Get Applet Config: " + configName + " - User Agent: " +
                userAgentInfo);

        String result = null;

        DistributionConfiguration config = retrieveAppletConfig(configName);

        if (config == null) {
            // TODO return a string got from ResourceBundle
            result = "Applet Configuration Not Available";

        } else {

            ParameterConfiguration pc = config.getParameterConfig();

            /*
             * Include the serialized launcher configuration java beans as an
             * applet embedding parameter.
             */
            Map<String, String> parameters = new HashMap<String, String>();

            if (pc.getParams() != null)
                parameters.putAll(pc.getParams());
            
            if (additionalParams != null)
                parameters.putAll(additionalParams);

            parameters.put(LAUNCHER_CONFIG_PARAM_NAME, config
                    .getB64LauncherConfig());

            /*
             * Generate the HTML embedding tags according to browser's specifics
             * and avoid mal-functioning or non applet viewing on web page.
             */
            BrowserDetect ua = new BrowserDetect(userAgentInfo);
            EmbeddingStrategy appletHtml = EmbeddingStrategyFactory.getInstance(ua);

            result = appletHtml.render(objectId, pc, parameters);
        }

        return result;
    }

    private DistributionConfiguration retrieveAppletConfig(String configName) {

        DistributionConfiguration dc = null;
        try {
            dc = configCache.get(configName.toLowerCase());
        } catch (Exception e) {
            logger.trace("Failed to retrieve applet configuration cache.",
                    e);
            // do nothing
        }
        return dc;
    }

    public void addPlugin(ComponentPlugin plugin) {

        logger.debug("Adding applet configuration.");

        if (plugin instanceof DistributionConfiguration) {
            try {

                DistributionConfiguration dc = (DistributionConfiguration) plugin;
                String configName = dc.getAppletConfigName().toLowerCase();
                configCache.put(configName, dc);

                logger.debug("Applet Configuration: " + configName);

            } catch (Exception e) {
                // do nothing
                logger.trace("Error adding applet configuration.", e);
            }
        }
    }

}
