/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import org.apache.commons.fileupload.FileItem;
import org.exoplatform.commons.utils.ISO8601;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIDateInput.java,v 1.2 2004/10/16 21:13:50 tuan08 Exp $
 */
public class UICalendarInput extends UIStringInput {

  private boolean displayTime;

  protected GregorianCalendar calendar_;
  private SimpleDateFormat sdf_;
  
  public UICalendarInput(String name, String text) {
    super(name, text);
    sdf_ = new SimpleDateFormat("dd/MM/yyyy");
    if (displayTime)
      sdf_ = new SimpleDateFormat("dd/MM/yyyy hh:mm");
    Date date = new Date();
    if (text != null && !text.equals("")) {
      calendar_ = new GregorianCalendar();
      try {
        date = sdf_.parse(text);
      } catch (ParseException e) {
        e.printStackTrace();
      }
      calendar_.setTime(date);
    }
    setRendererType("CalendarInputRenderer");
  }

  public Object getDecodedValue() {
    try {
	  Date d = calendar_.getTime();
	  if (displayTime)
	      sdf_ = new SimpleDateFormat("dd/MM/yyyy hh:mm");
	  return sdf_.format(d);
	}catch (Exception e){
	  return "";
	}
  }

  public String getValue() {
    if (calendar_ != null)
      return ISO8601.format(calendar_);
    else
      return ISO8601.format(new GregorianCalendar());
  }

  public UIInput setValue(String value) {
	if (calendar_ == null){
	  calendar_ = new GregorianCalendar();
	}
    calendar_.setTime(ISO8601.parse(value).getTime());
    return this;
  }

  public Calendar getCalendar() {
    return calendar_;
  }

  public UICalendarInput addTime(Date date) {
	  if (date != null)
		  calendar_.setTime(date);
	  else
		  calendar_= null;
    return this;
  }

  public boolean isDisplayTime() {
    return displayTime;
  }

  public void setDisplayTime(boolean displayTime) {
    this.displayTime = displayTime;
  }

  public void decodeFromMultipartFields(FacesContext context,
      UIComponent component, List items) {
    Iterator iter = items.iterator();
    String choosenDate = "";
    while (iter.hasNext()) {
      FileItem item = (FileItem) iter.next();
      if (item.isFormField()) {
        String name = item.getFieldName();
        if (name != null && name.equalsIgnoreCase(getName())) {
          choosenDate = item.getString();
        }
      }
    }
    SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
    if (displayTime)
      sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
    Date date = null;
    if (choosenDate != null && !choosenDate.trim().equals("")) {
      calendar_ = new GregorianCalendar();
      try {
        date = sdf.parse(choosenDate);
      } catch (ParseException e) {
        e.printStackTrace();
      }
      addTime(date);
    } else if (choosenDate != null && choosenDate.trim().equals("")) {
      addTime(date);
    }
  }

  public void setProperty(String propertyName, Object value) {
    if (properties == null)
      properties = new HashMap(3);
    properties.put(propertyName, value);
  }
}