/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.community;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Dec 4, 2004
 * @version $Id$
 */
public class CommunityConfigService {
  static CommunityPortal NO_COMMUNITY_PORTAL = new CommunityPortal() ;
  static CommunityNavigation NO_COMMUNITY_NAVIGATION = new CommunityNavigation() ;
  
  private OrganizationService orgService_ ;
  private HibernateService hservice_ ;
  private ExoCache cpCache_ ;
  private ExoCache cnCache_ ;
  private List<CommunityEventListener> 
            listeners_  = new ArrayList<CommunityEventListener>(3);
  
  public CommunityConfigService(InitParams params,
                                    HibernateService hservice,
                                    CacheService cservice ,
                                    OrganizationService  orgService) throws Exception {
    orgService_ = orgService ;
    hservice_ = hservice ;
    cpCache_ = cservice.getCacheInstance(CommunityPortal.class.getName()) ;
    cnCache_ = cservice.getCacheInstance(CommunityNavigation.class.getName()) ;
    initDb(params) ;
  }
  
  final public List getCommunityEventListeners() {  return listeners_ ;  }

  public void addCommunityEventListener(ComponentPlugin listener) {
    if(listener instanceof CommunityEventListener)
      listeners_.add((CommunityEventListener)listener) ;
  }
 
  private void initDb(InitParams params) throws Exception {
    try {
    List list = 
      hservice_.openSession().createCriteria(CommunityPortal.class).list() ;
    if(list.size() == 0) {
      list = params.getObjectParamValues(CommunityPortal.class);
      if(list != null) {
        for(int i = 0; i < list.size(); i++) {
          CommunityPortal cp = (CommunityPortal) list.get(i) ;
          hservice_.save(cp) ;
          //addCommunityPortal(cp) ; 
        }
      }
      
      list = params.getObjectParamValues(CommunityNavigation.class);
      if(list != null) {
        for(int i = 0; i < list.size(); i++) {
          CommunityNavigation cn = (CommunityNavigation) list.get(i) ;
          hservice_.save(cn) ;
          //cnCache_.remove(cn.getGroupId()) ;
          //addCommunityNavigation(cn) ; 
        }
      }
    }
    } catch (Exception ex) {
      ex.printStackTrace() ;
      throw ex ;
    }
  }
  
  public CommunityPortal createCommunityPortal() { return new CommunityPortal(); }

  public CommunityPortal getCommunityPortal(String groupId) throws Exception {
    CommunityPortal cportal = (CommunityPortal) cpCache_.get(groupId) ;
    if(cportal == null) {
      cportal =  (CommunityPortal)hservice_.findOne(CommunityPortal.class, groupId) ;
      if(cportal == null) cpCache_.put(groupId, NO_COMMUNITY_PORTAL) ;
      else cpCache_.put(groupId, cportal) ;
    }
    if(cportal == NO_COMMUNITY_PORTAL)  return null ;
    return cportal ;
  }
  
  public CommunityPortal findCommunityPortal(String user) throws Exception {
    Collection memberships = orgService_.getMembershipHandler().findMembershipsByUser(user) ;
    CommunityPortal found = null ;
    Iterator mitr = memberships.iterator() ;
    while(mitr.hasNext()) {
      Membership m = (Membership) mitr.next() ;
      CommunityPortal cp = getCommunityPortal(m.getGroupId()) ;
      if(cp != null && cp.getMembership().equals(m.getMembershipType())) {
        if(found == null || cp.getPriority() < found.getPriority()) found =  cp ;
      }
    }
    return found;
  }
  
  public void addCommunityPortal(CommunityPortal cp) throws Exception {
    hservice_.save(cp) ;
    cpCache_.remove(cp.getGroupId()) ;
    onChange(cp) ;
  }

  public void removeCommunityPortal(CommunityPortal cp) throws Exception {
    hservice_.remove(CommunityPortal.class, cp.getGroupId()) ;
    cpCache_.remove(cp.getGroupId()) ;
    onChange(cp) ;
  }

  public CommunityNavigation createCommunityNavigation() { return new CommunityNavigation(); }
  
  public CommunityNavigation getCommunityNavigation(String groupId) throws Exception {
    CommunityNavigation cnav = (CommunityNavigation) cnCache_.get(groupId) ;
    if(cnav == null) {
      cnav = (CommunityNavigation)hservice_.findOne(CommunityNavigation.class, groupId) ;
      if(cnav == null) cnav =  NO_COMMUNITY_NAVIGATION  ;
      cnCache_.put(groupId, cnav) ;
    }
    if(cnav == NO_COMMUNITY_NAVIGATION)  {
      return null ;
    }
    return cnav ;
  }

  public List findCommunityNavigation(String user) throws Exception {
    List<CommunityNavigation> list = new ArrayList<CommunityNavigation>() ;
    Collection memberships = orgService_.getMembershipHandler().findMembershipsByUser(user) ;
    Iterator mitr = memberships.iterator() ;
    while(mitr.hasNext()) {
      Membership m = (Membership) mitr.next() ;
      CommunityNavigation cn  = getCommunityNavigation(m.getGroupId());
      if(cn != null && cn.getMembership().equals(m.getMembershipType())) {
        list.add(cn) ;
      }
    }
    return list;
  }
  
  public void addCommunityNavigation(CommunityNavigation cn) throws Exception {
    hservice_.save(cn) ;
    cnCache_.remove(cn.getGroupId()) ;
    onChange(cn) ;
  }

  public void removeCommunityNavigation(CommunityNavigation cn) throws Exception {
    hservice_.remove(CommunityNavigation.class, cn.getGroupId());
    cnCache_.remove(cn.getGroupId()) ;
    onChange(cn) ;
  }
  
  private void onChange(CommunityNavigation cn) throws Exception {
    for (CommunityEventListener listener : listeners_)  listener.onChange(cn) ;
  }
  
  private void onChange(CommunityPortal cp) throws Exception {
    for (CommunityEventListener listener : listeners_)  listener.onChange(cp) ;
  } 

}
