/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;

import java.util.Comparator;
import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.portal.community.CommunityConfigService;
import org.exoplatform.portal.community.CommunityNavigation;
import org.exoplatform.portal.community.CommunityPortal;
import org.exoplatform.portal.config.PortalConfigService;
import org.exoplatform.portal.config.Query;
import org.exoplatform.portal.config.impl.NodeNavigationData;
import org.exoplatform.portal.config.impl.PageDescriptionData;
import org.exoplatform.portal.config.impl.PortalConfigData;
import org.exoplatform.portal.config.model.Node;
import org.exoplatform.portal.config.model.Page;
import org.exoplatform.portal.config.model.Permission;
import org.exoplatform.portal.config.model.PortalConfig;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;

/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestPortalConfigService.java,v 1.19 2004/10/20 20:56:28 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestPortalConfigService  extends PortalServiceTestBase {
  static protected PortalConfigService   service_ ;       
  static protected CommunityConfigService comService_ ;  
  
  private static String admin = "exoadmin" ;            

  public TestPortalConfigService(String name) {
    super(name);
  }

  public void setUp() throws Exception {     
    if (service_ == null) {      
      PortalContainer manager  = PortalContainer.getInstance();      
      service_ = 
        (PortalConfigService) manager.getComponentInstanceOfType(PortalConfigService.class) ;
      orgService_ = 
        (OrganizationService) manager.getComponentInstanceOfType(OrganizationService.class);
      comService_ = 
        (CommunityConfigService)manager.getComponentInstanceOfType(CommunityConfigService.class) ;
      hservice_ = 
        (HibernateService)manager.getComponentInstanceOfType(HibernateService.class) ;
    }
  }
  
  public void tearDown() throws Exception {
    orgService_ = null ;
  }
  
  public void testPortalConfigService() throws Exception {    
    prepareOrganizationData() ;
    Permission editPer = createPermission("/"+Group1,"*") ;
    Permission viewPer = createPermission("/"+Group1,memtype1) ;
        
    System.out.println("======================Test portal config service with admin=======") ;
    createUser(admin) ;
    PortalConfig adminconfig = service_.getPortalConfig(admin, admin) ;
    Node adminnavigation = service_.getNodeNavigation(admin, admin) ;
    assertTrue("No Community config", comService_.findCommunityPortal("exoadmin") != null) ;
    assertTrue("expect the portal config is not shared", adminconfig.isSharedLayout()) ;
    assertTrue("expect owner of portal config is 'admin'",adminconfig.getOwner().equals(admin)) ;
    assertTrue("expect view permission is not null", adminconfig.getViewPermission() != null) ;
//    assertTrue("expect edit permission is not null", adminconfig.getEditPermission() != null) ;
    /*-------admin can edit portal config----*/
    adminconfig.setEditPermission(editPer) ;
    adminconfig.setViewPermission(viewPer) ;
    service_.savePortalConfig(adminconfig, admin) ;
//    assertObject(service_.getPortalConfig(admin, admin).getEditPermission(),editPer,new PermissionComparator()) ;
//    assertObject(service_.getPortalConfig(admin, admin).getViewPermission(),viewPer,new PermissionComparator()) ;
    /*--------------addmin navigation-------------------------*/            
    assertTrue("expect root is home node '/'",adminnavigation.getUri().equals("/")) ;   
      
    assertTrue("expect to found some node ",adminnavigation.findNode("/home/organization")!=null) ;  
    assertTrue("expect to found some node ",adminnavigation.findNode("/home/portal")!=null) ;
    assertTrue("expect to found some node ",adminnavigation.findNode("/home/monitoring")!=null) ;
    assertTrue("expect to found some node ",adminnavigation.findNode("/home/monitoring")!=null) ;
    assertTrue("expect to found some node ",adminnavigation.findNode("/home/organization/management")!=null) ;
    System.out.println("======================Test complete================================") ;    
    
    System.out.println("======================Test portal config service with user1========") ;    
    /*------------Portal config data isn't created after user register---------*/
    PortalConfigData u1configdata = 
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, username1);
    assertTrue("expect Portal Config Data is null", u1configdata == null) ;
    
    NodeNavigationData u1navdata = 
      (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, username1) ;
    assertTrue("expect Portal Config Data is null", u1navdata==null) ;
    
    /*------------after user login, portal config data of users is created-----*/
    PortalConfig u1config = service_.getPortalConfig(username1, username1) ;
    Node user1Nav = service_.getNodeNavigation(username1, username1) ;    
    u1configdata =       
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, username1);    
    assertTrue("expect owner of portal config data is user1",u1configdata.getOwner().equals(username1)) ;       
    assertTrue("expect owner of portal config is 'portal'",u1config.getOwner().equals(username1)) ;
    assertTrue("expect the portal config of user1 is shared", u1config.isSharedLayout()) ;    
    assertTrue("expect no user is permited to edit permission", u1config.getEditPermission()==null) ;
    assertTrue("expect user can view permission",u1config.getViewPermission() != null) ;
    
    u1navdata = 
      (NodeNavigationData)hservice_.findOne(NodeNavigationData.class, username1) ;
    assertTrue("expect node navigation data is created", u1navdata!=null) ;                
    assertTrue("expect root is home node '/'",user1Nav.getUri().equals("/")) ;
    
    user1Nav = service_.getNodeNavigation("exo", "exo") ;    
    assertTrue("expect to found some node ",user1Nav.findNode("/home/my-portal")!=null) ;
    //assertTrue("expect to found some node ",user1Nav.findNode("/home/profile")!=null) ;
    assertTrue("expect to found some node ",user1Nav.findNode("/home/search")!=null) ;
    assertTrue("expect to found some node ",user1Nav.findNode("/home/sitemap")!=null) ;
    
    /*--------------------------test Page-----------------*/
    Query q = new Query(username1 , null, null ) ;
    List<PageDescriptionData> listPageBefore = service_.findAllPageDescriptions(q).getAll() ;
    assertTrue("expect pages of user1 is found",listPageBefore.size()>0) ;
    /*---------add page-------*/        
    Page page = createPage("/user1Page","user1Page",username1,editPer,viewPer) ;
    service_.savePage(page,username1) ;
    
    Page user1Page = service_.getPage(username1+":user1Page", username1) ;
    assertTrue("expect 'user1Page' is found", user1Page !=null ) ;    
    assertObject(page,user1Page,new PageComparator()) ;
    q = new Query(username1 , null, null ) ;
    List<PageDescriptionData> listPageAfter = service_.findAllPageDescriptions(q).getAll() ;
    assertTrue("Expect numbers of page is increase 1", listPageAfter.size()-listPageBefore.size()==1) ;    
    q = new Query(username1 , viewPer.toString() , null ) ;
    List<PageDescriptionData> list = service_.findAllPageDescriptions(q).getAll() ;    
    assertTrue("Expect 1 page ",  list.size() == 1) ;    
    /*--------edit page-------*/
    user1Page.setTitle("test") ;
    service_.savePage(user1Page, username1) ;
    user1Page = service_.getPage(username1+":user1Page", username1) ;
    assertTrue("expect page property is updated", user1Page.getTitle().equals("test")) ;    
    /*------remove a page----*/
    service_.removePage(username1+":user1Page",username1) ;
    assertTrue("expect page is removed",service_.getPage(username1+":user1Page", username1) == null) ;
    /*-------save page from page object----*/
    service_.savePage(user1Page,username1) ;
    /*-----expect the page before remove is the same with the page after save page object----*/
    assertObject(user1Page,service_.getPage(username1+":user1Page", username1),new PageComparator()) ;
    
    /*---------save page from Xml---------*/
//    String pageXml = service_.getPageAsXmlString(username1+":user1Page") ;
//    assertTrue("expect page is found", pageXml!=null) ;
//    service_.removePage(username1+":user1Page") ;        
//    assertTrue("expect page is removed",service_.getPage(username1+":user1Page") == null) ;
//    
//    service_.savePage(pageXml) ;
//    Page pagefromXml = service_.getPage(username1+":user1Page") ;
//    /*-----expect the page before remove is the same with the page after save from xml----*/
//    assertObject(user1Page,pagefromXml,new PageComparator()) ;
            
    /*-------Edit Node navigation of user1------*/    
    Node child = service_.createNodeInstance() ;
    child.setUri("/home/testNode") ;
    child.setName("testNode") ;
    child.setEditPermission(editPer) ;
    child.setViewPermission(viewPer) ;        
    user1Nav.addChild(child) ;
    
    child = service_.createNodeInstance() ;
    child.setUri("/home/user1Node") ;
    child.setName("user1Node") ;
    child.setEditPermission(editPer) ;
    child.setViewPermission(viewPer) ;        
    user1Nav.addChild(child) ;
    
    service_.saveNodeNavigation(username1,user1Nav, username1) ;
    user1Nav = service_.getNodeNavigation(username1, username1) ;    
    assertTrue("expect child node is inserted", user1Nav.findNode("/home/testNode")!=null) ;
    assertTrue("expect child node is inserted",user1Nav.findNode("/home/user1Node")!=null) ;
    assertTrue("expect to found some node ",user1Nav.findNode("/home/sitemap")!=null) ;    
    /*----edit node property----*/
    Node testNode = user1Nav.findNode("/home/testNode") ;
    assertTrue("test node description is null",testNode.getDescription()==null) ;
    testNode.setDescription("this is the test Node") ;
    service_.saveNodeNavigation(username1,user1Nav, username1) ;
    assertTrue("expect node property is updated",
        service_.getNodeNavigation(username1, username1).findNode("/home/testNode").getDescription()!=null) ;           
    /*------edit portal config property-----*/    
    assertTrue("expect portal congfig of use1 is shared",u1config.isSharedLayout()) ;
    /*---------user1 can't edit portal properties because it's portal is shared---*/
    Permission u1ViewPer = u1config.getViewPermission() ;    
    
    u1config.setViewPermission(viewPer) ;
    u1config.setEditPermission(editPer) ;
    service_.savePortalConfig(u1config, username1) ;
    /*--------expect portal properties isn's updated----*/
    PortalConfig u1configAfter = service_.getPortalConfig(username1, username1) ;    
    assertObject(u1configAfter.getViewPermission(),u1ViewPer,new PermissionComparator()) ;
    assertTrue("expect edit permission is still null",u1configAfter.getEditPermission() == null) ; 
    /*--------remove portal config and navigation -----*/
    String user1Xml = service_.getPortalConfigAsXmlString(username1, username1) ;    
    assertTrue("expect portal config xml is not null", user1Xml!=null) ;              
    
    service_.removePortalConfig(username1, username1) ;    
    service_.removeNodeNavigation(username1, username1) ;
    u1configdata = 
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, username1);
    assertTrue("expect Portal Config Data is removed", u1configdata==null) ;
    u1navdata = (NodeNavigationData)hservice_.findOne(NodeNavigationData.class,username1) ;
    assertTrue("expect node navigation data is removed",u1navdata==null) ;    
    /*-----------------save portal config from Portal config object---------*/         
    service_.savePortalConfig(u1config, username1) ;
    service_.saveNodeNavigation(username1,user1Nav, username1) ;
    u1configAfter = service_.getPortalConfig(username1, username1) ;    
    /*-----potal config before remve and after remove is the same----*/
    assertObject(u1config, u1configAfter, new PortalConfigComparator()) ;
    /*-------- save portal config from xml------*/
//    service_.removePortalConfig(username1) ;
//    service_.savePortalConfig(username1,user1Xml) ;
    
    System.out.println("======================Test complete================================") ;
    
    System.out.println("======================Test Portal config service with user2 =======") ;                
    PortalConfig u2config = service_.getPortalConfig(username2, username2) ;
    Node user2Nav = service_.getNodeNavigation(username2, username2) ;
        
    assertTrue("expect owner of portal config is 'user2'",u2config.getOwner().equals(username2)) ;            
    assertTrue("expect node navigation of is found",user2Nav!=null) ;    
    /*------user 2 is in group:'users' and membershiptype: 'member'---*/                 
    Membership mem = orgService_.getMembershipHandler().findMembershipByUserGroupAndType(username2,"/users","member") ;
    assertTrue("expect mem is found",mem!=null) ;
    
    comService_.removeCommunityPortal(comService_.getCommunityPortal("/users")) ;
    comService_.removeCommunityNavigation(comService_.getCommunityNavigation("/users")) ;
    /*-------------Now community config of user2 is null-----*/       
    assertTrue("expect portal community now is null",comService_.findCommunityPortal(username2)==null) ;
    assertTrue("expect navigation community of user2 is null",
        comService_.findCommunityNavigation(username2).size()==0) ;                
    /*-------Set community config for user2-------*/            
    User user2 = orgService_.getUserHandler().findUserByName(username2) ;
    assertTrue("expect user2 is created", user2!=null) ;
    Group group1 = orgService_.getGroupHandler().findGroupById("/"+Group1) ;
    assertTrue("expect group1 is created",group1!=null) ;   
    MembershipType  mtypeObj= orgService_.getMembershipTypeHandler().findMembershipType(memtype1) ;
    assertTrue("expect membershiptype 1 is created",mtypeObj!=null) ;
    //--------link user2 to group1 with membershiptype 1
    orgService_.getMembershipHandler().linkMembership(user2,group1,mtypeObj,true) ;
    CommunityPortal cp = comService_.createCommunityPortal() ;
    cp.setGroupId("/"+Group1) ;
    cp.setPortal(username1) ;
    cp.setMembership(memtype1) ;
    cp.setPriority(3) ;
    comService_.addCommunityPortal(cp) ;
   
    CommunityNavigation cn = comService_.createCommunityNavigation() ;
    cn.setGroupId("/"+Group1) ;
    cn.setMembership(memtype1) ;
    cn.setNavigation(username1) ;
    cn.setPriority(8) ;
    comService_.addCommunityNavigation(cn) ;
    
    CommunityPortal u2com= comService_.findCommunityPortal(username2) ;    
    assertTrue("expect community portal of user2 is found",comService_.findCommunityPortal(username2)!=null) ;
    assertTrue("expect community navigation of user2 is found", comService_.findCommunityNavigation(username2).size()>0) ;
    assertTrue("expect portal of community portal of user2 is: user1 ",u2com.getPortal().equals(username1)) ; 
    assertTrue("expect portal of community portal od user2 is: user1 ",u2com.getGroupId().equals("/"+Group1)) ;            
                           
    user2Nav = service_.getNodeNavigation(username2, username2) ;    
    assertTrue("expect node from user1", user2Nav.findNode("/home/testNode")!=null) ;    
    assertTrue("expect node from user1", user2Nav.findNode("/home/user1Node")!=null) ;
    /*-------remove child node in navigation of user1------*/
    user1Nav = service_.getNodeNavigation(username1, username1) ;
    user1Nav.removeChild("/home/testNode") ;
    service_.saveNodeNavigation(username1,user1Nav, username1) ;
    assertTrue("expect childnode of user1 is removed", user1Nav.findNode("/home/testNode")==null) ;
    /*-------this child node in navigation of user2 is removed,too-----*/
    user2Nav = service_.getNodeNavigation(username2, username2) ;    
//    assertTrue("expect childnode of user2 is removed,too", user2Nav.findNode("/home/testNode")==null) ;
    assertTrue("expectto have a childnode of user2 come from user1", user2Nav.findNode("/home/user1Node")!=null) ;            
    /*--------------------remove a user---------*/
    orgService_.getUserHandler().removeUser(username1,true) ;    
    u1configdata = 
      (PortalConfigData)hservice_.findOne(PortalConfigData.class, username1);
    assertTrue("expect Portal Config Data is removed", u1configdata==null) ;
    u1navdata = (NodeNavigationData)hservice_.findOne(NodeNavigationData.class,username1) ;
    assertTrue("expect node navigation data is removed",u1navdata == null) ;
    q = new Query(username1,null,null) ;
    List<PageDescriptionData> u1PageDesc = service_.findAllPageDescriptions(q).getAll() ;
    assertTrue("expect all page of user1 is removed",u1PageDesc.size()==0) ;
    /*---------now, user2 hasn't got navigation config from user1------*/
    user2Nav = service_.getNodeNavigation(username2, username2) ;
    assertTrue("expect no node is navigation of user2 come from user1", user2Nav.findNode("/home/user1Node")==null) ;
    /*-------remove all page of user 2---------*/
    service_.removePageOfOwner(username2, username2) ;
    q = new Query(username1,null,null) ;
    List<PageDescriptionData> u2pages = service_.findAllPageDescriptions(q).getAll() ;
    assertTrue("expect all pages of user2 is remove",u2pages.size() == 0) ;    
    System.out.println("======================Test complete================================") ;
    removeOrganizationData() ;
    }    
  
  private Permission createPermission(String groupId, String membership) throws Exception {
    Permission permission = new Permission() ;
    permission.setGroup(groupId) ;
    permission.setMembership(membership) ;
    return permission ;
  }
    
  private Page createPage (String id, String name,String owner,Permission editPer,Permission viewPer) throws Exception {
    Page page = new Page() ;
    page.setId(id) ;
    page.setName(name) ;
    page.setOwner(owner) ;
    page.setEditPermission(editPer) ;
    page.setViewPermission(viewPer) ;
    page.setTitle("testPage") ;
    page.setRenderer("default") ;
    page.setDecorator("default") ;    
    return page ;
  }
  
  private static class PermissionComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Permission per1 = (Permission)o1 ;
      Permission per2 = (Permission)o2 ;
      if((per1 == null)&&(per2 == null)) { return 0 ;}
      if((per1 == null)&&(per2 != null)) { return -1 ;}
      if((per1 != null)&&(per2 == null)) { return -1 ;}
      if((per1.toString()==null)&&(per2.toString()==null)) { return 0 ;}
      if(per1.toString().equals(per2.toString())) { return 0 ;}
      return -1;
    }
    
  }
  private static class PageComparator implements Comparator {

    public int compare(Object o1, Object o2) {
       Page page1 = (Page) o1 ;
       Page page2 = (Page) o2 ;
       if((page1.getOwner().equals(page2.getOwner()))
           &&(page1.getName().equals(page2.getName()))
           &&(page1.getId().equals(page2.getId()))
           &&(page1.getEditPermission().equals(page2.getEditPermission()))
           &&(page1.getViewPermission().equals(page2.getViewPermission()))) {return 0 ; }
      return -1;
    }
    
  }
  
  private static class PortalConfigComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      PortalConfig c1 = (PortalConfig)o1  ;
      PortalConfig c2 = (PortalConfig)o2  ;
      if((c1.getDecorator().equals(c2.getDecorator()))
          &&(c1.getRenderer().equals(c2.getRenderer()))
          &&(c1.getLocale().equals(c2.getLocale()))
          &&(c1.getOwner().equals(c2.getOwner()))) { return 0 ;}
      return -1;
    }    
  }
      
  protected String getDescription() {
    return "Test User Portal Configuration service" ;
  }
}
