/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component.logic;

import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.portal.config.PortalACL;
import org.exoplatform.portal.config.PortalConfigService;
import org.exoplatform.portal.config.model.Node;
import org.exoplatform.portal.config.model.Portlet;
import org.exoplatform.portal.faces.component.UIPage;
import org.exoplatform.portal.faces.component.UIPortal;
import org.exoplatform.portal.faces.component.UIPortalComponent;
import org.exoplatform.portal.faces.component.UIPortlet;
import org.exoplatform.portal.faces.component.Util;
import org.exoplatform.portal.session.RequestInfo;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 18, 2005
 * @version $Id$
 */
public class PortalComponentSecurity {
  private PortalACL portalACL_ ;
  
  public PortalComponentSecurity() {
    PortalConfigService configService =
      (PortalConfigService) PortalContainer.getComponent(PortalConfigService.class);
    portalACL_ = configService.getPortalACL() ;
  }
  
  public PortalComponentSecurity(PortalConfigService configService) {
    portalACL_ = configService.getPortalACL() ;
  }
  
  public void setCapabilities(UIPortal uiPortal) throws Exception {
    String remoteUser = SessionContainer.getInstance().getRemoteUser() ;
    boolean adminPortal =  false ;
    RequestInfo rinfo = (RequestInfo)SessionContainer.getComponent(RequestInfo.class);    
    if(rinfo.getAccessibility() != RequestInfo.PUBLIC_ACCESS) {
      adminPortal = portalACL_.hasEditPortalPermission(uiPortal.getPortalConfigModel(), remoteUser) ;      
    }
    if(adminPortal){
      uiPortal.setComponentAdminRole(true) ;
      setEditPortalCapability(uiPortal) ;
      setEditNavigationCapability(uiPortal) ;     
    } else {      
      uiPortal.setComponentAdminRole(false) ;   
      uiPortal.setEditPortalCapability(false) ;
      uiPortal.setEditNavigationCapability(false) ;
    }
    String portletOwner = uiPortal.getPortalConfigModel().getOwner() ;
    List descendants = Util.findDescendantsOfTypeInPortal(uiPortal, UIPortalComponent.class) ;
    for(int i = 0; i < descendants.size(); i++) {
      UIPortalComponent descendant = (UIPortalComponent) descendants.get(i) ;
      descendant.setComponentAdminRole(uiPortal.hasEditPortalCapability()) ;
      if(descendant instanceof UIPortlet) {
        UIPortlet uiPortlet = (UIPortlet) descendant ;
        updateSupportModes(uiPortlet, portletOwner, remoteUser, uiPortal.hasEditPortalCapability());
      }
    }
  }
  
  public void setCapabilities(UIPage uiPage) throws Exception {
    boolean adminPage = false ;
    RequestInfo rinfo = (RequestInfo)SessionContainer.getComponent(RequestInfo.class);
    String remoteUser = SessionContainer.getInstance().getRemoteUser() ;
    String portletOwner = uiPage.getPageModel().getOwner() ;
    if(!rinfo.getAccessibility().equals(RequestInfo.PUBLIC_ACCESS)) {
      adminPage = portalACL_.hasEditPagePermission(uiPage.getPageModel(), remoteUser) ;
    }
    uiPage.setComponentAdminRole(adminPage) ;
    List descendants = uiPage.findDescendantsOfType(UIPortalComponent.class) ;
    for(int i = 0; i < descendants.size(); i++) {
      UIPortalComponent descendant = (UIPortalComponent) descendants.get(i) ;
      descendant.setComponentAdminRole(adminPage) ;
      if(descendant instanceof UIPortlet) {
        UIPortlet uiPortlet = (UIPortlet) descendant ;
        updateSupportModes(uiPortlet, portletOwner, remoteUser, adminPage);
      }
    }
  }
  
  public void setEditPortalCapability(UIPortal uiPortal) throws Exception { 
    String remoteUser = SessionContainer.getInstance().getRemoteUser() ;
    boolean b = portalACL_.hasEditPortalPermission(uiPortal.getPortalConfigModel(), remoteUser) ;
    if(!uiPortal.getPortalConfigModel().isSharedLayout()) {
      uiPortal.setEditPortalCapability(b) ;
    } else {
      uiPortal.setEditPortalCapability(false) ;
    }
  }
  
  public void setEditNavigationCapability(UIPortal uiPortal) throws Exception { 
    String remoteUser = SessionContainer.getInstance().getRemoteUser() ;
    Node root = uiPortal.getRootNode() ;
    boolean b = portalACL_.hasEditNodePermission(root, uiPortal.getOwner(), remoteUser);
    uiPortal.setEditNavigationCapability(b) ;
  }
  
  private void updateSupportModes(UIPortlet uiPortlet, String portletOwner, 
                                  String remoteUser, boolean adminParent) throws Exception {
    Portlet model = uiPortlet.getPortletModel() ;
    if(!adminParent) {
      removePortletMode(uiPortlet, "edit") ;
    } else if(model.getEditPreferencesPermission() != null) {
      boolean editPref = portalACL_.hasEditPreferencesPermission(model, portletOwner, remoteUser);
      if(!editPref) removePortletMode(uiPortlet, "edit") ;
    } 
//    if(!"admin".equals(remoteUser)){
////      if(uiPortlet.getDisplayTitle() != null && uiPortlet.getDisplayTitle().toLowerCase().indexOf("map")>-1)
//        System.out.println("truong 3 "+portletOwner+" / "+remoteUser);
//      removePortletMode(uiPortlet, "config") ;
//    }
  }
  
  private void removePortletMode(UIPortlet uiPortlet, String mode) {
    List modes = uiPortlet.getHtmlSupportModes() ;
    if(modes == null ) return ;
    for(int i =0; i < modes.size(); i++ ) {
      if(mode.equals(modes.get(i))) {
        modes.remove(i) ;
        return ;
      }
    }
  }
}