/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.log.test;

import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.portal.log.PageLogData;
import org.exoplatform.portal.log.PortalActivitiesLogService;
import org.exoplatform.portal.log.Query;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.test.BasicTestCase;

/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestSkinConfigService.java,v 1.7 2004/07/24 16:34:06 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestPageActivitiesLogService  extends BasicTestCase {
  PortalActivitiesLogService service_ ;
  Query q ;
  
  public TestPageActivitiesLogService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    service_ = (PortalActivitiesLogService) PortalContainer.getInstance().
                getComponentInstanceOfType(PortalActivitiesLogService.class) ;
     q = new Query() ;
    //q.setId("pageInstanceId") ;
    GregorianCalendar calendar = new GregorianCalendar() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    q.setFromTimeSlot(calendar.getTime()) ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    q.setToTimeSlot(calendar.getTime()) ;
    q.setComponentId("") ;
  }

  public void tearDown() throws Exception {
  }
  
  public void testSave() throws Exception {
    service_.logPageAccess("page1"/*pageID*/, "admin"/*pageOwner*/, 20/*execTime*/, false/*error*/) ;
    service_.logPageAccess("page1", "admin", 50, false) ;
    service_.logPageAccess("page2", "exo",   20, true) ;
    Thread.sleep(2*1000) ;
    
    List list = service_.getPageLogDatas(q).getAll() ;
    
    assertEquals("expect  2 log data in db", 2, list.size()) ;
    PageLogData  page1data = (PageLogData) list.get(0) ;
    assertEquals("expect  page'owner is admin ", "admin", page1data.getPageOwner());
    assertEquals("expect  mydata is page1 ", "page1", page1data.getPageId()) ;
    assertEquals("expect  sum time access  of page1 is ", 70, page1data.getSummaryAccessTime()) ;
    assertEquals("expect  access counter of page1 is ", 2, page1data.getAccessCounter()) ;
    
    PageLogData page2data = (PageLogData) list.get(1) ;
    assertEquals("expect  page'owner is exo ", "exo", page2data.getPageOwner());
    assertEquals("expect access counter of page2 is ", 1, page2data.getAccessCounter()) ;
    assertEquals("expect error access counter of page2 is ", 1, page2data.getAccessErrorCounter()) ;
    
    service_.logPageAccess("page2", "exo", 20, false) ;
    service_.logPageAccess("page3", "test", 20, false) ;
    service_.logPageAccess("page4", "exo-test", 20, true) ; 
    Thread.sleep(2000) ;    
    
    resetHibernateSession() ;
    
    List nextList = service_.getPageLogDatas(q).getAll() ;
    assertEquals("expect  4 log data in db", 4, nextList.size()) ;
   
    
    page2data = (PageLogData) nextList.get(1) ;
    assertEquals("expect  page'owner is exo ", "exo", page2data.getPageOwner());
    assertEquals("expect  mydata is portal2 ", "page2", page2data.getPageId()) ;
    assertEquals("expect access counter of page2 is ", 2, page2data.getAccessCounter()) ;
    assertEquals("expect error access counter of page2 is ", 1, page2data.getAccessErrorCounter()) ;
    
    PageLogData page4data = (PageLogData) nextList.get(3) ;
    assertEquals("expect  page'owner is exo ", "exo-test", page4data.getPageOwner());
    assertEquals("expect  mydata is page4 ", "page4", page4data.getPageId()) ;
    assertEquals("expect access counter of page4 is ", 1, page4data.getAccessCounter()) ;
    assertEquals("expect error access counter of page4 is ", 1, page4data.getAccessErrorCounter()) ;
    assertEquals("expect summary access time of page4 is ", 20, page4data.getSummaryAccessTime()) ;
   
  }
  private  void  resetHibernateSession() {
    PortalContainer pcontainer = PortalContainer.getInstance() ;
    List<HibernateService> list = 
      pcontainer.getComponentInstancesOfType(HibernateService.class) ; 
    for(HibernateService hservice : list) hservice.closeSession() ;
    PortalContainer.setInstance(null) ;  
  }

  public void testQueryPagesLogChartData() throws Exception {
        
    List l = service_.getPageLogChartDatas(q) ;
    
    Object[] row0 = (Object[])l.get(0);
    assertEquals("expect  mydata1 pageId is","page1",/*owner*/(String)row0[0]) ;
    assertEquals("expect  access counter is",2,/*accessCounter*/Integer.parseInt(row0[1].toString()));
    assertEquals("expect  error access counter is",0,/*errorCounter*/Integer.parseInt(row0[2].toString())) ;
    assertEquals("expect  sum access time is",70,/*sumTime*/Integer.parseInt(row0[3].toString())) ;
    
    Object[] row1 = (Object[])l.get(1);
    assertEquals("expect  mydata2 pageId is","page2",row1[0].toString()) ;
    assertEquals("expect  access counter is",2,Integer.parseInt(row1[1].toString())) ;
    assertEquals("expect  error access counter is",1,Integer.parseInt(row1[2].toString())) ;
    assertEquals("expect  sum access time is",40,Integer.parseInt(row1[3].toString())) ;
    
    Object[] row2 = (Object[])l.get(2);  
    assertEquals("expect  mydata3 pageId is","page3",row2[0].toString()) ;
    assertEquals("expect  access counter is",1,Integer.parseInt(row2[1].toString())) ;
    assertEquals("expect  error access counter is",0,Integer.parseInt(row2[2].toString())) ;
    assertEquals("expect  sum access time is",20,Integer.parseInt(row2[3].toString())) ;
    
    Object[] row3 = (Object[])l.get(3);
    assertEquals("expect  mydata4 pageId is","page4", row3[0].toString()) ;
    assertEquals("expect  access counter is",1, Integer.parseInt(row3[1].toString())) ;
    assertEquals("expect  error access counter is",1, Integer.parseInt(row3[2].toString())) ;
    assertEquals("expect  sum access time is",20, Integer.parseInt(row3[3].toString())) ;
  }
}