package org.exoplatform.portlets.jobmanager.component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICalendarInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.scheduler.JobSchedulerService;
import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.SimpleTrigger;
import org.quartz.Trigger;

public class UIJobDetailForm extends UISimpleForm {
  
  public final static String START_DATE = "startDate".intern() ;
  public final static String END_DATE = "endDate".intern() ;
  public final static String REPEAT_COUNT = "repeatCount".intern() ;
  public final static String REPEAT_INTERVAL = "period".intern() ;
  public final static String CRON_EXPRESSION = "cronExpression".intern() ;
  public final static String JOB_NAME = "jobName".intern() ;
  public final static String TRIGGER_TYPE = "triggerType".intern() ;
  
	private JobDetail currentJob_;
	private Trigger currentTrigger_;
	private JobSchedulerService jobSchedulerService_;
	private static Log log_ = ExoLogger.getLogger("admin.UIJobDetailForm");
	private UICalendarInput startDateInput_ ;
  UICalendarInput endDateInput_ ;
  
	public UIJobDetailForm(JobSchedulerService jobSchedulerService) {
		super("UIJobDetailForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    
    jobSchedulerService_ = jobSchedulerService;
    
    UIStringInput name = new UIStringInput(JOB_NAME, "");
    name.setEditable(false);
    add(name);
    
    List<SelectItem> ttOptions = new ArrayList<SelectItem>();
    ttOptions.add(new SelectItem(UIJobManagerPortlet.SIMPLE_TRIGGER, UIJobManagerPortlet.SIMPLE_TRIGGER));
    ttOptions.add(new SelectItem(UIJobManagerPortlet.CRON_TRIGGER, UIJobManagerPortlet.CRON_TRIGGER));
    UISelectBox triggerType = new UISelectBox(TRIGGER_TYPE, UIJobManagerPortlet.SIMPLE_TRIGGER, ttOptions);
    triggerType.setEditable(false);
    add(triggerType);
    
    UIStringInput expression = new UIStringInput(CRON_EXPRESSION, "");
    expression.setRendered(false);
    add(expression);
    
    startDateInput_ = new UICalendarInput(START_DATE,"") ;
    startDateInput_.setDisplayTime(true) ;
    add(startDateInput_) ;
    endDateInput_ = new UICalendarInput(END_DATE,"") ;
    endDateInput_.setDisplayTime(true) ;
    add(endDateInput_) ;
    UIStringInput repeat = new UIStringInput(REPEAT_COUNT, "");
    repeat.setReadonly(false);
    add(repeat);
    UIStringInput period = new UIStringInput(REPEAT_INTERVAL, "");
    period.setReadonly(false);
    add(period);
    
    addActionListener(RescheduleActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
	}
	
	
	
	public void encodeBegin(FacesContext faces) throws IOException {
		super.encodeBegin(faces);
		setupForm();
	}

	public boolean isSimpleTrigger(){
		if (currentTrigger_ instanceof SimpleTrigger)
			return true;
		return false;
	}
     
	public boolean isCronTrigger(){    
		if (currentTrigger_ instanceof CronTrigger)
			return true;
		return false;
	}

	private void setupForm(){
		String triggerType = currentTrigger_.getClass().getSimpleName();
		
		getUIStringInput(JOB_NAME).setValue(currentJob_.getName());
		getUISelectBox("triggerType").setValue(currentTrigger_.getClass().getSimpleName());		
		if (triggerType.equals(UIJobManagerPortlet.CRON_TRIGGER)){
			CronTrigger t = (CronTrigger)currentTrigger_;
			getUIStringInput(CRON_EXPRESSION).setValue(t.getCronExpression());
			getUIStringInput(CRON_EXPRESSION).setRendered(true);
		}
		if (triggerType.equals(UIJobManagerPortlet.SIMPLE_TRIGGER)){      
			SimpleTrigger simpleTrigger = (SimpleTrigger)currentTrigger_;               
			getUIStringInput(REPEAT_COUNT).setValue(String.valueOf(simpleTrigger.getRepeatCount()));
			getUIStringInput(REPEAT_INTERVAL).setValue(String.valueOf(simpleTrigger.getRepeatInterval()));
		}
	}
	
	public String getJobName(){
		return currentJob_.getName();
	}
	
	public void setCurrentJob(JobDetail jobDetail){
		currentJob_ = jobDetail;
	}
	
	public void setCurrentTrigger(Trigger trigger){
		currentTrigger_ = trigger;
	}
	
	
	public static class CancelActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
    	UIJobDetailForm uicomp = (UIJobDetailForm)event.getSource();
    	uicomp.setRenderedSibling(UIJobList.class);
    }
	}
	
	public static class RescheduleActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UIJobDetailForm uicomp = (UIJobDetailForm)event.getSource();
      String triggerType = uicomp.getUISelectBox(UIJobDetailForm.TRIGGER_TYPE).getValue();
      Information iprovider = findInformationProvider(uicomp) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Date nextFireTime = null ;
      String fullName = null ;      
    	try{    	    	
    	if(triggerType.equals(UIJobManagerPortlet.CRON_TRIGGER)){
    		String expression = uicomp.getUIStringInput(UIJobDetailForm.CRON_EXPRESSION).getValue();
    		log_.info(UIJobManagerPortlet.CRON_TRIGGER + " " + expression);
    		CronTrigger cronTrigger = (CronTrigger)uicomp.currentTrigger_;
    		cronTrigger.setCronExpression(expression);
        fullName = cronTrigger.getFullJobName() ;
        nextFireTime = uicomp.jobSchedulerService_.rescheduleJob(cronTrigger.getName(), cronTrigger.getGroup(), cronTrigger);
    	}
      else if(triggerType.equals(UIJobManagerPortlet.SIMPLE_TRIGGER)){
    		int repeat = Integer.parseInt(uicomp.getUIStringInput(UIJobDetailForm.REPEAT_COUNT).getValue());
    		long period = Long.parseLong(uicomp.getUIStringInput(UIJobDetailForm.REPEAT_INTERVAL).getValue());
        String startTime = uicomp.getUIStringInput(UIJobDetailForm.START_DATE).getValue() ;
        String endTime = uicomp.getUIStringInput(UIJobDetailForm.END_DATE).getValue() ;
    		log_.info(UIJobManagerPortlet.SIMPLE_TRIGGER + " " + repeat + " " + period);
    		SimpleTrigger simpleStrigger = (SimpleTrigger)uicomp.currentTrigger_;
        if(startTime !=null && !startTime.equals("")) {
          try{
            Date startDate = ISO8601.parse(startTime).getTime() ;
            simpleStrigger.setStartTime(startDate) ; 
          }catch (Exception e) { 
            e.printStackTrace() ; 
           }          
        }
        if(endTime != null && !endTime.equals("")) {
          try {
            Date endDate = ISO8601.parse(endTime).getTime() ;
            simpleStrigger.setEndTime(endDate) ; 
          } catch (Exception e) {
            e.printStackTrace();
          }          
        }
    		simpleStrigger.setRepeatCount(repeat);
    		simpleStrigger.setRepeatInterval(period);
        fullName = simpleStrigger.getFullJobName() ;        
        nextFireTime = uicomp.jobSchedulerService_.rescheduleJob(simpleStrigger.getName(), simpleStrigger.getGroup(), simpleStrigger);
    	}      
    	}catch(Exception e){        
    		e.printStackTrace();
        iprovider.addMessage(new ExoFacesMessage(e.getMessage())) ;
    	}            
      Object [] arg =  {fullName,nextFireTime} ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobDetailForm.msg.reschedule-successfull"),arg)) ;
      uicomp.setRenderedSibling(UIJobList.class);
    }        
	}
}
