package org.exoplatform.portlets.jobmanager.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.scheduler.JobInfo;
import org.exoplatform.services.scheduler.JobSchedulerService;
import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.SimpleTrigger;
import org.quartz.Trigger;

public class UIJobList extends UIExoCommand {

  public final static String NOMAL_STATE = "Nomal".intern() ;
  public final static String PAUSING_STATE = "Pausing".intern() ;  
  public final static String COMPLETE_STATE = "Completed".intern();
  public final static String ERROR_STATE = "Error".intern() ;
  public final static String BLOCKED_STATE = "Blocked".intern() ;
  public final static String NONE_STATE = "None Trigger".intern() ;
  
	public final static String EXECUTE_ACTION = "execute".intern() ;
	public final static String REFRESH_ACTION = "refreshList".intern() ;
  public final static String PAUSE_ACTION = "pause".intern() ;  
  public final static String RESUME_ACTION = "resume".intern() ;
  
  public final static String PERIOD_TYPE = "Period".intern() ;
  public final static String CRON_TYPE = "Quartz Cron".intern() ;
  public final static String UNKNOWN_TYPE = "Unknown".intern() ;
  public final static String N_A = "N/A".intern() ;
      
	private static Parameter[] EXECUTE_PARAMS = {new Parameter(ACTION , EXECUTE_ACTION) };
	private static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION) } ;
	private static Parameter[] REFRESH_PARAMS = { new Parameter(ACTION,REFRESH_ACTION)} ;
  private static Parameter[] PAUESE_PARAMS = { new Parameter(ACTION,PAUSE_ACTION)} ;
  private static Parameter[] REMOVE_PARAMS = { new Parameter(ACTION,REMOVE_ACTION)} ;
  private static Parameter[] RESUME_PARAMS = { new Parameter(ACTION,RESUME_ACTION)} ;
  
	private JobSchedulerService jobSchedulerService_;
	private List jobList_ = new ArrayList();
	private Map<String, Trigger> triggerMap_ = new HashMap<String,Trigger>();
  
  private String filterType_ ; 
  
	private static Log log_ = ExoLogger.getLogger("admin.UIJobList");
	
	public UIJobList(JobSchedulerService jobSchedulerService) throws Exception {
		setId("UIJobList");
		setRendererType("VelocityRenderer");	
		jobSchedulerService_ = jobSchedulerService;    
		updateJobList();		
    
		addActionListener(EditActionListener.class, EDIT_ACTION) ;
		addActionListener(RefreshActionListener.class,REFRESH_ACTION) ;
		addActionListener(ExecuteActionListener.class, EXECUTE_ACTION) ;
    addActionListener(PauseActionListener.class,PAUSE_ACTION) ;
    addActionListener(RemoveActionListener.class,REMOVE_ACTION) ;
    addActionListener(ResumeActionListener.class,RESUME_ACTION) ;
	}
	
	public Parameter[] getEditParams() { return EDIT_PARAMS ; }
	public Parameter[] getExecuteParams()  { return EXECUTE_PARAMS ; }
	public Parameter[] getRefreshParams() { return REFRESH_PARAMS; }
  public Parameter[] getPauseParams() { return PAUESE_PARAMS; }
  public Parameter[] getRemoveParams() { return REMOVE_PARAMS; }
  public Parameter[] getResumeParams() { return RESUME_PARAMS; }
	
  public String getFilterType() { return filterType_ ; }
  public void setFilterType(String type) { this.filterType_ = type;  }
  
	public List getJobList() { return jobList_;  }
	
	public Trigger getTriggersOfJob(String jobFullName){
		return triggerMap_.get(jobFullName);
	}
  
  public JobSchedulerService getJobSchedulerService(){ return jobSchedulerService_ ;}
  
	public void updateJobList() throws Exception {   
    jobList_ = jobSchedulerService_.getAllJobs();     		
    triggerMap_.clear() ;
    for(Object obj: jobList_) {
      JobDetail jobDetail= (JobDetail)obj ;      
      Trigger[] tr = jobSchedulerService_.getTriggersOfJob(jobDetail.getName(), jobDetail.getGroup());
      if(tr.length>0) {
        triggerMap_.put(jobDetail.getFullName(), tr[0]); 
      }      
      
    }		
	}
  
  public String getScheduleType(Trigger trigger) {
    if(trigger instanceof SimpleTrigger) {
      return PERIOD_TYPE;
    }else if(trigger instanceof CronTrigger) {
      return CRON_TYPE ;
    }
    return UNKNOWN_TYPE;
  }
  
  public String getNextFireTime(Trigger trigger) {
    if(trigger.getNextFireTime() != null)
      return trigger.getNextFireTime().toString() ;
    return N_A ;
  }
  
  public String getPreviousFireTime(Trigger trigger) {    
    if(trigger.getPreviousFireTime() != null) 
      return trigger.getPreviousFireTime().toString() ;
    return N_A ;
  }
  
   public String getTriggerStatus(Trigger trigger) throws Exception {
      int status = jobSchedulerService_.getTriggerState(trigger.getName(),trigger.getGroup()) ;
      switch(status) {    
      case Trigger.STATE_NORMAL: return NOMAL_STATE ;
      case Trigger.STATE_BLOCKED: return BLOCKED_STATE ;
      case Trigger.STATE_COMPLETE: return COMPLETE_STATE ;
      case Trigger.STATE_ERROR: return ERROR_STATE ;
      case Trigger.STATE_PAUSED: return PAUSING_STATE ;    
      }
      return NONE_STATE ;
    }  
   
	public JobDetail getJob(String jobFullName){
    for(Object obj: jobList_) {
      JobDetail jobDetail = (JobDetail)obj;     
      if (jobFullName.equals(jobDetail.getFullName())){
        return jobDetail;
      }
    }
    return null ;
	}
	
	private void editDetail(String param){
		String jobFullName = param ;    
  	UIJobDetailForm jobDetailForm = (UIJobDetailForm)getSibling(UIJobDetailForm.class);
  	jobDetailForm.setCurrentJob(getJob(jobFullName));
  	jobDetailForm.setCurrentTrigger(triggerMap_.get(jobFullName));
  	setRenderedSibling(UIJobDetailForm.class);
	}
	
	static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	try{
    	UIJobList jobList = (UIJobList)event.getComponent();
    	String param = event.getParameter(OBJECTID);
    	jobList.editDetail(param);
    	}catch(Exception e){
    		log_.error(e.getMessage());
    		throw e;
    	}
    }
	}
	
	static public class RefreshActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	UIJobList jobList = (UIJobList)event.getComponent();
    	jobList.updateJobList();
    }
	}    
  
  public static class ExecuteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJobList jobList = (UIJobList)event.getComponent();
      String jobFullName = event.getParameter(OBJECTID) ;
      JobDetail selectedJob = jobList.getJob(jobFullName) ;
      JobSchedulerService jobSchedulerService = jobList.getJobSchedulerService() ;
      Information iprovider = findInformationProvider(jobList) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      try{                
        jobSchedulerService.executeJob(selectedJob.getName(),selectedJob.getGroup(),selectedJob.getJobDataMap()) ;
        Object[] args = {jobFullName } ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobList.msg.execute-succesfully"),args)) ;
      }catch (Exception e) {  
        iprovider.addMessage(new ExoFacesMessage(e.getMessage())) ; 
        //e.printStackTrace() ;
      }            
    }
    
  }
  
  public static class RemoveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJobList jobList = (UIJobList)event.getComponent();
      String jobFullName = event.getParameter(OBJECTID) ;
      JobDetail selectedJob = jobList.getJob(jobFullName) ;
      JobSchedulerService jobSchedulerService = jobList.getJobSchedulerService() ;
      JobInfo jInfo = new JobInfo(selectedJob.getName(),selectedJob.getGroup(),selectedJob.getClass()) ;
      Information iprovider = findInformationProvider(jobList) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      try{              
        boolean b = jobSchedulerService.removeJob(jInfo) ; 
        if(!b) {
          iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobList.msg.unable-remove"))) ;          
        }else {
          Object[] args = {jobFullName } ;
          iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobList.msg.remove-succesfully"),args)) ; 
        }        
      }catch (Exception e) {      
        iprovider.addMessage(new ExoFacesMessage(e.getMessage())) ; 
        //e.printStackTrace() ;
      }
      
    }    
  }
  
  public static class PauseActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJobList jobList = (UIJobList)event.getComponent();
      String jobFullName = event.getParameter(OBJECTID) ;
      JobDetail selectedJob = jobList.getJob(jobFullName) ;
      JobSchedulerService jobSchedulerService = jobList.getJobSchedulerService() ;
      Information iprovider = findInformationProvider(jobList) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      try{        
        jobSchedulerService.pauseJob(selectedJob.getName(),selectedJob.getGroup()) ;        
        Object[] args = {jobFullName } ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobList.msg.pause-succesfully"),args)) ;
      }catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(e.getMessage())) ;
        //e.printStackTrace() ;
      }
    }
    
  }
  
  public static class ResumeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {    
      
      UIJobList jobList = (UIJobList)event.getComponent();
      String jobFullName = event.getParameter(OBJECTID) ;
      JobDetail selectedJob = jobList.getJob(jobFullName) ;      
      JobSchedulerService jobSchedulerService = jobList.getJobSchedulerService() ;
      Information iprovider = findInformationProvider(jobList) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      try{
        jobSchedulerService.resumeJob(selectedJob.getName(),selectedJob.getGroup()) ;
        Object[] args = {jobFullName } ;
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIJobList.msg.resume-succesfully"),args)) ;
      }catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(e.getMessage())) ;
        //e.printStackTrace() ;
      }
      
    }    
  }
    
}
