/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.management.registry.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.faces.context.FacesContext;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.Constants;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.faces.component.model.PortletCategoryData;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletregistery.Portlet;
import org.exoplatform.services.portletregistery.PortletCategory;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 18 juin 2004
 */
public class UIAvailablePortlet extends UISimpleForm {
  
  private Map allPortletMetaData_;
  private PortletCategoryData categoryData_;

  public UIAvailablePortlet(PortletContainerService portletContainerService) {
    super("portletForm", "post", null);
  	setId("UIPortletList") ;
    setRendererType("SimpleFormVelocityRenderer") ;
    allPortletMetaData_ = new TreeMap(new PortletKeyComparator());
    allPortletMetaData_.putAll(portletContainerService.getAllPortletMetaData()) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public Map getAllPortletMetaData() { return allPortletMetaData_ ; }
  
  public class PortletKeyComparator implements Comparator {
    public int compare(Object o1, Object o2) throws ClassCastException {
      StringTokenizer key1 = new StringTokenizer((String) o1, Constants.PORTLET_META_DATA_ENCODER);
      StringTokenizer key2 = new StringTokenizer((String) o2, Constants.PORTLET_META_DATA_ENCODER);
      
      if (key1.countTokens() == 2 && key2.countTokens() == 2) {
      	key1.nextToken();
      	key2.nextToken();
      	return key1.nextToken().compareToIgnoreCase(key2.nextToken());
      }
      else return 0;
    }
  }

  public void setPortletCategoryData(PortletCategoryData category) {
  	categoryData_ = category ;
  }

  public String getSaveAction() { return SAVE_ACTION ; }
  public String getCancelAction() { return CANCEL_ACTION ; }
  
  static public class SaveActionListener extends ExoActionListener {
    public SaveActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception{
    	UIAvailablePortlet uicomp = (UIAvailablePortlet) event.getComponent() ;
      Map paramValuesMap = 
      	FacesContext.getCurrentInstance().getExternalContext().getRequestParameterValuesMap() ;
      List portletIDs = new ArrayList();
      for(int i = 0; i < uicomp.getAllPortletMetaData().size(); i++){
        String [] values = (String[])paramValuesMap.get("portlet_"+String.valueOf(i));        
        if(values  == null || values.length < 1) continue;        
        portletIDs.add(values[0]);
      }
      
      PortletCategory portletCategory = uicomp.categoryData_.getPortletCategory() ;
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      for (int i = 0; i < portletIDs.size(); i++) {
        String windowId = portletIDs.get(i).toString();
        String[] keys = StringUtils.split(windowId, "/") ;
        Portlet portlet = service.createPortletInstance();
        portlet.setDisplayName(keys[1]);
        portlet.setPortletApplicationName(keys[0]);
        portlet.setPortletName(keys[1]);
        List existingPortlet = service.getPortlets(portletCategory.getId()) ;
        boolean existFlag = false;        
        for(int j = 0; j < existingPortlet.size() ; j ++){
          Portlet existPortlet = (Portlet)existingPortlet.get(j) ;
          if(keys[1].equals(existPortlet.getPortletName())){
            existFlag = true ;
            break ;
          }             
        }        
        if(!existFlag)
          service.addPortlet(portletCategory, portlet);
      }
      Collection portlets = service.getPortlets(portletCategory.getId()) ;
      uicomp.categoryData_.reset(portletCategory, portlets) ;
      uicomp.setRenderedSibling(UIPortletCategories.class) ;
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAvailablePortlet uicomp = (UIAvailablePortlet) event.getComponent() ;
      uicomp.setRenderedSibling(UIPortletCategories.class);
    }
  }
}