package org.exoplatform.portlets.content.database.component;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.portlet.PortletPreferences;
import javax.sql.DataSource;

import org.apache.commons.beanutils.RowSetDynaClass;
import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.core.component.ComponentTemplate;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.naming.InitialContextInitializer;

/**
 * Author : Davide Dalle Carbonare davide.dallecarbonare@eng.it
 */

public class UIDBContent extends UIExoCommand implements ComponentTemplate {

  private String                    query_;

  private String[]                  queries_;

  private String[]                  parameters_;

  private String                    datasourceName_;

  private UIPageListIterator        uiPageListIterator_;

  private int                       nbPerPage_;

  private String                    template_;

  private List<String>              headers_;

  private int                       step_;

  private InitialContextInitializer contextInitializer_;

  public final static String        NEXT_ACTION    = "next".intern();

  public final static String        CLOSE_ACTION   = "closeDocument".intern();

  public final static String        RESTART_ACTION = "restart".intern();

  static Parameter[]                RESTART_PARAMS = { new Parameter(ACTION, RESTART_ACTION) };

  static Parameter[]                NEXT_PARAMS    = { new Parameter(ACTION, NEXT_ACTION) };

  static Parameter[]                STEP_PARAMS    = { new Parameter(ACTION, DatabaseUtil.STEP) };

  private static Log                log_           = ExoLogger.getLogger("content.UIDBContent");

  public UIDBContent(InitialContextInitializer contextInitializer) throws Exception {
    setId("UIDetailDocument");
    setRendererType("VelocityRenderer");
    contextInitializer_ = contextInitializer;
    
    PortletPreferences prefs = FacesUtil.getPortletPreferences();

    uiPageListIterator_ = new UIPageListIterator(new PageListDataHandler());
    getChildren().add(uiPageListIterator_);

    datasourceName_ = prefs.getValue(DatabaseUtil.DATASOURCE_NAME, "");

    for (String aQuery : prefs.getValues(DatabaseUtil.QUERIES, new String[] {})) {
      log_.info("query set: " + aQuery);
    }

    try {
      queries_ = prefs.getValues(DatabaseUtil.QUERIES, new String[] {});
      setStep(0);
      setQuery();
      setQueryParameters(new String[] {});
    } catch (Exception e) {
      e.printStackTrace();
    }

    try {
      nbPerPage_ = Integer.parseInt(prefs.getValue(DatabaseUtil.NB_PER_PAGE, ""));
    } catch (Exception e) {
      nbPerPage_ = 5;
    }

    template_ = prefs.getValue(DatabaseUtil.TEMPLATE, "");

    getData();

    addActionListener(NextActionListener.class, NEXT_ACTION);
    addActionListener(StepActionListener.class, DatabaseUtil.STEP);

    addActionListener(RestartActionListener.class, RESTART_ACTION);
    addActionListener(RestartActionListener.class, CLOSE_ACTION);

    log_.info("UIDBContent executed!!!!");

  }

  private void getData() {
    DataSource ds = null;
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    try {
      InitialContext ctx = new InitialContext();
      ds = (DataSource)contextInitializer_.getInitialContext().lookup(datasourceName_);

      //ds = (DataSource) ctx.lookup(datasourceName_);
      conn = ds.getConnection();

      log_.info("use query : " + query_);
      pstmt = conn.prepareStatement(query_);
      for (int i = 0; i < parameters_.length; i++) {
        log_.info("set parameter : " + i + " - " + parameters_[i]);
        pstmt.setString(i + 1, parameters_[i]);
      }
      rs = pstmt.executeQuery();

      setHeaders(rs);
      setPaginator(rs);

    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (rs != null) {
        try {
          rs.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
      if (pstmt != null) {
        try {
          pstmt.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
      if (conn != null) {
        try {
          conn.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
    }
  }

  private void setQuery() {
    query_ = queries_[step_];
  }

  private void setHeaders(ResultSet rs) throws Exception {
    headers_ = new ArrayList();
    ResultSetMetaData rsmt = rs.getMetaData();
    for (int i = 1; i <= rsmt.getColumnCount(); i++) {
      log_.info("header : " + i + " - " + rsmt.getColumnName(i));
      headers_.add(rsmt.getColumnName(i));
    }
  }

  public int getStep() {
    return step_;
  }

  public List getHeaders() {
    return headers_;
  }

  private void setPaginator(ResultSet rs) throws Exception {
    RowSetDynaClass rsdc = new RowSetDynaClass(rs);
    List list = rsdc.getRows();
    ObjectPageList objectPageList = new ObjectPageList(list, nbPerPage_);
    uiPageListIterator_.setPageList(objectPageList);
  }

  public Collection getDocumentPageList() throws Exception {
    return uiPageListIterator_.getPageListDataHandler().getDatas();
  }

  public String getTemplatePath() {
    return template_;
  }

  public void setQueryParameters(String[] params) {
    parameters_ = params;
  }

  private int setStep(int step) {
    step_ = step;
    return step_;
  }

  private int nextStep() {
    return step_++;
  }

  public Parameter[] addParameter(Parameter[] Params, String name, String value) {
    ArrayList<Parameter> arrayList = new ArrayList();
    for (Parameter p : Params) {
      arrayList.add(p);
    }
    arrayList.add(new Parameter(name, value));
    return (Parameter[]) arrayList.toArray(new Parameter[arrayList.size()]);
  }

  public Parameter[] getNextParams() {
    return NEXT_PARAMS;
  }

  public Parameter[] getStepParams() {
    return STEP_PARAMS;
  }

  public Parameter[] getRestartParams() {
    return RESTART_PARAMS;
  }

  static public class NextActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDBContent uicomp = (UIDBContent) event.getComponent();

      Map params = event.getParameterMap();
      int i = 1;
      ArrayList<String> queryParams = new ArrayList();
      while (params.containsKey(DatabaseUtil.PARAM + i)) {
        log_.info("parameter " + DatabaseUtil.PARAM + i + " : "
            + (String) params.get(DatabaseUtil.PARAM + i));
        queryParams.add((String) params.get(DatabaseUtil.PARAM + i));
        params.remove(DatabaseUtil.PARAM + i);
        i++;
      }
      uicomp.setQueryParameters((String[]) queryParams.toArray(new String[queryParams.size()]));
      uicomp.nextStep();
      uicomp.setQuery();
      uicomp.getData();
    }
  }

  static public class StepActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIDBContent uicomp = (UIDBContent) event.getComponent();

        Map params = event.getParameterMap();
        int i = 1;
        ArrayList<String> queryParams = new ArrayList();
        while (params.containsKey(DatabaseUtil.PARAM + i)) {
          log_.info("parameter " + DatabaseUtil.PARAM + i + " : "
              + (String) params.get(DatabaseUtil.PARAM + i));
          queryParams.add((String) params.get(DatabaseUtil.PARAM + i));
          params.remove(DatabaseUtil.PARAM + i);
          i++;
        }
        uicomp.setQueryParameters((String[]) queryParams.toArray(new String[queryParams.size()]));

        String step = (String) params.get(DatabaseUtil.STEP);
        int s = Integer.parseInt(step);
        uicomp.setStep(s);

        uicomp.setQuery();
        uicomp.getData();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  static public class RestartActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDBContent uicomp = (UIDBContent) event.getComponent();
      uicomp.setQueryParameters(new String[] {});
      uicomp.setStep(0);
      uicomp.setQuery();
      uicomp.getData();
    }
  }
}
