package org.exoplatform.portlets.content.database.component;

import java.util.ResourceBundle;

import javax.portlet.PortletPreferences;

import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;

/**
 * Author : Davide Dalle Carbonare
 *          davide.dallecarbonare@eng.it
 */

public class UIDBContentConfig extends UISimpleForm {

	private UIStringInput datasourceName_;
	private UITextArea queries_;
	private UIStringInput nbPerPage_;
	private UIStringInput template_;
	
	public UIDBContentConfig() throws Exception {
		super("DBContentConfig","post",null);		
	    setRendererType("SimpleFormVelocityRenderer");
	    	    
	    PortletPreferences prefs = FacesUtil.getPortletPreferences();   
	    
	    datasourceName_ = new UIStringInput(DatabaseUtil.DATASOURCE_NAME, prefs.getValue(DatabaseUtil.DATASOURCE_NAME, ""));
	    add(datasourceName_);
	    
	    String queryList = "";
	    for (String s : prefs.getValues(DatabaseUtil.QUERIES, new String[] {})){
	    	queryList += s + ";";
	    }
	    queries_ = new UITextArea(DatabaseUtil.QUERIES, queryList);
	    add(queries_);
	    
	    nbPerPage_ = new UIStringInput(DatabaseUtil.NB_PER_PAGE,prefs.getValue(DatabaseUtil.NB_PER_PAGE,"")) ;
	    add(nbPerPage_) ;
	    
	    template_ = new UIStringInput(DatabaseUtil.TEMPLATE,prefs.getValue(DatabaseUtil.TEMPLATE,"")) ;
	    add(template_) ;
	 
	    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
	}
	
	private void saveDatabaseConfig(String datasourceName, String[] queries, String nbPerPage, String template) throws Exception {
		PortletPreferences prefs = FacesUtil.getPortletPreferences();
		
		prefs.setValue(DatabaseUtil.DATASOURCE_NAME, datasourceName);
		prefs.setValues(DatabaseUtil.QUERIES, queries);
		prefs.setValue(DatabaseUtil.NB_PER_PAGE, nbPerPage);
		prefs.setValue(DatabaseUtil.TEMPLATE, template);

		prefs.store();
	}
	
	public static class SaveActionListener extends ExoActionListener {
	    public void execute(ExoActionEvent event) throws Exception {
	    	UIDBContentConfig config = (UIDBContentConfig)event.getSource();
	    	Information iprovider = findInformationProvider(config);
	    	ResourceBundle res = getApplicationResourceBundle();
	    	
	    	String datasourceName = config.getUIInput(DatabaseUtil.DATASOURCE_NAME).getValue();
	    	String queries = config.getUIInput(DatabaseUtil.QUERIES).getValue();
	    	String nbPerPage = config.getUIInput(DatabaseUtil.NB_PER_PAGE).getValue() ;
	    	String template = config.getUIInput(DatabaseUtil.TEMPLATE).getValue() ;
	    	
	    	String[] list = queries.split(";");
	    	
	    	config.saveDatabaseConfig(datasourceName, list, nbPerPage, template);
	    	
	    	iprovider.addMessage(new ExoFacesMessage(res.getString("UIDBContentConfig.msg.save-config-successfully"))) ;
	    }
	}
}
