/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.security.component;

import java.util.GregorianCalendar;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIDateTimeInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.ticket.Ticket;
import org.exoplatform.services.ticket.TicketService;
import org.exoplatform.services.ticket.impl.TicketImpl;
/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 18, 2006
 */
public class UITicketUpdate extends UISimpleForm  {

  private static String UPDATE = "update"; 
  private static String BACK = "back";
  static Parameter[] UPDATE_PARAMS   = { new Parameter(ACTION, UPDATE)};
  static Parameter[] BACK_PARAMS     = { new Parameter(ACTION, BACK) };
  protected UIStringInput user_,permission_,creator_,limit_,desc_,id_ ;
  private UIDateTimeInput toDate_;
  protected TicketService service_;
  private GregorianCalendar calendar;
  private String creator;
  private String ticketId_="";
  protected PortalContainer pcontainer;
  
  public UITicketUpdate() {
    super("updateForm","post",null);
    setId("UITicketUpdate") ;
    setRendererType("SimpleFormVelocityRenderer");
    pcontainer =  PortalContainer.getInstance() ;
    service_ = (TicketService)pcontainer.getComponentInstanceOfType(TicketService.class) ;
    creator=pcontainer.getSessionManager().getLiveSessions().get(0).getRemoteUser() ;
    //  ---------------- Time-------------------------
    calendar = new GregorianCalendar() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    
    user_ = new UIStringInput("users","") ;
    
    permission_=new UIStringInput("permission","");
    creator_=new UIStringInput("creator",creator);
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    toDate_=new UIDateTimeInput("accesstime",calendar.getTime());
    limit_=new UIStringInput("limit","-1");
    desc_=new UIStringInput("description","");
    id_=new UIStringInput("idticket","");
    id_.setEditable(false) ;
    
    add(user_) ;
    add(permission_);
    add(creator_);
    add(toDate_);
    add(limit_);
    add(desc_);
    add(id_);
    addActionListener(UpdateActionListerner.class, UPDATE);
    addActionListener(BackActionListerner.class, BACK);
  }
  
  public String getUpdateParams() { return UPDATE ; }
  public String getBackParams() { return BACK ; }
  
  public void setDefaultValues(String ticketId) throws Exception{
    TicketImpl ticketImpl = null ;
    ticketId_ = ticketId ;
    try {
       ticketImpl =(TicketImpl) service_.getTicket(ticketId);
    } catch (Exception e) {
      e.printStackTrace();
    }
    id_.setEditable(false);
    id_.setValue(ticketId);
    String user = ticketImpl.getUsers();
    if(user == null || user == "") user = "";
    user_.setValue(user);
    toDate_.addTime(ticketImpl.getLastAccessTime());
    limit_.setValue(""+ticketImpl.getLimitAccess());
    creator_.setValue(ticketImpl.getCreator());
    permission_.setValue("");
    desc_.setValue(ticketImpl.getDescription());
    
  }
  
  static public class UpdateActionListerner extends ExoActionListener{
    private UITicketUpdate uiForm;
    private TicketImpl ticketImpl;
    private HibernateService hservice_ ;
    private boolean update = true ;
    public void execute(ExoActionEvent event) throws Exception {
        uiForm = (UITicketUpdate) event.getSource();
        ticketImpl = (TicketImpl) uiForm.service_.getTicket(uiForm.ticketId_);
        hservice_ = (HibernateService)uiForm.pcontainer.getComponentInstanceOfType(HibernateService.class);
        String creator = uiForm.creator_.getValue();
        
        String users = uiForm.user_.getValue();
        if(users == "") users = null ;
        
        int limitAccess = 0;
        try{
          limitAccess = Integer.parseInt(uiForm.limit_.getValue());
        }catch (Exception e) {
          e.printStackTrace();
        }
        
        String description = uiForm.desc_.getValue();
        if(description == "") description = null;
        
        String permission = uiForm.permission_.getValue();
        if(permission == "") permission = null ;
        
        long createTime = ticketImpl.getCreationTime().getTime();
        long longTime = uiForm.getUIDateTimeInput("accesstime").
            getCalendar().getTimeInMillis() - createTime ;
        
        ticketImpl.setCreator(creator);
        ticketImpl.setUsers(users);
        ticketImpl.setDescription(description);
        ticketImpl.setPermissions(permission);
        ticketImpl.setLimitAccess(limitAccess);
        ticketImpl.setAllowedUsers(null);
        
        if(limitAccess < -1) ticketImpl.setStatus(Ticket.LOCKED_STATUS);
        if(limitAccess == -1) ticketImpl.setStatus(Ticket.UNLIMITED_ACCESS);
        if(limitAccess == 0) ticketImpl.setStatus(Ticket.LOCKED_STATUS);
        if(limitAccess > 0) {
          int count =limitAccess -ticketImpl.getAccessCounter();
          if(count <=0 ) ticketImpl.setStatus(Ticket.LOCKED_STATUS);
          else ticketImpl.setStatus(Ticket.VALID_STATUS);
        }
        
        if(longTime > 0) {
          ticketImpl.setLiveTime(longTime);
        }else update = false ;
        if(update){
          Ticket ticket = (Ticket) ticketImpl;
          hservice_.save(ticket);
        }
        
    }
  }
  
  static public class BackActionListerner extends ExoActionListener{

    public void execute(ExoActionEvent event) throws Exception {
         UITicketUpdate ticketUpdate = (UITicketUpdate) event.getComponent();
         UITicketMonitor ticketMonitor = (UITicketMonitor)ticketUpdate.getSibling(UITicketMonitor.class);
         ticketMonitor.update(null);
         ticketUpdate.setRenderedSibling(UITicketMonitor.class);
    }   
  }
}
