/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.event.UIComponentObserver;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;


/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIViewUserInGroup.java,v 1.1 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIViewUserInGroup extends UIExoCommand {  
  
	UIPageListIterator uiPageIterator_ ;
  UIMembershipForm uiMembershipForm_ ;
  UISelectUserManager uiSelectUserManager_ ;
  private OrganizationService service_ ;
  private String groupId_ ;
  private List<MembershipUser> memberships_ ;
  private boolean isShowMember_  = true;
	static Parameter[] DELETE =  { new Parameter(ACTION , DELETE_ACTION) } ;
	
  public UIViewUserInGroup(OrganizationService service) throws Exception {
    setRendererType("VelocityRenderer") ;
    uiMembershipForm_ = new UIMembershipForm() ;
    uiMembershipForm_.setRendered(false);
    uiMembershipForm_.setId("UIMembershipFormForGroup") ;
    uiSelectUserManager_ = new UISelectUserManager(service) ;
    uiSelectUserManager_.setRendered(false) ;
    //uiSelectUserManager_.setId("UISelectUserManager") ;
		uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    service_ = service ;
    getChildren().add(uiPageIterator_) ;
    getChildren().add(uiMembershipForm_) ;
    getChildren().add(uiSelectUserManager_) ;
    addActionListener(DeleteUserActionListener.class, DELETE_ACTION) ;    
    
    memberships_ = new ArrayList<MembershipUser>();
	}
  
  public Parameter[] getDeleteParams() { return DELETE; };
  
  public Collection getListUserInGroup() throws Exception {
    memberships_.clear();
    if(groupId_ != null) {
      Iterator it = uiPageIterator_.getPageListDataHandler().getDatas().iterator() ;
      boolean add = true;
      while(it.hasNext()){
        User user = (User)it.next() ;
        add = true;
        for(MembershipUser ele : memberships_ ){
          if(ele.getUser() != user) continue;
          add = false;
          break;
        }
        if(add) loadMemberships(user) ;
      } 
    }    
 
    return  memberships_;
  } 
  
  private void loadMemberships(User user) throws Exception{
    Collection mt = service_.getMembershipHandler().findMembershipsByUserAndGroup(user.getUserName(), groupId_) ;
    Iterator it = mt.iterator() ;
    while(it.hasNext()){
      Membership type = (Membership)it.next() ;
      memberships_.add(new MembershipUser(user, type.getMembershipType(), type.getId()));
    }
  }
  
  private List<MembershipUser> getMemberships() { return memberships_; }
  
  public void registerComponentObserver(UIExoComponent parent)  {
	  UIGroupExplorer uiExplorer = 
      (UIGroupExplorer) parent.getAncestorOfType(UIGroupExplorer.class) ;
    uiExplorer.addObserver(new GroupChangeObserver());
	}
	
  public boolean isShowMembershipForm() {
   return isShowMember_ ; 
  }
  public void setShowMembership(boolean value) {
    isShowMember_ = value ;
  }
  
  public void update() throws Exception {
    UIGroupExplorer uiExplorer =  (UIGroupExplorer) getAncestorOfType(UIGroupExplorer.class) ;
    uiPageIterator_.setPageList(uiExplorer.getMemberOfTheCurrentGroup()) ;
    Group group  = uiExplorer.getCurrentGroup();
    groupId_ = group.getId();    
  }
  
  public static class MembershipUser {    
    
    private User user;
    private String mtype;
    private String id;
    
    public MembershipUser(User user, String mtype, String id){
      this.mtype = mtype;
      this.user = user;
      this.id = id;
    }

    public String getMembershipType() { return mtype; }
    public void setMembershipType(String mtype) { this.mtype = mtype; }

    public User getUser() { return user; }
    public void setUser(User user) { this.user = user; }
    
    public String getId(){ return id; }
  }
  
	public class GroupChangeObserver extends UIComponentObserver {
	  public void onChange(UIExoComponent target) throws Exception  {
	    UIGroupExplorer uiExplorer = (UIGroupExplorer) target ;
      uiPageIterator_.setPageList(uiExplorer.getMemberOfTheCurrentGroup()) ;
      Group group  = uiExplorer.getCurrentGroup();
      if(group == null) {
        uiMembershipForm_.setRendered(false);
      } else {
        groupId_ = group.getId();
        uiMembershipForm_.setRendered(true);
        uiMembershipForm_.populateFormByGroup(group.getId());
      }
	  }
	}
  static public class DeleteUserActionListener extends ExoActionListener  {
    public DeleteUserActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIViewUserInGroup uiUserInGroup = (UIViewUserInGroup) event.getComponent() ;
      String id = event.getParameter(OBJECTID) ;    
      if(id == null) return;
      uiUserInGroup.service_.getMembershipHandler().removeMembership(id, true) ;            
      uiUserInGroup.update() ;      
    }
  }
  
}
