/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web;

import groovy.lang.GroovyClassLoader;

import java.net.URL;
import java.util.Date;
import java.util.List;

import org.exoplatform.test.web.report.HTMLReport;
import org.exoplatform.test.web.report.TextReport;

/**
 * May 22, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: WebLoadRunner.java,v 1.1 2004/10/11 23:36:03 tuan08 Exp $
 **/
public class WebLoadRunner  implements Runnable {
  private int numberOfWorker_ = 1 ;
  private int numberOfTask_ =  1 ;
  private boolean validateWellFormedXhtml_  = true ;
  private boolean validateWebUnit_ = true ; 
  private String defaultURL_ ;
  private boolean verbose_ = false ;
  
  private WorkerThread[] workers_ ;
  private Tasks tasks_ ;
  
  public WebLoadRunner() {
  }
  
  public WebLoadRunner(int numberOfUser, int numberOfTask) {
    numberOfWorker_ = numberOfUser ;
    numberOfTask_ = numberOfTask ;
  }
  
  public void setValidateWellFormedXhtml(boolean b) {  validateWellFormedXhtml_ = b ; }
  public void setValidateWebUnit(boolean b) {  validateWebUnit_ = b ; }
  public void setDefaultURL(String s) { defaultURL_ = s ; }
  public void setNumberOfWorker(int number) { numberOfWorker_ = number ; }
  public void setNumberOfTask(int number) { numberOfTask_ = number ; }
  
  public boolean isVerbose() { return verbose_ ; }
  public void    setVerbose(boolean b) { verbose_ = b ; }
  
  public void run(ExoWebClient webClient) {
    if(isRunning()) return ;
    tasks_ = new Tasks(numberOfTask_ , webClient);
    ThreadGroup threadGroup_ = new ThreadGroup("Group Of Worker") ;
    int limit = numberOfWorker_ ;
    int unfinishedTaskCounter = tasks_.getUnfinishedTaskCounter() ;
    if (limit > unfinishedTaskCounter) limit = unfinishedTaskCounter ;
    workers_ = new WorkerThread[limit] ;
    for (int i = 0; i < limit ;  i++) {
      workers_[i] =  new WorkerThread(threadGroup_, "worker-" + i, tasks_) ; 
      workers_[i].setPriority(Thread.MIN_PRIORITY) ; 
      workers_[i].start() ;     
    }
  }
  
  public boolean isRunning() {
    if(workers_ == null) return false ;
    for(int i = 0 ; i < workers_.length; i++) {
      if(workers_[i].isAlive()) return true ;
    }
    return false ;
  }
  
  public void stop() {
    if(workers_ == null) return ;
    for(int i = 0 ; i < workers_.length; i++) {
      if(workers_[i].isAlive()) {
        workers_[i].interrupt();
      }
    }
  }
  
  public int getUnfinishedTaskCounter() {
    if(tasks_ == null) return 0 ;
    return tasks_.getUnfinishedTaskCounter() ;
  }
  
  public void runSuites(TestSuites suites) throws Exception {
    String user  = "client"+Long.toString(new Date().getTime()) ;
    ExoWebClient webClient = new ExoWebClient(user, defaultURL_);
    webClient.setValidateWebUnit(validateWebUnit_) ;
    webClient.setValidateWellFormedXhtml(validateWellFormedXhtml_) ;
    webClient.setSuites(suites);
    webClient.setVerbose(verbose_) ;
    run(webClient) ;
    while(isRunning()) Thread.sleep(1000) ;
  }
  
  public void run() {
    try {
      String runnableScript =  System.getProperty("script") ;
      if(runnableScript ==  null) {
        System.out.println("You need the argument -Dscript=path/script.groovy") ;
        return ;
      }
      Object object = createGroovyObject(runnableScript) ;
      if(object instanceof WebUnitSuite) {
        WebUnitSuite suite = (WebUnitSuite) object ;
        List requiredSuites = suite.getRequiredSuites() ;
        TestSuites suites = new TestSuites() ;
        if(requiredSuites != null) {
          for(int i = 0; i < requiredSuites.size(); i++) {
            WebUnitSuite rsuite = (WebUnitSuite)requiredSuites.get(i) ;
            suites.addSuite(rsuite) ;
          }
        }
        suites.addSuite(suite) ;
        runSuitesWithSysProperties(suites) ;
      } else if(object instanceof TestSuites){
        TestSuites suites = (TestSuites) object ;
        runSuitesWithSysProperties(suites) ;
      } else if(object instanceof Runnable){
        Runnable runnable = (Runnable) object ;
        runnable.run() ;
      } else {
        throw new Exception("The object: " + object.getClass().getName() +
                            " is not an instance of  Runnable or WebUnitSuite or TestSuites") ;
      }
    } catch(Exception ex) {
      ex.printStackTrace() ;
    }
    
  }
  
  private void runSuitesWithSysProperties(TestSuites suites) throws Exception {
    String url =  System.getProperty("url") ;
    if(url ==  null) throw new Exception("You don't specify a  url") ;
    setDefaultURL(url) ;
    setNumberOfWorker(geIntegerPropertyValue("worker", 1)) ;
    setNumberOfTask(geIntegerPropertyValue("task", 1)) ;
    setValidateWebUnit("true".equals(System.getProperty("validate-unit"))) ;
    setValidateWellFormedXhtml("true".equals(System.getProperty("validate-xhtml"))) ;
    setVerbose("true".equals(System.getProperty("verbose"))) ;
    runSuites(suites) ;
    
    String reportType = System.getProperty("type") ;
    if("html".equals(reportType)) {
      String outputDir = System.getProperty("outputDir") ;
      if(outputDir == null) outputDir =  "target/report" ;
      String groovyPath = System.getProperty("outputDir") ;
      new HTMLReport(suites, outputDir, groovyPath).report() ;
    } else {
      String report =  System.getProperty("report") ;
      new TextReport(suites, report).report() ;
    }
  }
  
  private static int geIntegerPropertyValue(String key, int defaultValue) {
    String valueStr =  System.getProperty(key) ;
    if (valueStr == null) return defaultValue ;
    try {
      return Integer.parseInt(valueStr) ;
    } catch (Exception ex) {
      System.out.println("Cannot convet value " + valueStr + " to an integer, use the default value") ;
    }
    return defaultValue ;
  }
  
  private static  Object createGroovyObject(String groovyscript) throws Exception {     
    ClassLoader parentLoader =  Thread.currentThread().getContextClassLoader() ;
    GroovyClassLoader classloader =  new GroovyClassLoader(parentLoader) ;
    URL  classURL = classloader.getResource(groovyscript) ;
    Class clazz = classloader.parseClass(classURL.openStream()) ;
    return clazz.newInstance() ;
  }
  
  static public void main(String[] args) throws Exception {
    WebLoadRunner runner =  new WebLoadRunner() ;
    runner.run() ;
  }
}
