/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web.unit;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.test.web.ExoWebClient;
import org.exoplatform.test.web.WebUnitMonitor;
import org.exoplatform.test.web.condition.Condition;
import org.exoplatform.test.web.validator.Validator;

import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;

/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: WebUnit.java,v 1.1 2004/10/11 23:36:06 tuan08 Exp $
 **/
abstract public class WebUnit {
  final static public String ACTION = "op" ;
  final static public int  STATUS_OK = 0 ;
  final static public int  STATUS_BROKEN = 1 ;
  final static public int  STATUS_INCOMPLETE = 2 ;
  final static public int  STATUS_REVIEW = 3 ;

  private String name_ ;
  private String description_ ;
  protected List validators_ ;
  protected List conditions_ ;
  protected WebUnitMonitor monitor_ ;
  protected String blockId_;
  protected String lastResponseText_  ;
  protected String comment_ ;
  protected int status_ = STATUS_OK  ;
  protected boolean stop_ = false ;

  public WebUnit() {   
    name_ = getClass().getName() ;
    description_ = name_ ;
    monitor_ = new WebUnitMonitor() ;
  }

  public WebUnit(String name , String desc) {
    name_ = name ;
    description_ = desc ;
    monitor_ = new WebUnitMonitor() ;
  }

  public void reset() { 
    monitor_.reset() ;
    stop_ = false ;
    lastResponseText_ =  null ;
  }

  public String getBlockId() { return blockId_ ; }

  public WebUnit setBlockId(String id) { 
    blockId_ = id ;
    return this ;
  }

  public String getName() { return name_ ; }
  public void  setName(String name) { name_ = name ; }

  public String getActionDescription() { return "No Description" ; }

  public String getDescription() { 
    if(description_ == null) return "No Description" ;
    return description_ ; 
  }

  public String getExtraInfo() { return "N/A" ; }

  public int getStatus() { return status_ ; }
  public WebUnit setStatus(int status) { 
    status_ =  status ;
    return this ;
  }

  public boolean isStop() { return stop_ ; }
  public void    setStop(boolean b) { stop_ = b ; }

  public String getComment() { return comment_ ; }
  public WebUnit   setComment(String s) { 
    comment_ = s ;
    return this ;
  }

  public String getLastResponseText() { return lastResponseText_ ;}
  public void setLastResponseText(String text) { lastResponseText_ = text ; }

  abstract public WebResponse execute(WebResponse previousResponse, WebTable block, 
      ExoWebClient client) throws Exception ;

  static public void print(String mesg) {
    System.out.println(mesg) ;
  }

  public List getValidators() { return validators_ ; }

  public WebUnit addValidator(Validator validator) {
    if(validators_ == null) validators_ = new ArrayList() ;
    validators_.add(validator) ;
    return this ;
  }


  public List getConditions() { return conditions_ ; }

  public WebUnit addCondition(Condition condition) {
    if(conditions_ == null) conditions_ = new ArrayList() ;
    conditions_.add(condition) ;
    return this ;
  }

  public WebUnitMonitor getMonitor() { return monitor_ ; }



  static public String getRealValue(ExoWebClient client, String value)  {    
    if(value.length() < 3 )  return value ;    
    int index = value.indexOf('#') ;   
    if(index<0) return value ;
    if(value.charAt(index+1)!='{' || value.indexOf('}')<=index)
      return value ;        
    String attName = value.substring(index+2,value.indexOf('}')) ;    
    String attValue = (String)client.getAttribute(attName) ;
    if(attValue == null) return value;
    String pattern = "#{" + attName + "}" ;    
    value = value.replace(pattern,attValue) ;
    return value ;
  }

  public boolean checkConditions(WebResponse response ,WebTable block, ExoWebClient client)  throws Exception {
    if(conditions_ != null) {
      for(int k = 0 ; k < conditions_.size(); k++) {
        Condition condition = (Condition) conditions_.get(k) ;
        if (!condition.checkCondition(response, block, client)) {
          return false ;
        }
      }
    }
    return true ;
  }

  public boolean validate(WebResponse response , ExoWebClient client)  throws Exception {
    if(validators_ != null) {
      for(int k = 0 ; k < validators_.size(); k++) {
        Validator validator = (Validator) validators_.get(k) ;
        if (!validator.validate(response, client)) {  
          return false  ;
        }
      }
    }
    return true ;
  }

  public void log(long executionTime , int contentLength, boolean error, boolean malformed) {
    monitor_.log(executionTime, contentLength, error, malformed) ;
  }

  public String toString() {
    StringBuffer b = new StringBuffer() ;
    b.append("Web Unit Name     : ").append(name_).append("\n"); 
    b.append("Description       : ").append(getDescription()).append("\n") ;
    b.append("Action Description:").append(getActionDescription()).append("\n") ;
    b.append("Comment           :").append(getComment()).append("\n") ;
    if(conditions_ != null) {
      b.append("Conditions      :\n") ;
      for(int i = 0 ; i < conditions_.size(); i++) {
        Condition c = (Condition) conditions_.get(i) ;
        b.append("  ").append(c.getName()).append(": ").append(c.getDescription()).append("\n") ;
      }
    }

    if(validators_ != null) {
      b.append("Validators      :\n") ;
      for(int i = 0 ; i < validators_.size(); i++) {
        Validator v = (Validator) validators_.get(i) ;
        b.append(v.getName()).append(": ").
        append(v.getDescription()).append("<br/>") ;
      }
    }
    b.append("Extra Information :\n").append(getExtraInfo()).append("\n") ;
    return b.toString() ;
  }
}
