/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * This suite does
 *     1) go to Admin Template Page
 *     2) Create a new template (add template)
 *          fill label, view, dialog and select a NodeType
 *     3) Edit the new Template for that NodeType
 *     3.1) Add a new dialog template with a custom permission
 *     3.2) Add a new view template with a custom permission
 *     3.4) Edit that created dialog and change the permission
 *     3.5) Test dialog versioning
 *     3.6) Edit that created view and change the permission
 *     3.7) Test view versioning      
 *     3.8) Remove the created dialog
 *     3.9) Remove the created view
 *     3.10) Try to remove the default view and dialog (should print message)
 *     4) Back to template list
 *     5) Go to ECM File Explorer, click addDocument icon and check
 *          the view of created template'label in document type selectbox  
 *     6) Back to template list       
 *     7) Delete the new template
 *     8) Go back to ECM admin home 
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminTemplatesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Template button and test all the basic actions" ;
  static String NAME = "AdminTemplatesSuite" ;
  String COMMENT = null;
    
  public AdminTemplatesSuite() {    
    super(NAME, DESC) ;
    goToPage("/Home/ecm") ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", DESC).
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;          
    addWebUnit(
      new ClickURLLinkUnit("ManageTemplates", "Click on Manage Templates button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageTemplates").                                
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=addNodeType"))      
    ) ;
    //Create a new template (add template)    
    addWebUnit(
      new ClickURLLinkUnit("AddNewNodeType", "Click on add new Action button").
      setURL("&uicomponent=UINodeTypesList&op=addNodeType").                      
      addValidator(new ExpectTextValidator("Add a new template"))      
    ) ;     
    addWebUnit(
      new SubmitFormUnit("SaveEmptyLabelTemplate", "Test with empty label").
      setFormId("nodetypeForm").         
      setField("label","").
      setField("nodetype","nt:resource").
      setField("dialog","exo:test-dialog").
      setField("view","exo:test-view").
      setField("op","save").                  
      addValidator(new ExpectTextValidator("This template need a label for user select when create new node"))      
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveNodeTypeForm", "Fill all info and click on save button").
      setFormId("nodetypeForm").         
      setField("nodetype","nt:resource").
      setField("label","template label test").
      setCheckBox("isDocumentTemplate",true).
      setField("dialog","exo:test-dialog").
      setField("view","exo:test-view").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=addNodeType"))      
    ) ;        
    //Go to page 2 and edit the created template
    addWebUnit(
      new ClickURLLinkUnit("GotoPage2", "Click on link to page 2").
      setURL("&uicomponent=UIPageListIterator&page=2").                
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=viewNodeType&objectId=nt:resource"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("EditCreatedTemplates", "Edit created node type").
      setURL("&uicomponent=UINodeTypesList&op=viewNodeType&objectId=nt:resource").            
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=editTemplate&objectId=dialog1"))           
    ) ;
    //-----Edit Dialog Template
    addWebUnit(
      new ClickURLLinkUnit("EditDialogTemplates", "Edit dialog created node type").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog1").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("dialog1"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdatedDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog(updated)").      
      setField("viewPermission","*:/admin").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=addTemplate"))                                                    
    ) ;
    //-----Edit View Template
    addWebUnit(
      new ClickURLLinkUnit("EditViewTemplates", "Edit view created node type").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view1").          
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("view1"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view(updated)").      
      setField("viewPermission","*:/admin").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=addTemplate"))        
    ) ;
    //-----Add a new dialog template with a custom permission
    addWebUnit(
      new ClickURLLinkUnit("AddNewDialogTemplates", "Add new a dialog template").
      setURL("&uicomponent=DialogList&op=addTemplate").           
      addValidator(new ExpectFormIdValidator("templateForm"))           
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDialogPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2").      
      setField("template","dialog2").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=addTemplate"))      
    ) ;    
    //-----Add a new view template with a custom permission
    addWebUnit(
      new ClickURLLinkUnit("AddNewViewTemplates", "Add new a view template").
      setURL("&uicomponent=ViewList&op=addTemplate").                    
      addValidator(new ExpectFormIdValidator("templateForm"))           
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectViewPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2").   
      setField("template","view2").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=addTemplate"))      
    ) ;            
    //-----Edit that new dialog template and change the permission    
    addWebUnit(
      new ClickURLLinkUnit("EditDialog", "Edit and change dialog's permission").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("dialog2"))
    ) ;
    /*
    //---------Update with an invalidPermission    
    addWebUnit(
      new SubmitFormUnit("UpdateInvalidPermission", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2(updated)").   
      setField("viewPermission","#").
      setField("template","dialog2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectTextValidator("Invalid permission, permission style should be membership:/groupId."))           
    ) ;      
    addWebUnit(
      new SubmitFormUnit("UpdateInvalidGroupTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2(updated)").   
      setField("viewPermission","*:/user-test").
      setField("template","dialog2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectTextValidator("Membership type or Group name not found."))           
    ) ;
    */
    //---------Update with validPermission
    addWebUnit(
      new SubmitFormUnit("SearchDialogPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDialogPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2(updated)").   
      setField("template","dialog2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=addTemplate"))                                                       
    ) ;         
    //-----Test dialog versioning
    //----------Edit first time
    addWebUnit(
      new ClickURLLinkUnit("EditDialogVersion1", "Edit first time for test version").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("dialog2"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2-version1").   
      setField("template","dialog2-version1").     
      setCheckBox("enableVersioning",true).
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=editTemplate&objectId=dialog2"))        
    ) ;     
    //----------Edit second time    
    addWebUnit(
      new ClickURLLinkUnit("EditDialogVersion2", "Edit second time for test version").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog2").                         
      addValidator(new ExpectFormIdValidator("selectTemplateVersionForm")).     
      addValidator(new ExpectTextValidator("1")).        
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("exo:test-dialog2-version1")).
      addValidator(new ExpectTextValidator("dialog2-version1")).
      addValidator(new ExpectTextValidator("Enable Versioning")) 
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2-version2").   
      setField("template","dialog2-version2").          
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=editTemplate&objectId=dialog2"))                                                       
    ) ;     
    //----------Restore version
    addWebUnit(
      new ClickURLLinkUnit("EditForSelectVersion", "Edit for select test version").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog2").  
      addValidator(new ExpectTextValidator("2")).
      addValidator(new ExpectTextValidator("exo:test-dialog2-version2")).
      addValidator(new ExpectTextValidator("dialog2-version2")).                       
      addValidator(new ExpectLinkWithTextValidator("restore"))
    ) ;       
    addWebUnit(
      new SubmitFormUnit("Select version 1", "select version 1 from select box").
      setFormId("selectTemplateVersionForm").     
      setField("version","1").
      setField("op","changeVersion").
      addValidator(new ExpectTextValidator("1")).
      addValidator(new ExpectTextValidator("exo:test-dialog2-version1"))            
    ) ;  
    addWebUnit(
      new SubmitFormUnit("RestoreVersion", "Click on restore icon").
      setFormId("templateForm").      
      setField("op","restoreVersion").      
      addValidator(new ExpectTextValidator("The version 1 was restored successfully.")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=deleteTemplate&objectId=dialog2"))           
    ) ;     
    //-----Edit that new view template and change the permission    
    addWebUnit(
      new ClickURLLinkUnit("EditView", "Edit and change view's permission").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("view2"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectViewPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2(updated)").   
      setField("template","view2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=addTemplate"))       
    ) ;        
    //-----Remove the new dialog template
    addWebUnit(
      new ClickURLLinkUnit("RemoveCreatedDialog", "Remove created dialog").
      setURL("&uicomponent=DialogList&op=deleteTemplate&objectId=dialog2").      
      addValidator(new ExpectNoTextValidator("dialog2"))
    ) ;
    //-----Test view versioning
    //----------Edit first time
    addWebUnit(
      new ClickURLLinkUnit("EditViewVersion1", "Edit first time for test version").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("view2"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2-version1").   
      setField("template","view2-version1").     
      setCheckBox("enableVersioning",true).
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=editTemplate&objectId=view2"))        
    ) ;     
    //----------Edit second time    
    addWebUnit(
      new ClickURLLinkUnit("EditViewVersion2", "Edit second time for test version").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view2").                         
      addValidator(new ExpectFormIdValidator("selectTemplateVersionForm")).     
      addValidator(new ExpectTextValidator("1")).        
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("exo:test-view2-version1")).
      addValidator(new ExpectTextValidator("view2-version1")).
      addValidator(new ExpectTextValidator("Enable Versioning")) 
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2-version2").   
      setField("template","view2-version2").          
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=editTemplate&objectId=view2"))                                                       
    ) ;     
    //----------Restore version
    addWebUnit(
      new ClickURLLinkUnit("EditForRestoreVersion", "Edit for restore version").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view2").  
      addValidator(new ExpectTextValidator("2")).
      addValidator(new ExpectTextValidator("exo:test-view2-version2")).
      addValidator(new ExpectTextValidator("view2-version2")).                       
      addValidator(new ExpectLinkWithTextValidator("restore"))
    ) ;       
    addWebUnit(
      new SubmitFormUnit("Select version 1", "select version 1 from select box").
      setFormId("selectTemplateVersionForm").     
      setField("version","1").
      setField("op","changeVersion").
      addValidator(new ExpectTextValidator("1")).
      addValidator(new ExpectTextValidator("exo:test-view2-version1"))           
    ) ;  
    addWebUnit(
      new SubmitFormUnit("RestoreVersion", "Click on restore icon").
      setFormId("templateForm").      
      setField("op","restoreVersion").      
      addValidator(new ExpectTextValidator("The version 1 was restored successfully.")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=ViewList&op=deleteTemplate&objectId=view2"))           
    ) ;     
    //-----Remove the new view template
    addWebUnit(
      new ClickURLLinkUnit("RemoveCreatedView", "Remove created dialog").
      setURL("&uicomponent=ViewList&op=deleteTemplate&objectId=view2").      
      addValidator(new ExpectNoTextValidator("view2"))
    ) ;
    //-----Try to remove the default view and dialog (should print message)
    addWebUnit(
      new ClickURLLinkUnit("RemoveDefaultDialog", "Remove default dialog").
      setURL("&uicomponent=DialogList&op=deleteTemplate&objectId=dialog1").           
      addValidator(new ExpectTextValidator("This template can not be deleted, it is the default one"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RemoveDefaultView", "Remove default view").
      setURL("&uicomponent=ViewList&op=deleteTemplate&objectId=view1").         
      addValidator(new ExpectTextValidator("This template can not be deleted, it is the default one"))
    ) ;
    //Back to template list
    addWebUnit(
      new ClickURLLinkUnit("BackToTemplateList", "click on back link").
      setURL("&uicomponent=UIListsManager&op=back").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=nt:resource"))
    ) ;
    //Go to ECM File Explorer, refresh session, click addDocument icon 
    //and check the view of created template'label in document type selectbox 
    addWebUnit(
      new ClickURLLinkUnit("GotoECMFileExplorer", "click on ECM File Explorer link").
      setURL("portal:componentId=ECMPage&portal:action=changeTab&objectId=jcr")     
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDriver", "click on select driver icon").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Admin on production"))
    ) ;  
    addWebUnit(
      new ClickLinkWithText("ClickOnAdminOnProduction", "click on Admin on production link").
      setTextLink("Admin on production").         
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))                                                   
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on refresh session icon").
      setURL("&uicomponent=MainToolbar&op=refreshSession").               
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddDocument", "click on Add Document Icon").
      setURL("&uicomponent=UIToolbar&op=addDocument").        
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).     
      addValidator(new ExpectTextValidator("template label test"))
    ) ;
    //Back to template list      
    addWebUnit(
      new ClickURLLinkUnit("BackToTemplateList", "click on Admin the ECM").
      setURL("&portal:action=changeTab&objectId=ecm").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=nt:resource"))
    ) ;
    //Delete the new template  
    addWebUnit(
      new ClickURLLinkUnit("DeleteCreatedTemplate", "delete created template").
      setURL("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=nt:resource").
      addValidator(new ExpectTextValidator("The template for node type nt:resource has been removed."))
    ) ;
    //Go back to ECM admin home
    addWebUnit(
      new ClickURLLinkUnit("Back", "Click on back button").
      setURL("uicomponent=UINodeTypesList&op=back").     
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageScripts"))      
    ) ;    
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}