/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.temp;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * This suite does
 *     1) go to Admin Template Page
 *     2) Create a new template (add template)
 *     3) Fill view and dialog template and select any NodeType
 *     4) Edit the new Template for that NodeType
 *     5) Add a new dialog template with a custom permission
 *     6) Add a new view template with a custom permission
 *     7) Edit that new template and change the permission
 *     8) Remove the new dialog template
 *     9) Remove the new view template
 *     10) try to remove the default view and dialog (should print message)
 *     11) Go back to template list
 *     12) Delete the new template
 *     13) Go back to ECM admin home 
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminMultiTemplatesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Template button and test all the basic actions" ;
  static String NAME = "AdminTemplatesSuite" ;
  String COMMENT = null;
    
  public AdminMultiTemplatesSuite() {    
    super(NAME, DESC) ;
    //Go to Admin Template Page   
    goToPage("/Home/ecm") ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", DESC).
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;          
    addWebUnit(
      new ClickURLLinkUnit("ManageTemplates", "Click on Manage Templates button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageTemplates").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=addNodeType"))      
    ) ;
    for (i in 1..63){
      //Delete if existed template
      addWebUnit(
        new ClickURLLinkUnit("DeleteExistedTemplates", "DeleteExistedTemplates").
        setURL("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=exo:test-actionType"+i).
        addCondition(new HaveLinkWithURLCondition("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=exo:test-actionType"+i)).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=exo:test-actionType"+i,0))      
      ) ;
      //Create a new template (add template)    
      addWebUnit(
        new ClickURLLinkUnit("AddNewNodeType", "Click on add new Action button").
        setURL("&uicomponent=UINodeTypesList&op=addNodeType").                      
        addValidator(new ExpectTextValidator("Add a new template"))      
      ) ;    
      //Fill view and dialog template and select any NodeType, then Save
      addWebUnit(
        new SubmitFormUnit("SaveNodeTypeForm", "Fill all info and click on save button").
        setFormId("nodetypeForm").         
        setField("nodetype","exo:test-actionType"+i).
        setField("dialog","exo:test-dialog"+i).
        setField("view","exo:test-view"+i).
        setField("op","save").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=viewNodeType&objectId=exo:test-actionType"+i))      
      ) ;     
    }    
    //Go back to ECM admin home
    addWebUnit(
      new ClickURLLinkUnit("Back", "Click on back button").
      setURL("uicomponent=UINodeTypesList&op=back").     
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageScripts"))      
    ) ;    
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}