/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * Created by The eXo Platform SARL 
 * Author : Hoa Nguyen Thi
 *          hoa2501@yahoo.com
 * May 30, 2006
 */
public class UserMacViewSuite extends WebUnitSuite {
  static String DESC = "Go to ECM and test some basic actions with type view is mac-view" ;
  static String NAME = "UserMacViewSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
  
  public UserMacViewSuite() {    
    super(NAME, DESC) ;    
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;   
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectMacView", "select mac view").
      setFormId("UISelectViewForm").            
      setField("views","mac-view").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))      
    ) ;  
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/#{client.name}
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}")
    ) ;
    //-----Delete test-folder if does exist
    addWebUnit(
      new ClickURLLinkUnit("DeleteExistedFolder", "Delete if new-folder is existed").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/home/users/#{client.name}/new-folder").
      addCondition(new HaveLinkWithTextCondition("new-folder")).
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/new-folder has been removed."))
    ) ;        
    //-----Add folder directory
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddFolder", "click on Add Folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").                  
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    //----------Add nt:unstructured folder
    addWebUnit(
      new SubmitFormUnit("AddNewFolder", "Select type is nt:unstructured and click on Save button").
      setFormId("folderForm").   
      setField("folderType","nt:unstructured").
      setField("name","new-folder").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/new-folder"))      
    ) ;   
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }      
    //-----Go to test-folder
    addWebUnit(
      new ClickURLLinkUnit("Gotonew-folder", "click on new-folder").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/new-folder").         
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("new-folder"))
    ) ;  
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","test-doc").
      //setField("summary","content").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/new-folder/test-doc"))                                                   
    ) ;      
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    COMMENT = "You can't any field in uiResourceForm";
    addWebUnit(
      new TODO("can'tSetField", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    /*
    //Test action edit of test-doc
    addWebUnit(
        new ClickURLLinkUnit("ClickEditTest-doc", "click on edit icon of test-doc").
        setURL("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/new-folder/test-doc").        
        addValidator(new ExpectFormIdValidator("uiResourceForm"))      
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitEditDocument", "click on save button").
        setFormId("uiResourceForm").   
        setField("name","test-doc").
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/new-folder/test-doc"))                                                   
    ) ;
    */
    //Test action copy for test-doc
    addWebUnit(
        new ClickURLLinkUnit("ClickCopyTest-doc", "click on copy icon of test-doc").
        setURL("uicomponent=UIContentManager&op=copy&objectId=/home/users/#{client.name}/new-folder/test-doc").        
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/new-folder/test-doc has been copied."))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickParseTest-doc", "click on parse icon of test-doc").
        setURL("uicomponent=UIToolbar&op=paste").        
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/new-folder/test-doc has been pasted at the location /home/users/#{client.name}/new-folder/test-doc."))      
    ) ;
    //Test action cut
    addWebUnit(
        new ClickURLLinkUnit("ClickCutTest-doc", "click on cut icon of test-doc").
        setURL("uicomponent=UIContentManager&op=cut&objectId=/home/users/#{client.name}/new-folder/test-doc").        
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/new-folder/test-doc has been cut."))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("Goto current user folder", "click on current user link").
        setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=paste"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickParseTest-doc", "click on parse icon of test-doc").
        setURL("uicomponent=UIToolbar&op=paste").        
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/new-folder/test-doc has been pasted at the location /home/users/#{client.name}/test-doc."))      
    ) ;
    
    //Test action delete
    addWebUnit(
        new ClickURLLinkUnit("ClickDeleteTest-doc", "click on delete icon of test-doc").
        setURL("uicomponent=UIContentManager&op=removeNode&objectId=/home/users/#{client.name}/test-doc").        
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-doc has been removed."))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Return to Drive selection
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ; 
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}