/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserManageCategoriesSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on Info tab, click on Manage Categories button and to do somethings" ;
  static String NAME = "UserManageCategoriesSuite" ;
  String COMMENT  = "You must run AdminManageTaxonomySuite, AdminViewSuite, AdminDriveSuite before run this suite";
  
  public UserManageCategoriesSuite() {    
    super(NAME, DESC) ;       
    //Switch to exoadmin account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );
    String[] roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ; 
    //Create test-folder, goto test-folder and create a new document
    goToPage("/Home/ecm") ;  
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive"). 
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ; 
    addWebUnit(
      new ClickLinkWithText("SelectAdminOnProduction", "click on Admin On Production link").
      setTextLink("Admin on production").                  
      addValidator(new ExpectTextValidator("jcr:system"))     
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectView", "select admin-view").
      setFormId("UISelectViewForm").   
      setField("views","admin-view").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Info"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnActionTab", "click on action tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Action").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))       
    ) ;         
    addWebUnit(
       new ClickURLLinkUnit("ClickOnHome", "click on home link").
       setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
     ) ; 
     addWebUnit(
       new ClickURLLinkUnit("ClickOnUsers", "click on users link").
       setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/exoadmin"))      
     ) ;
     addWebUnit(
       new ClickURLLinkUnit("ClickOnExoadmin", "click on exoadmin user link").
       setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/exoadmin")
     ) ;
     //-----Delete test-folder if does exist    
     addWebUnit(
       new ClickURLLinkUnit("DeleteExistedFolder", "Delete if test-folder is existed").
       setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/home/users/exoadmin/test-folder").
       addCondition(new HaveLinkWithTextCondition("test-folder")).
       addValidator(new ExpectTextValidator("The node /home/users/exoadmin/test-folder has been removed."))
     ) ;            
     //-----Add folder directory
     addWebUnit(
       new ClickURLLinkUnit("ClickOnAddFolder", "click on Add Folder button").
       setURL("&uicomponent=UIToolbar&op=addFolder").                  
       addValidator(new ExpectFormIdValidator("folderForm"))      
     ) ;        
     //----------Add nt:unstructured folder
     addWebUnit(
       new SubmitFormUnit("Add Unstructured Folder", "Select type is nt:unstructured and click on Save button").
       setFormId("folderForm").   
       setField("folderType","nt:unstructured").
       setField("name","test-folder").
       setField("op","save").
       addValidator(new ExpectTextValidator("test-folder"))      
     ) ;
     if (UserECMFileExplorerSuite.isEnableJCR) {
       //Click on preferences icon
       addWebUnit(
         new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
         setURL("uicomponent=MainToolbar&op=preferences").                  
         addValidator(new ExpectFormIdValidator("preferencesForm"))      
       ) ; 
       addWebUnit(
         new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
         setFormId("preferencesForm").
         setCheckBox("checkbox",true).
         setField("op","save").
         addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
       ) ;
       //Save session
       addWebUnit(
         new ClickURLLinkUnit("SavedSession", "click on save Session button").
         setURL("&uicomponent=MainToolbar&op=saveSession").        
         addValidator(new ExpectTextValidator("All the changes have been saved"))      
       ) ;
     }         
     //-----Go to test-folder
     addWebUnit(
       new ClickURLLinkUnit("GotoTest-folder", "click on test-folder").
       setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/exoadmin/test-folder")      
     ) ;      
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectNoTextValidator("Image to upload")).
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","test-doc").
      //setField("summary","content").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=/home/users/exoadmin/test-folder/test-doc"))
    ) ;                
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }  
    //Goto created nt:node, then add it to a Category
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-doc", "click on test-doc").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/exoadmin/test-folder/test-doc").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=/home/users/exoadmin/test-folder/test-doc",0))                
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageCategories")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=exportNode")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=importNode"))
    ) ;   
    //-----Manage Category
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageCategory", "Click on Manage Category icon").
      setURL("uicomponent=UIToolbar&op=manageCategories").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoriesList&op=add"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickAddCategory", "Click on Add Category icon").
      setURL("&uicomponent=UICategoriesList&op=add").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClicknewCategory", "Click on newCategory").
      setURL("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectchildCategory", "Select childCategory").
      setURL("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory").                  
      addValidator(new ExpectTextValidator("/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeletechildCategory", "Click on delete icon of childCategory").
      setURL("uicomponent=UICategoriesList&op=delete&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory").                  
      addValidator(new ExpectNoTextValidator("/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoriesList&op=add"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickAddCategory", "Click on Add Category icon").
      setURL("&uicomponent=UICategoriesList&op=add").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClicknewCategory", "Click on newCategory").
      setURL("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectchildCategory", "Select childCategory").
      setURL("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory").                  
      addValidator(new ExpectTextValidator("/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoriesList&op=PhaseId[1].cancel"))
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }         
    //Browser category node
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "Go to ECM home").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system"))
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnJCR:Systen", "Go to jcr:system").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnExo:ecm", "Go to jcr:system/exo:ecm").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnExo:taxonomies", "Go to jcr:system/exo:ecm/exo:taxonomies").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/calendar"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnPreferencesIcon", "click on prefereces icon on MainToolBar").
      setURL("&uicomponent=MainToolbar&op=preferences").
      addValidator(new ExpectFormIdValidator("preferencesForm"))
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveForm", "select checkbox 'Show non document type', 'Show preferenced documents', then click save").
      setFormId("preferencesForm").
      setCheckBox("showNonDocumentType",true).
      setCheckBox("showPreferences",true).
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory"))
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnNewCategory", "click on NewCategory link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnChildCategory", "click on ChildCategory link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=viewNode&objectId=production:/home/users/exoadmin/test-folder/test-doc")).                                                   
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=production:/home/users/exoadmin/test-folder/test-doc"))                                                    
    ) ; 
    /*//-----Test all action on test-doc
    addWebUnit(
      new ClickURLLinkUnit("SelectActionTab", "click on action tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=paste"))      
    ) ;
    COMMENT = "You can't any field in uiResourceForm";
    addWebUnit(
      new TODO("can'tSetField", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;    
    //-----------lock and unlock
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLock", "click on lock icon").
      setURL("&uicomponent=UIContentManager&op=lock&objectId=production;/home/users/exoadmin/test-folder/test-doc").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=unlock&objectId=production;/home/users/exoadmin/test-folder/test-doc"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUnlock", "click on unlock icon").
      setURL("&uicomponent=UIContentManager&op=unlock&objectId=production;/home/users/exoadmin/test-folder/test-doc").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=lock&objectId=production;/home/users/exoadmin/test-folder/test-doc"))      
    ) ;
    //----------copy and paste
    addWebUnit(
      new ClickURLLinkUnit("ClickCopyTest-doc", "click on copy icon of test-doc").
      setURL("&uicomponent=UIContentManager&op=copy&objectId=production;/home/users/exoadmin/test-folder/test-doc").        
      addValidator(new ExpectTextValidator("The node production;/home/users/exoadmin/test-folder/test-doc has been copied."))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickPasteTest-doc", "click on paste icon of test-doc").
      setURL("uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("The node /home/users/exoadmin/test-folder/test-doc has been pasted at the location /jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory/test-doc."))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeletedPastedTest-doc", "click on delete icon of pasted test-doc").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory/test-doc").         
      addValidator(new ExpectTextValidator("The node /jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory/test-doc has been removed."))      
    ) ;
    //----------rename
    addWebUnit(
      new ClickURLLinkUnit("ClickRename", "click on rename icon").
      setURL("&uicomponent=UIContentManager&op=rename&objectId=production;/home/users/exoadmin/test-folder/test-doc").              
      addValidator(new ExpectFormIdValidator("renameForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveRename", "click on save button").
      setFormId("renameForm").   
      setField("oldName","test-doc").
      setField("newName","renamed-test-doc").      
      setField("op","save").                      
      addValidator(new ExpectTextValidator("The node \"test-doc\" was renamed to \"renamed-test-doc\" successfully."))      
    ) ;
    //----------cut and paste
    addWebUnit(
      new ClickURLLinkUnit("ClickCutRenamed-Test-doc", "click on cut icon of renamed-test-doc").
      setURL("&uicomponent=UIContentManager&op=cut&objectId=production;/home/users/exoadmin/test-folder/renamed-test-doc").                
      addValidator(new ExpectTextValidator("The node production;/home/users/exoadmin/test-folder/renamed-test-doc has been cut."))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnNewCategory", "click on NewCategory link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory").            
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=paste"))      
    ) ;           
    addWebUnit(
      new ClickURLLinkUnit("ClickParseRenamed-Test-doc", "click on parse icon of renamed-test-doc").
      setURL("uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("The node /home/users/exoadmin/test-folder/renamed-test-doc has been pasted at the location /jcr:system/exo:ecm/exo:taxonomies/newCategory/renamed-test-doc."))                                            
    ) ;         
    //----------delete
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteRenamed-Test-doc", "click on delete icon of renamed-test-doc").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/renamed-test-doc").              
      addValidator(new ExpectTextValidator("The node /jcr:system/exo:ecm/exo:taxonomies/newCategory/renamed-test-doc has been removed."))                                                     
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }         */
    //Select Driver
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ; 
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );
  }                 
  public java.util.List getRequiredSuites() {     
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}